#==============================================================================
# package      DE_EPAGES::Core::API::BaseInstaller
# state        public
# description  base installer
#==============================================================================
package DE_EPAGES::Core::API::BaseInstaller;
use base Exporter;

use strict;
our @EXPORT_OK = qw(
    ResetInstallerCache
);

use Sort::Versions qw(versioncmp);
use DE_EPAGES::Core::API::CartridgeConfig qw (CartridgeConfig);
use DE_EPAGES::Core::API::Error qw (Error ExistsError GetError );
use DE_EPAGES::Core::API::Log qw (GetLog LogDebug);
use DE_EPAGES::Core::API::PerlTools qw (IndexOf UsePackage );
use DE_EPAGES::Core::API::Warning qw ( Warning );

sub ErrorDefinition {
    return {
        'CARTRIDGE_NOT_EXISTS' => 'cartridge #Package (module API/Cartridge) doesn\'t exists',
        'STORENAME_MISSING' => 'StoreName isn\'t defined.',
        'STORETYPE_MISSING' => 'StoreType isn\'t defined.',
        'ANTIQUE_PATCHES_MISSING' => "Can't patch from #Version without #File.\nFirst run:\n   $ENV{PERL} $ENV{EPAGES_CARTRIDGES}/DE_EPAGES/Installer/Scripts/addAntiquePatches.pl\n",
    }
}

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer = DE_EPAGES::Core::API::BaseInstaller->new(%Options);
#----------------------------------------------------------------------------------------
# description  Creates a new installer object.
#----------------------------------------------------------------------------------------
# input        %Options | option
#               <ul>
#                   <li>CartridgeDirectory - cartridge base directory (DE_EPAGES/Core) - string
#                   <li>StoreName - store name for in/uninstall (optional default undef) - string
#                   <li>StoreType - store type for in/uninstall (optional default undef) - string
#                   <li>IsRecusive - (un)install dependent and required cartridges (optional default 1)- boolean
#                   <li>Force - uninstall are forced (not stops) (optional default 0)- boolean
#                   <li>NoScheduler - install without scheduler files (optional default 0)- boolean
#                   <li>NoSharedConfig - install without config files (optional default 0)- boolean
#               </ul>
#               | hash
# return       $Installer | cartridge installer | object
#========================================================================================
sub new {
    my ($class, %options) = @_;
    $options{'IsRecursive'} //= 0;
    $options{'Force'}       //= 0;
    $options{'Version'}     //= 0;
    $options{'Reasons'}     //= [];
    $options{'NoScheduler'} //= 0;
    $options{'NoSharedConfig'} //= 0;

    foreach my $Option (qw (CartridgeDirectory)) {
        Error('OPTION_MISSING', {'Option'=>$Option}) unless exists $options{$Option};
    };
    return bless \%options, $class;
}

#========================================================================================
# function     startInstall
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer->startInstall;
#----------------------------------------------------------------------------------------
# description  Prepares the cartridge installation.
#========================================================================================
sub startInstall ($) {
    my $self = shift;
    GetLog->info(sprintf('start install %s on store %s.', $self->cartridgePackage, $self->storeName));
}

#========================================================================================
# function     install
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer->install;
#----------------------------------------------------------------------------------------
# description  Installs the files and database objects of a cartridge to the current
#               store/database.
#========================================================================================
sub install ($) {
    my $self = shift;
}

#========================================================================================
# function     test
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->test;
#----------------------------------------------------------------------------------------
# description  base method for overwriting in subclasses
#========================================================================================
sub test {
    my $self = shift;
}

#========================================================================================
# function     finishInstall
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer->finishInstall;
#----------------------------------------------------------------------------------------
# description  Finalizes the cartridge installation. Writes a info 'finish install 'at log.
#               Sets the cartridge version.
#========================================================================================
sub finishInstall ($) {
    my $self = shift;
    $self->setVersion($self->version);
    GetLog->info(sprintf('finish install %s on store %s.', $self->cartridgePackage, $self->storeName));
}

#========================================================================================
# function     startUninstall
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer->startUninstall;
#----------------------------------------------------------------------------------------
# description  Prepares the cartridge uninstallation.
#========================================================================================
sub startUninstall ($) {
    my $self = shift;
    GetLog->info(sprintf('start uninstall %s on store %s.', $self->cartridgePackage, $self->storeName));
}

#========================================================================================
# function     uninstall
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer->uninstall;
#----------------------------------------------------------------------------------------
# description  Removes cartridge files and database objects from the current
#               store/database.
#========================================================================================
sub uninstall ($$) {
    my $self = shift;
}

#========================================================================================
# function     finishUninstall
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer->finishUninstall;
#----------------------------------------------------------------------------------------
# description  Finalizes the cartridge uninstallation. Writes 'finish uninstall' to log.
#               Delete Cartridge entry from Cartridges.d/$Store.conf
#========================================================================================
sub finishUninstall ($) {
    my $self = shift;

    # remove cartridge from Cartridges.d/$Store.conf
    my $IniConfig = CartridgeConfig( $self->storeName );
    $IniConfig->delete('Cartridges', $self->cartridgePackage);
    $IniConfig->rewrite;

    GetLog->info(sprintf('finish uninstall %s on store %s.', $self->cartridgePackage, $self->storeName));
}

#========================================================================================
# function     startPatch
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer->startPatch;
#----------------------------------------------------------------------------------------
# description  Prepares the cartridge patch. Writes a info 'start patch' at log.
#========================================================================================
sub startPatch ($) {
    my $self = shift;
    GetLog->info(sprintf('start patch %s on store %s.', $self->cartridgePackage, $self->storeName));
}

#========================================================================================
# function     checkAntiquePatches
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer->checkAntiquePatches;
#----------------------------------------------------------------------------------------
# description  Adds Cartridges/DE_EPAGES/{CARTRIDGE}/Patches/Patch_{VERSION}/
#               if installed VERSION of CARTRIDGE is <= 7.4.0.
#========================================================================================
sub checkAntiquePatches {
    my $self = shift;

    my $InstalledVersion = $self->installedVersion;
    my $AntiqueVersion = '7.4.0';

    # return if cartridgePackage is not DE_EPAGES::
    return if $self->cartridgePackage !~ /^DE_EPAGES::/;

    # return if $InstalledVersion is > $AntiqueVersion
    return if versioncmp( $InstalledVersion, $AntiqueVersion ) > 0;

    my $File = "$ENV{EPAGES_CARTRIDGES}/DE_EPAGES/.antique-patches";
    Error('ANTIQUE_PATCHES_MISSING', {'Version'=>$InstalledVersion, 'File'=>$File})
        unless -f $File;

    return;
}

#========================================================================================
# function     doPatch
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer->doPatch;
# syntax       $Installer->doPatch('6.12.0');
#----------------------------------------------------------------------------------------
# description  Patches cartridge in active store. This function
#               calls methods named "patch_$InstalledVersion". InstalledVersion is actual version
#               of cartridge '.' are replaced with '_' (version on 1.0.1.0 will call
#               patch_1_0_1_0). At the end of each patch call the called function returns
#               the new version (eg: 1.1.0.0).
#----------------------------------------------------------------------------------------
# input        $Version | goal version number, current number is chosen if undefined | string
#========================================================================================
sub doPatch ($) {
    my $self = shift;
    my ($Version) = @_;

    #if no version is given, the version from Installer is chosen
    $Version = $self->version unless defined $Version;

    my $InstalledVersion = $self->installedVersion;

    #get available patches
    my $aPatchVersions = $self->patches;

    #check if given version exists in cartridge versions
    if (not (grep {$_ eq $Version} @$aPatchVersions)) {
        #log message if version is not current version
        GetLog->info(
            'warning cartridge version not found while doPatch Package => ' .
            $self->cartridgePackage .
            ' OldVersion=> ' .
            $InstalledVersion .
            ' NewVersion=> ' .
            $Version
        ) if ($Version ne $self->version);
        GetLog->info(
            "\npatch cartridge to current version => " .
            $self->version
        );

        #set patch version to current version
        $Version = $self->version;
    }

    while ($InstalledVersion ne $Version) {
        my $i = IndexOf($InstalledVersion, @$aPatchVersions);
        Error('NO_PATCH_FOUND', { 'Package'=>$self->cartridgePackage, 'OldVersion'=>$InstalledVersion, 'NewVersion'=>$Version})
            if $i == -1; #InstalledVersion is not included at available patches

        my $PatchSubDir = "Patch_$InstalledVersion";
        $PatchSubDir =~ s/\./_/g;

        my $FunctionName = "patch_$InstalledVersion";
        $FunctionName =~ s/\./_/g;
        if ($self->can($FunctionName) ) {
            GetLog->info(sprintf('patch %s function %s on store %s.', $self->cartridgePackage,$FunctionName, $self->storeName));
            $InstalledVersion = $self->$FunctionName;
        } else {
            $InstalledVersion = $self->importPatchDir( $PatchSubDir, 'Patch' );
            unless( defined $InstalledVersion ) {
                $InstalledVersion = (scalar(@$aPatchVersions) <= $i+1) ? $Version : $aPatchVersions->[$i+1];
            }
        }

        # patch languages
        $self->patchLanguages( $PatchSubDir, 'Patch' );

        #update new version
        $self->setVersion($InstalledVersion);
    }
    return;
}

# dummy method, implemented in DE_EPAGES::XML::API::XMLInstaller
sub importPatchDir {
    my ($PatchSubDir, $PatchSection) = @_;
    return;
}

# dummy method, implemented in DE_EPAGES::Object::API::ObjectInstaller
sub patchLanguages {
    my $self=shift;
    my ($PatchSubDir, $PatchSection) = @_;
    return;
}

#========================================================================================
# function     finishPatch
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer->finishPatch;
#----------------------------------------------------------------------------------------
# description  Finalizes the cartridge patch. Writes a info 'finish patch 'at log.
#========================================================================================
sub finishPatch ($) {
    my $self = shift;
    GetLog->info(sprintf('finish patch %s on store %s.', $self->cartridgePackage, $self->storeName));
}

#========================================================================================
# function     patch
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer->patch;
# syntax       $Installer->patch('6.12.0');
#----------------------------------------------------------------------------------------
# description  Patches the files and database objects of a cartridge to the current
#               store/database if the cartridge is installed.
#----------------------------------------------------------------------------------------
# input        $Version | goal version number, current number is chosen if undefined | string
#========================================================================================
sub patch ($) {
    my $self = shift;
    my($Version) = @_;

    if ($self->isInstalled) {
        $self->startPatch();
        $self->checkAntiquePatches();
        $self->doPatch($Version);
        $self->finishPatch();
    } else {
        GetLog->info(sprintf('No patch %s on store %s, because cartridge is not installed.', $self->cartridgePackage, $self->storeName));
    }
    return;
}

#========================================================================================
# function     patchCartridgeDir
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer->patchCartridgeDir;
# syntax       $Installer->patchCartridgeDir('6.13.0');
#----------------------------------------------------------------------------------------
# description  Patches cartridge directory. This function
#               calls methods named "patchCartridgeDir_$CartridgeDirVersion". CartridgeDirVersion is first
#               patch version of cartridge '.' are replaced with '_' (version on 1.0.1.0 will call
#               patchCartridgeDir_1_0_1_0). At the end of each patch call the called function returns
#               the new version (eg: 1.1.0.0).
#               note: if version is given, the function is only called for this specific version,
#               not for all versions between current version and goal version
# input        $Version | version number, all patchCartridgeDir functions are executed if undefined | string
#========================================================================================
sub patchCartridgeDir {
    my $self = shift;
    my ($PatchVersion) = @_;

    GetLog->info(sprintf('patch cartridge dir %s.', $self->cartridgePackage));

    foreach my $Version ( @{ $self->patches } ) {

        next if ((defined $PatchVersion) and ($PatchVersion ne $Version));

        my $FunctionName = "patchCartridgeDir_$Version";
        $FunctionName =~ s/\./_/g;

        my $PatchSubDir = "Patch_$Version";
        $PatchSubDir =~ s/\./_/g;

        # call patchCartridgeDir_x_x if exists, otherwise import patch.xml if exists
        if ($self->can($FunctionName) ) {
            $self->$FunctionName;
        }
        else {
            $self->importPatchDir( $PatchSubDir, 'PatchCartridgeDir' );
        }

        # patch languages
        $self->patchLanguages( $PatchSubDir, 'PatchCartridgeDir' );
    }
    GetLog->info(sprintf("cartridge directory of cartridge %s is patched to version %s\n", $self->cartridgePackage, $self->version));
}

#========================================================================================
# function     patchStoreTypeDir
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer->patchStoreTypeDir;
#----------------------------------------------------------------------------------------
# description  Patches cartridge directory. This function
#               calls methods named "patchStoreTypeDir_$CartridgeDirVersion". CartridgeDirVersion is first
#               patch version of cartridge '.' are replaced with '_' (version on 1.0.1.0 will call
#               patchStoreTypeDir_1_0_1_0). At the end of each patch call the called function returns
#               the new version (eg: 1.1.0.0).
#========================================================================================
sub patchStoreTypeDir {
    my $self = shift;
    GetLog->info(sprintf('patch store type dir %s.', $self->cartridgePackage));

    foreach my $Version ( @{ $self->patches } ) {
        my $FunctionName = "patchStorTypeDir_$Version";
        $FunctionName =~ s/\./_/g;

        my $PatchSubDir = "Patch_$Version";
        $PatchSubDir =~ s/\./_/g;

        # call patchStoreTypeDir_x_x if exists, otherwise import patch.xml if exists
        if ($self->can($FunctionName) ) {
            $self->$FunctionName;
        }
        else {
            $self->importPatchDir( $PatchSubDir, 'PatchStoreTypeDir' );
        }

        # patch languages
        $self->patchLanguages( $PatchSubDir, 'PatchStoreTypeDir' );
    }
    GetLog->info(sprintf("store type dir of cartridge %s is patched to version %s\n", $self->cartridgePackage, $self->version));
}

#========================================================================================
# function     setVersion
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer->setVersion( $Version );
# example      $Installer->setVersion('1.0.2');
#----------------------------------------------------------------------------------------
# description  Updates the cartridge version in the current database.
#----------------------------------------------------------------------------------------
# input        $Version | new version string, e.g. '1.0.2' | string
#========================================================================================
sub setVersion {
    my $self = shift;
    my ($NewVersion) = @_;

    GetLog->info(sprintf('Set cartridge (%s) version \'%s\' on store %s.', $self->cartridgePackage, $NewVersion, $self->storeName));

    #set version to Cartridges.conf
    my $IniConfig = CartridgeConfig( $self->storeName );
    $IniConfig->set('Cartridges', $self->cartridgePackage, $NewVersion );
    $IniConfig->rewrite;
    return;
}

#========================================================================================
# function     installedVersion
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $Version = $Installer->installedVersion();
#----------------------------------------------------------------------------------------
# description  Returns the installed version of the cartridge in the current database.
#----------------------------------------------------------------------------------------
# return       $Version | version string, e.g. '1.0' | string
#========================================================================================
sub installedVersion {
    my $self = shift;

    return CartridgeConfig( $self->storeName )->get( 'Cartridges', $self->cartridgePackage );
}

#========================================================================================
# function     cartridgePackage
# state        public
#----------------------------------------------------------------------------------------
# syntax       $CartridgePackage = $Installer->cartridgePackage;
#----------------------------------------------------------------------------------------
# description  Gets cartridge package name.
#----------------------------------------------------------------------------------------
# return       $CartridgePackage | cartridge package name (DE_EPAGES::Core) | string
#========================================================================================
sub cartridgePackage {
    my $self = shift;
    my $Package = $self->{'CartridgeDirectory'};
    $Package =~ s/\//::/g;
    return $Package;
}

#========================================================================================
# function     cartridgeDirectory
# state        public
#----------------------------------------------------------------------------------------
# syntax       $CartridgeDirectory = $Installer->cartridgeDirectory;
#----------------------------------------------------------------------------------------
# description  Gets cartridge directory name.
#----------------------------------------------------------------------------------------
# return       $CartridgeDirectory | cartridge directory name (DE_EPAGES/Core) | string
#========================================================================================
sub cartridgeDirectory {
    my $self = shift;
    return $self->{'CartridgeDirectory'};
}

#========================================================================================
# function     patchesDirectory
# state        public
#----------------------------------------------------------------------------------------
# syntax       $CartridgePatchDirectory = $Installer->patchesDirectory;
#----------------------------------------------------------------------------------------
# description  Gets cartridge patch directory name.
#----------------------------------------------------------------------------------------
# return       $CartridgePatchDirectory | cartridge patch directory name (DE_EPAGES/Core/Patches) | string
#========================================================================================
sub patchesDirectory {
    my $self = shift;
    return join '/', $self->cartridgeDirectory , 'Patches';
}

#========================================================================================
# function     testDirectory
# state        public
#----------------------------------------------------------------------------------------
# syntax       $TestDirectory = $Installer->testDirectory;
#----------------------------------------------------------------------------------------
# description  Gets test directory path of a cartridge.
#----------------------------------------------------------------------------------------
# return       $TestDirectory | test directory path (e.g. C:\ePages\Cartridges\DE_EPAGES\Core\t) | string
#========================================================================================
sub testDirectory {
    my $self = shift;
    return join('/', $ENV{'EPAGES_CARTRIDGES'}, $self->cartridgeDirectory, 't');
}

#========================================================================================
# function     storeName
# state        public
#----------------------------------------------------------------------------------------
# syntax       $StoreName = $Installer->storeName;
#----------------------------------------------------------------------------------------
# description  The cartridge will be installed at this store.
#----------------------------------------------------------------------------------------
# return       $StoreName | store name | string
#========================================================================================
sub storeName {
    my $self = shift;
    my $StoreName = $self->{'StoreName'};
    Error('STORENAME_MISSING' ) unless defined $StoreName;
    return $StoreName;
}

#========================================================================================
# function     storeType
# state        public
#----------------------------------------------------------------------------------------
# syntax       $StoreType = $Installer->storeType;
#----------------------------------------------------------------------------------------
# description  The cartridge will be installed for this store type.
#----------------------------------------------------------------------------------------
# return       $StoreType | store type | string
#========================================================================================
sub storeType {
    my $self = shift;
    my $StoreType = $self->{'StoreType'};
    Error('STORETYPE_MISSING' ) unless defined $StoreType;
    return $StoreType;
}

#========================================================================================
# function     storeTypeVersion
# state        public
#----------------------------------------------------------------------------------------
# syntax       $StoreTypeVersion = $Installer->storeTypeVersion;
#----------------------------------------------------------------------------------------
# description  The cartridge will be installed for this store type version.
#----------------------------------------------------------------------------------------
# return       $StoreTypeVersion | store type version | string
#========================================================================================
sub storeTypeVersion {
    my $self = shift;
    my $StoreTypeVersion = $self->{'StoreTypeVersion'};
    Error('STORETYPEVERSION_MISSING' ) unless defined $StoreTypeVersion;
    return $StoreTypeVersion;
}

#========================================================================================
# function     storeTypeDir
# state        public
#----------------------------------------------------------------------------------------
# syntax       $StoreTypeDir = $Installer->storeTypeDir;
#----------------------------------------------------------------------------------------
# description  The cartridge will be installed into this store type directory.
#----------------------------------------------------------------------------------------
# return       $StoreTypeDir | store type directory | string
#========================================================================================
sub storeTypeDir {
    my $self = shift;

    return join('/', 'StoreTypes', $self->storeTypeVersion, $self->storeType);
}

#========================================================================================
# function     isInstalled
# state        public
#----------------------------------------------------------------------------------------
# syntax       $IsInstalled = $Installer->isInstalled;
#----------------------------------------------------------------------------------------
# description  Returns true if the cartridge is installed in the current database.
#----------------------------------------------------------------------------------------
# return       $IsInstalled | true if the cartridge is installed | boolean
#========================================================================================
sub isInstalled {
    my $self = shift;

    return defined $self->installedVersion ? 1 : 0;
}

#========================================================================================
# function     force
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Force = $Installer->force;
# example      die "Installation failed" unless $Installer->force;
#----------------------------------------------------------------------------------------
# description  Forces uninstallation.
#----------------------------------------------------------------------------------------
# return       $Force | ignore errors during install or uninstall | boolean
#========================================================================================
sub force {
    my $self = shift;
    return $self->{'Force'};
}

#========================================================================================
# function     call
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer->call( $Code );
# example      $Installer->call( sub { ImportObject( "Objects.xml" ) } );
#----------------------------------------------------------------------------------------
# description  Executes the code reference. Depending on the <function force> flag,
#               errors are degraded to warnings.
#----------------------------------------------------------------------------------------
# input        $Code | code reference | code reference
#========================================================================================
sub call {
    my $self = shift;
    my ($Code) = @_;

    eval { &$Code };
    if( ExistsError() ) {
        my $Error = GetError();
        if( $self->force ) {
            Warning( $Error );
        }
        else {
            Error( $Error );
        }
    }
}

#========================================================================================
# function     isRecursive
# state        public
#----------------------------------------------------------------------------------------
# syntax       $IsRecursive = $Installer->isRecursive;
#----------------------------------------------------------------------------------------
# description  Returns true if required cartridges are to be installed automatically or
#               dependent cartridges are uninstalled automatically.
#----------------------------------------------------------------------------------------
# return       $IsRecursive | recursive flag | boolean
#========================================================================================
sub isRecursive {
    my $self = shift;
    return $self->{'IsRecursive'};
}

#========================================================================================
# function     version
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Version = $Installer->version;
#----------------------------------------------------------------------------------------
# description  Returns the current cartridge version.
#----------------------------------------------------------------------------------------
# return       $Version | current cartridge varsion (source code version), for example
#                   '1.0' | string
#========================================================================================
sub version {
    my $self = shift;
    return $self->{'Version'};
}

#========================================================================================
# function     patches
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aPatchVersions = $Installer->patches;
#----------------------------------------------------------------------------------------
# description  Returns older versions which can be patched by this cartridge.
#----------------------------------------------------------------------------------------
# return       $aPatchVersions | list upgradable versions of this cartridge, for exapme:
#                   [ '1.0', '1.1' ] | ref.array.string
#========================================================================================
sub patches {
    my $self = shift;
    return defined $self->{'Patches'} ? $self->{'Patches'} : [];
}

#========================================================================================
# function     id
# state        public
#----------------------------------------------------------------------------------------
# syntax       $CartridgeID = $Installer->id;
#----------------------------------------------------------------------------------------
# description  Returns the cartridge id from the current database or undef if the
#               cartridge is not installed in the current database.
#----------------------------------------------------------------------------------------
# return       $CartridgeID | cartridge id | int
#========================================================================================
sub id {
    my $self = shift;
    return undef;
}

#========================================================================================
# function     addReason
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer->addReason($hReason);
#----------------------------------------------------------------------------------------
# description  Adds reason to collect problems on install/uninstall/patch/...
#----------------------------------------------------------------------------------------
# input        $hReason | keys can include keys:
#               <ul>
#                   <li>Reason - key - string
#                   <li>Package - patch cartridge package  - string
#                   <li>Version - patch cartridge version - string
#               </ul> | ref.hash
#========================================================================================
sub addReason ($$) {
    my $self = shift;
    my ($Reason) = @_;
    push @{$self->{'Reasons'}}, $Reason;
}

#========================================================================================
# function     getReasons
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ahReasons = $Installer->getReasons();
#----------------------------------------------------------------------------------------
# description  Returns collected reasons and deletes reasons.
#----------------------------------------------------------------------------------------
# return       $ahReasons | keys can include keys:
#               <ul>
#                   <li>Reason - key - string
#                   <li>Package - patch cartridge package  - string
#                   <li>Version - patch cartridge version - string
#               </ul> | ref.array.hash
#========================================================================================
sub getReasons ($) {
    my $self = shift;
    my @Reasons = @{$self->{'Reasons'}};
    $self->{'Reasons'} = [];
    return \@Reasons;
}

#========================================================================================
# function     hasReasons
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Has = $Installer->hasReasons();
#----------------------------------------------------------------------------------------
# description  Returns if reasons exists.
#----------------------------------------------------------------------------------------
# return       $Has | has reasons | boolean
#========================================================================================
sub hasReasons ($) {
    my $self = shift;
    return scalar @{$self->{'Reasons'}} ? 1 : 0;
}

#========================================================================================
# function     requiredCartridges
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aRequiredCartridges = $Installer->requiredCartridges;
#----------------------------------------------------------------------------------------
# description  Returns a list of all required cartridges, including indirectly required
#               cartridges. The cartridges can be installed in the order of the list.
#               The default implementation returns an empty list.
#----------------------------------------------------------------------------------------
# return       $aRequiredCartridges | list of cartridge objects | ref.array.object
#========================================================================================
sub requiredCartridges {
    my $self = shift;

    my %Cartridges;
    my @Cartridges;
    foreach my $CDirect ( @{ $self->directRequiredCartridges } ) {
        # add indirectly required cartridges before direct dependencies
        foreach my $Cartridge ( @{ $CDirect->requiredCartridges }, $CDirect ) {
            my $Package = $Cartridge->cartridgePackage;
            if( !exists $Cartridges{$Package} ) {
                $Cartridges{$Package} = $Cartridge;
                push @Cartridges, $Cartridge;
            }
        }
    }
    LogDebug('requiredCartrides of '. $self->cartridgePackage, @Cartridges);
    return \@Cartridges;
}

#========================================================================================
# function     directRequiredCartridges
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aRequiredCartridges = $Installer->directRequiredCartridges;
#----------------------------------------------------------------------------------------
# description  Returns a list of all cartridges that are directly required by this
#               cartridges. The list does not include cartridges that are required by
#               the required cartridges.
#----------------------------------------------------------------------------------------
# return       $aRequiredCartridges | list of cartridge objects | ref.array.object
#========================================================================================
sub directRequiredCartridges {
    return [];
}

#========================================================================================
# function     dependentCartridges
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aDependentCartridges = $Installer->dependentCartridges;
#----------------------------------------------------------------------------------------
# description  Returns a list of all cartridges that depend on this cartridge. The
#               list includes direct and indirect dependencies. The cartridges can be
#               uninstalled in the order of the list.
#               The default implementation returns an empty list.
#----------------------------------------------------------------------------------------
# return       $aDependentCartridges | list of cartridge objects | ref.array.object
#========================================================================================
sub dependentCartridges {
    my $self = shift;

    return $self->{DependentCartridges} if exists $self->{DependentCartridges};

    GetLog()->debug( 'get dependentCartridges of '. $self->cartridgePackage );

    my %Cartridges;
    my @Cartridges;
    foreach my $CDirect ( @{ $self->directDependentCartridges } ) {
        # add indirect dependencies before direct dependencies
        foreach my $Cartridge ( @{ $CDirect->dependentCartridges }, $CDirect ) {
            my $Package = $Cartridge->cartridgePackage;
            if( !exists $Cartridges{$Package} ) {
                $Cartridges{$Package} = $Cartridge;
                push @Cartridges, $Cartridge;
            }
        }
    }

    $self->{DependentCartridges} = \@Cartridges;
    return $self->{DependentCartridges};
}

#========================================================================================
# function     directDependentCartridges
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aDependentCartridges = $Installer->directDependentCartridges;
#----------------------------------------------------------------------------------------
# description  Returns a list of cartridges that have declared dependencies on this
#               cartridge. The list does not include indirect dependencies.
#               The default implementation returns an empty list.
#----------------------------------------------------------------------------------------
# return       $aDependentCartridges | list of cartridge objects | ref.array.object
#========================================================================================
sub directDependentCartridges {
    return [];
}

#========================================================================================
# function     installRequiredCartridges
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer->installRequiredCartridges;
#----------------------------------------------------------------------------------------
# description  Installs required cartridges.
#========================================================================================
sub installRequiredCartridges {
    my $self = shift;
    my $aCartridges = $self->directRequiredCartridges;
    LogDebug($self->cartridgePackage, [map { $_->cartridgePackage} @$aCartridges]);
    foreach my $Cartridge ( @$aCartridges) {
        if (not $Cartridge->isInstalled) {
            $Cartridge->installRequiredCartridges;
            $Cartridge->install;
        }
    }
    return;
}

#========================================================================================
# function     uninstallDependentCartridges
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer->uninstallDependentCartridges;
#----------------------------------------------------------------------------------------
# description  Uninstalls dependent cartridges (does nothing).
#========================================================================================
sub uninstallDependentCartridges {
    my $self = shift;
    my $aCartridges = $self->dependentCartridges;
    LogDebug($self->cartridgePackage, [map { $_->cartridgePackage} @$aCartridges]);
    foreach my $Cartridge ( @$aCartridges) {
        if ($Cartridge->isInstalled or $Cartridge->force) {
            $Cartridge->uninstall;
        }
    }
    return;
}

#========================================================================================
# function     testExists
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer->testExists;
#----------------------------------------------------------------------------------------
# description  Adds a reason if cartridge is not installed.
#               Calls <function addReason> if the cartridge is not yet installed.
#========================================================================================
sub testExists {
    my $self = shift;

    if (not $self->isInstalled) {
        $self->addReason({
            'Reason'    => 'CARTRIDGE_NOT_INSTALLED',
            'Package'   => $self->cartridgePackage,
            'Version'   => $self->version,
        });
    }
    return;
}

#========================================================================================
# function     testNotExists
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer->testNotExists;
#----------------------------------------------------------------------------------------
# description  Adds a reason if cartridge is installed.
#               Calls <function addReason> if the cartridge is already installed.
#========================================================================================
sub testNotExists {
    my $self = shift;

    if ($self->isInstalled) {
        $self->addReason({
            'Reason'    => 'CARTRIDGE_IS_INSTALLED',
            'Package'   => $self->cartridgePackage,
            'Version'   => $self->version,
        });
    }
    return;
}

#========================================================================================
# function     testInstall
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer->testInstall;
#----------------------------------------------------------------------------------------
# description  Tests installation of cartridge.
#               Calls <function addReason> for each reason why the cartridge cannot be
#               installed.
#========================================================================================
sub testInstall {
    my $self = shift;

    # don't run the test twice
    return if exists $self->{'TestInstall'};
    $self->{'TestInstall'} = 1;

    # test required cartridges
    $self->testInstallRequiredCartridges;
    return if $self->hasReasons;
    # test cartridge self
    $self->testNotExists;
    return;
}

#========================================================================================
# function     testInstallRequiredCartridges
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer->testInstallRequiredCartridges;
#----------------------------------------------------------------------------------------
# description  Tests installation of required cartridges.
#               Calls <function addReason> for each reason why a required cartridge
#               cannot be installed.
#========================================================================================
sub testInstallRequiredCartridges {
    my $self = shift;
    # test required cartridges
    my $aCartridges = $self->directRequiredCartridges;
    foreach my $Cartridge (@$aCartridges) {
        if ($self->isRecursive) {
            # test if all exists or can be installed
            next if $Cartridge->isInstalled;
            $Cartridge->testInstall;
        }
        else { # test if all exists
            if( ! $Cartridge->isInstalled ) {
                $self->addReason({
                    'Reason'    => 'REQUIRED_CARTRIDGE_NOT_INSTALLED',
                    'Version'   => $self->version,
                    'Package'   => $self->cartridgePackage,
                    'RequiredPackage' => $Cartridge->cartridgePackage,
                });
            }
        }
        $self->addReason($_) foreach @{$Cartridge->getReasons};
        $self->testCartridgeCompliant($Cartridge);
    }
}

#========================================================================================
# function     testUninstall
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer->testUninstall;
#----------------------------------------------------------------------------------------
# description  Tests uninstallation of cartridge.
#               Calls <function addReason> for each reason why the cartridge cannot be
#               uninstalled.
#========================================================================================
sub testUninstall {
    my $self = shift;

    # don't run the test twice
    return if exists $self->{'TestUninstall'};
    $self->{'TestUninstall'} = 1;

    # test cartridge self
    $self->testExists;
    # test dependent cartridges
    $self->testUninstallDependentCartridges if $self->isInstalled;
    return;
}

#========================================================================================
# function     testUninstallDependentCartridges
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer->testUninstallDependentCartridges;
#----------------------------------------------------------------------------------------
# description  Tests deinstallation of dependent cartridges.
#               Calls <function addReason> for each reason why a dependent cartridge
#               cannot be uninstalled.
#========================================================================================
sub testUninstallDependentCartridges {
    my $self = shift;

    GetLog()->debug( 'testUninstallDependentCartridges of ' .$self->cartridgePackage );

    my $aCartridges = $self->dependentCartridges;

    foreach my $Cartridge (@$aCartridges) {
        if ($self->isRecursive) {
            next unless $Cartridge->isInstalled;
            $Cartridge->testUninstall;
        } else {
            $Cartridge->testNotExists;
        }
        $self->addReason($_) foreach @{$Cartridge->getReasons};
    }
    return;
}

# This cache works only because the cartridge installer is invoked from a command-line script.
# If the cartridge installer is used by the application server, the cache will never be updated.
my %INSTALLERS;

#========================================================================================
# function     loadInstaller
# state        public
#----------------------------------------------------------------------------------------
# syntax       $OtherInstaller = $Installer->loadInstaller( $Package );
#----------------------------------------------------------------------------------------
# description  Loads other installer.
#----------------------------------------------------------------------------------------
# input        $Package | cartridge package, e.g. DE_EPAGES::Shop | string
# return       $OtherInstaller | cartridge installer object | object
#========================================================================================
sub loadInstaller {
    my $self = shift;
    my ($Package) = @_;

    return $INSTALLERS{$Package} if exists $INSTALLERS{$Package};

    GetLog()->debug( "loadInstaller $Package" );

    my $CartridgePackage = $Package . '::API::Cartridge';
    UsePackage($CartridgePackage);
    $INSTALLERS{$Package} = $CartridgePackage->new(
        'StoreName'   => $self->{'StoreName'},
        'StoreType'   => $self->{'StoreType'},
        'Force'       => $self->force,
        'IsRecursive' => $self->isRecursive,
        'NoScheduler' => $self->noScheduler,
        'NoSharedConfig' => $self->noSharedConfig,
    );

    return $INSTALLERS{$Package};
}

#========================================================================================
# function     ResetInstallerCache
# state        public
#----------------------------------------------------------------------------------------
# syntax       ResetInstallerCache();
#----------------------------------------------------------------------------------------
# description  Reset the internal installer cache
#========================================================================================
sub ResetInstallerCache() {
    %INSTALLERS = ();
    return;
}

#========================================================================================
# function     isCompliant
# state        public
#----------------------------------------------------------------------------------------
# syntax       $IsCompliant = $Installer->isCompliant($RequiredVersion, $CurrentVersion);
# example      $IsCompliant = $Installer->isCompliant('5.0.0.4');
# example      $IsCompliant = $Installer->isCompliant('5.0.0.4', '6.1'); # false
# example      $IsCompliant = $Installer->isCompliant('5.0.0.4', '5.0.1'); # true
#----------------------------------------------------------------------------------------
# description  Returns whether or not the current version is compliant to the new
#               version of patch.
#               Returns true if the major version is equal and the current minor version
#               is equal or higher than required.
#----------------------------------------------------------------------------------------
# input        $RequiredVersion | required version | string
# input        $CurrentVersion | current version (optional default $self->version) | string
# return       $IsCompliant | true if the cartridge is compliant | boolean
#========================================================================================
sub isCompliant ($$) {
    my $self = shift;
    my ($RequiredVersion, $CurrentVersion) = @_;
    $CurrentVersion = $self->version unless defined $CurrentVersion;

    # get version from database, else (install) current version
    my $RequiredMajor = ( split /\./, $RequiredVersion )[0];
    my $CurrentMajor = ( split /\./, $CurrentVersion )[0];

    # return false if major number differs
    return 0 if $RequiredMajor != $CurrentMajor;

    # return true if minor number(s) are equal or higher than required
    return versioncmp( $RequiredVersion, $CurrentVersion ) <= 0 ? 1 : 0;
}

#========================================================================================
# function     testCartridgeCompliant
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer->testCartridgeCompliant($Cartridge);
#----------------------------------------------------------------------------------------
# description  Tests if a required cartridge exists with a supported version.
#               Calls <function addReason> if the version of the required cartridge is
#               not compatible.
#----------------------------------------------------------------------------------------
# input        $Cartridge | installer object of required cartridge, calls methods
#                   <function cartridgePackage>,
#                   <function isCompliant> and
#                   <function version>
#                   | object
#========================================================================================
sub testCartridgeCompliant {
    my $self = shift;
    my ($Cartridge) = @_;
    my $SupportedVersions = $self->supportedVersions($Cartridge->cartridgePackage);
    return unless scalar @$SupportedVersions;
    foreach my $Version (@$SupportedVersions) {
        return if $Cartridge->isCompliant($Version);
    }
    $self->addReason({
        'Reason'                    => 'REQUIRED_CARTRIDGE_NOT_COMPLIANT',
        'RequiredPackage'           => $Cartridge->cartridgePackage,
        'RequiredPackageVersion'    => $Cartridge->version,
        'Package'                   => $self->cartridgePackage,
        'Version'                   => $self->version,
        'SupportedVersions'         => join ('/', @$SupportedVersions),
    });
    return;
}

#========================================================================================
# function     supportedVersions
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aVersions = $Installer->supportedVersions($Package);
#----------------------------------------------------------------------------------------
# description  Returns a list of supported versions of the required cartridge.
#----------------------------------------------------------------------------------------
# input        $Package | package name of a required cartridge | string
# return       $aVersions | list of version strings | ref.array.string
#========================================================================================
sub supportedVersions {
    my $self = shift;
    my ($Package) = @_;
    return [];
}

#========================================================================================
# function     noScheduler
# state        public
#----------------------------------------------------------------------------------------
# syntax       $NoScheduler = $Installer->noScheduler;
#----------------------------------------------------------------------------------------
# description  Returns true if cartridges are to be installed without scheduler files
#----------------------------------------------------------------------------------------
# return       $NoScheduler | no scheduler flag | boolean
#========================================================================================
sub noScheduler {
    my $self = shift;
    return $self->{'NoScheduler'};
}

#========================================================================================
# function     noSharedConfig
# state        public
#----------------------------------------------------------------------------------------
# syntax       $NoSharedConfig = $Installer->noSharedConfig;
#----------------------------------------------------------------------------------------
# description  Returns true if cartridges are to be installed without config files
#----------------------------------------------------------------------------------------
# return       $NoSharedConfig | no config flag | boolean
#========================================================================================
sub noSharedConfig {
    my $self = shift;
    return $self->{'NoSharedConfig'};
}

#========================================================================================
# function     dependencyComment
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Installer->dependencyComment($hPackage);
#----------------------------------------------------------------------------------------
# example      my $Comment = $Installer->dependencyComment('DE_EPAGES::ExternalPayment');
#----------------------------------------------------------------------------------------
# description  Returns the comment for a cartridge dependency. These method is abstract.
#----------------------------------------------------------------------------------------
# input        $Package | package of dependend cartridge | string
# input        $hParams | patch file parameters, keys are
#               <ul>
#                   <li>DependsOnActualCartridge - show if given package depends on cartridge or cartridge depends on package - bool
#               </ul> | ref.hash.*
# return       $Comment | dependency comment | string
#========================================================================================
sub dependencyComment{
    my $self = shift;
    my ($Package, $hParams) = @_;

    #abstract method

    return '';
}

#========================================================================================
# function     logInfo
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Installer->logInfo($Message);
#----------------------------------------------------------------------------------------
# example      $Installer->logInfo('note message XYZ ...');
#----------------------------------------------------------------------------------------
# description  Writes the info into theinfo log. Uses GetLog()->info().
#----------------------------------------------------------------------------------------
# input        $Message | log message | string
#========================================================================================
sub logInfo{
    my $self = shift;
    my ($Message) = @_;

    # set log category to caller function
    my $CallerFunction = (caller(1))[3];

    GetLog($CallerFunction)->info($Message);

    return;
}


1;
