#========================================================================================
# package      DE_EPAGES::EPagesJ::API::Object::HTTPClient
# state        public
#----------------------------------------------------------------------------------------
# description  HTTP communication adapter<br>
#               All HTTP requests namely <function get>, <function post>, <function delete>
#               have in commen: <br>
#           <ul>
#               <li>They apply default HTTP header to each request provided by <function new></li>
#               <li>They throw an error 'REQUEST_FAILED' if HTTP::Response::is_success() is false
#               or the response don't arrive in a timely manner.<br>
#               The Response time limit can be set by calling by $Client->userAgent->timeout($seconds),
#               default timeout is 10 seconds.</li>
#           </ul>
#========================================================================================
package DE_EPAGES::EPagesJ::API::Object::HTTPClient;

use strict;

use LWP::UserAgent;
use Net::Ping;
use URI::URL ();
use HTTP::Request::Common qw ( DELETE GET POST PUT );

use DE_EPAGES::Core::API::Error qw ( Error );
use DE_EPAGES::Core::API::Log qw ( LogContext );
use DE_EPAGES::EPagesJ::API::Object::CommunicationLogger;

sub ErrorDefinition {
    return { 'REQUEST_FAILED' => "HTTP request failed: #StatusLine.\nRequest:\n#Request\n\nResponse:\n#Response", };
}

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $Adapter = DE_EPAGES::EPagesJ::API::Object::HTTPClient->new( %options );
#----------------------------------------------------------------------------------------
# description  creates a new remote adapter
#----------------------------------------------------------------------------------------
# input        %options - used keys: <ul>
#                <li>UserAgent - LWP::UserAgent </li>
#                <li>Log - Logger based on interface compatible to
#                    <package DE_EPAGES::EPagesJ::API::Object::CommunicationLogger>
#                </li>
#                <li>HTTPTimeOut - Timeout for HTTP communication - int</li>
#                <li>DefaultHeaders - default HTTP headers to send - ref.hash </li>
#                </ul> | hash
# return       $Adapter | new instance | DE_EPAGES::EPagesJ::API::Object::HTTPClient
#========================================================================================
sub new {
    my $class = shift;
    my (%options) = @_;
    $options{UserAgent} //= _createUserAgent($options{'HTTPTimeOut'});
    $options{Log} //= DE_EPAGES::EPagesJ::API::Object::CommunicationLogger->new(%options);

    my $self = bless \%options, $class;
    return $self;
}

#========================================================================================
# function     get
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $HTTPResponse = $Adapter->get($Path, $hHeaders );
#----------------------------------------------------------------------------------------
# description  Construct and execute a HTTP::Request::Common::GET request using $URL and $hHeader
#----------------------------------------------------------------------------------------
# input        $URL | URL | URI::URL
# input        $hHeaders | additional http headers | ref.hash
# return       $HTTPResponse | HTTP::Response object | HTTP::Response
#========================================================================================
sub get {
    my $self = shift;
    my ( $URL, $hHeaders ) = @_;
    my $Request = GET( $URL, %{ $self->headers($hHeaders) } );
    return $self->send($Request);
}

#========================================================================================
# function     post
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $HTTPResponse = $Adapter->post($Path, $hHeader, $Content );
#----------------------------------------------------------------------------------------
# example      my $HTTPResponse = $Adapter->post($Path, {'PriorityHeader' => true}, 'Body of my POST...' );
#----------------------------------------------------------------------------------------
# description  Construct a HTTP::Request::Common::POST using $Content, $hHeaders and
#               execute the request.
#----------------------------------------------------------------------------------------
# input        $URL | URL to post to | ref.hash
# input        $hHeaders | HTTP headers | ref.hash
# input        $Content | Body/Content of POST request | String
# return       $HTTPResponse | HTTP::Response object | HTTP::Response
#========================================================================================
sub post {
    my $self = shift;
    my ( $URL, $hHeaders, $Content ) = @_;
    $Content //= '';
    my $Request = POST( $URL, %{ $self->headers($hHeaders) }, Content => $Content );
    my $Response = $self->send($Request);
    return $Response;
}

#========================================================================================
# function     put
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $HTTPResponse = $Adapter->put($Path, $hHeader, $Content );
#----------------------------------------------------------------------------------------
# example      my $HTTPResponse = $Adapter->put($Path, {'PriorityHeader' => true}, 'Body of my put...' );
#----------------------------------------------------------------------------------------
# description  Construct a HTTP::Request::Common::put using $Content, $hHeaders and
#               execute the request.
#----------------------------------------------------------------------------------------
# input        $URL | URL to put to | ref.hash
# input        $hHeaders | HTTP headers | ref.hash
# input        $Content | Body/Content of put request | String
# return       $HTTPResponse | HTTP::Response object | HTTP::Response
#========================================================================================
sub put {
    my $self = shift;
    my ( $URL, $hHeaders, $Content ) = @_;
    $Content //= '';
    my $Request = PUT( $URL, %{ $self->headers($hHeaders) }, Content => $Content );
    my $Response = $self->send($Request);
    return $Response;
}

#========================================================================================
# function     delete
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $HTTPResponse = $Adapter->delete($Path, $hHeader );
#----------------------------------------------------------------------------------------
# example      my $HTTPResponse = $Adapter->delete($Path, {'PriorityHeader' => true});
#----------------------------------------------------------------------------------------
# description  Construct and execute a HTTP::Request::Commen::DELETE using $URL and $hHeaders
#----------------------------------------------------------------------------------------
# input        $URL | URL to post to | ref.hash
# input        $hHeaders | HTTP headers | ref.hash
# return       $HTTPResponse | HTTP::Response object | HTTP::Response
#========================================================================================
sub delete {
    my $self = shift;
    my ( $URL, $hHeaders ) = @_;
    my $Request = DELETE( $URL, %{ $self->headers($hHeaders) } );
    my $Response = $self->send($Request);
    return $Response;
}

#========================================================================================
# function     userAgent
# state        private
#----------------------------------------------------------------------------------------
# syntax       my $UserAgent = $Adapter->userAgent();
#----------------------------------------------------------------------------------------
# description  Accessor to LWP::UserAgent
#----------------------------------------------------------------------------------------
# return       $UserAgent | return LWP::UserAgent used by EPagesJ | object.ref
#========================================================================================
sub userAgent {
    return shift->{UserAgent};
}

#========================================================================================
# function     logger
# state        private
#----------------------------------------------------------------------------------------
# syntax       my $Logger = $Adapter->logger();
#----------------------------------------------------------------------------------------
# description  Accessor to Logger
#----------------------------------------------------------------------------------------
# return       $Logger | Logger object | DE_EPAGES::EPagesJ::API::Object::CommunicationLogger
#========================================================================================
sub logger {
    return shift->{Log};
}

#========================================================================================
# function     defaultHeaders
# state        private
#----------------------------------------------------------------------------------------
# syntax       my $hDefaultHeaders = $Adapter->defaultHeaders();
#----------------------------------------------------------------------------------------
# description  Accessor to default HTTP headers
#----------------------------------------------------------------------------------------
# return       $hDefaultHeaders | default HTTP headers | ref.hash
#========================================================================================
sub defaultHeaders {
    return shift->{DefaultHeaders} // {};
}

#========================================================================================
# function     send
# state        private
#----------------------------------------------------------------------------------------
# syntax       my $Response = $Adapter->send( $Request );
#----------------------------------------------------------------------------------------
# description  Sends and logs an HTTP request and returns the HTTP response. Throws error
#               'REQUEST FAILED' if HTTP::Response::is_success() is false.<br>
#               <br>
#               Time limit is defined by $Adapter->userAgent->timeout($seconds).
#               See <function userAgent>
#----------------------------------------------------------------------------------------
# input        $Request | request | HTTP::Request
# return       $Response | response | HTTP::Response
#========================================================================================
sub send {
    my $self = shift;
    my ($Request) = @_;

    my $Response;

    LogContext($self->{LogContext}//{}, sub {
        $self->logger->logCommunication($Request);
        $Response = $self->userAgent()->request($Request);
        $self->logger->logCommunication($Response);

        if ( !$Response->is_success() ) {
            Error('REQUEST_FAILED', {
                Request     => $Request->method . ' ' . $Request->uri . "\n" . $Request->content,
                Response    => $Response->decoded_content,
                Status      => $Response->code,
                StatusLine  => $Response->status_line,
                HTTPTimeout => $self->userAgent->timeout,
            });
        }
    });
    return $Response;
}

#========================================================================================
# function     headers
# state        private
#----------------------------------------------------------------------------------------
# syntax       my $hHeaders = $Adapter->headers( $customHeaders );
#----------------------------------------------------------------------------------------
# description  Combines default HTTP headers and custom HTTP headers
#----------------------------------------------------------------------------------------
# input        $customHeaders | HTTP headers | ref.hash
# return       $hHeaders | ref.hash
#========================================================================================
sub headers {
    my $self = shift;
    my ($customHeaders) = @_;
    return { %{ $self->defaultHeaders }, %{ $customHeaders // {} }, };
}

#========================================================================================
# function     _createUserAgent
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->_createUserAgent( $Timeout );
#----------------------------------------------------------------------------------------
# description  Creates an LWP::UserAgent
#----------------------------------------------------------------------------------------
# input        $Timeout | timeout in seconds | integer
# return       $ua | UserAgent | LWP::UserAgent
#========================================================================================
sub _createUserAgent {
    my ($Timeout) = @_;
    my $ua = LWP::UserAgent->new;
    if (defined $Timeout) {
        $ua->timeout($Timeout);
    }
    return $ua;
}

1;
