#========================================================================================
# package      DE_EPAGES::EPagesJ::API::PDF::PdfClient
# base         DE_EPAGES::EPagesJ::API::Object::RestClient
# state        public
# description  Client for PDF operations.
#========================================================================================
package DE_EPAGES::EPagesJ::API::PDF::PdfClient;
use base DE_EPAGES::EPagesJ::API::Object::RestClient;

use strict;
use Encode qw(encode);
use DE_EPAGES::Core::API::EpagesConfig qw ( EpagesConfig );
use DE_EPAGES::Core::API::Html qw ( BalanceHTMLTags );
use DE_EPAGES::Core::API::TestParameter qw ( TestObject );
use DE_EPAGES::EPagesJ::API::Object::SummaryLogger;
use DE_EPAGES::EPagesJ::API::Object::UriTemplate;
use DE_EPAGES::EPagesJ::API::UriTemplateFactory qw ( CreateUriTemplates );
use Readonly;

Readonly my $TRANSFORM_PATH => '{dsName}/{shopId}/pdf';
Readonly my $MERGE_PATH     => '{dsName}/{shopId}/pdf/merge';
Readonly my $DEFAULT_PDF_TIMEOUT => 180;
Readonly my $DEFAULT_HTTP_HEADERS => { 'Accept' => 'application/octet-stream' };

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       DE_EPAGES::EPagesJ::API::PDF::PdfClient->new( %args );
#----------------------------------------------------------------------------------------
# input        %args | Input arguments. Possible keys.<ul>
#                 <li>TransformUrl - url for HTML to PDF transformation - URI::URL</li>
#                 <li>MergeUrl - url for merging PDF files - URI::URL</li>
#                 <li>LogCategory - category for commlogger. Defaults to "PDF" - string</li>
#                 <li>DefaultHeaders - default HTTP headers - string</li>
#                 <li>Log - comm logger. Defaults to <package DE_EPAGES::EPagesJ::API::Object::SummaryLogger> - object</li>
#               </ul> | hash
# description  Instanciates a new PdfClient.
#========================================================================================
sub new {
    my $class = shift;
    my %args  = @_;
    $args{LogCategory}    //= 'PDF';
    $args{DefaultHeaders} //= $DEFAULT_HTTP_HEADERS;
    $args{Log}            //= DE_EPAGES::EPagesJ::API::Object::SummaryLogger->new(%args);

    foreach my $Mandatory (qw ( TransformUrl MergeUrl )) {
        TestObject($Mandatory, $args{$Mandatory});
    }

    return __PACKAGE__->SUPER::new(%args);
}

#========================================================================================
# function     create
# state        public
#----------------------------------------------------------------------------------------
# syntax       DE_EPAGES::EPagesJ::API::PDF->create($Shop, $hArgs);
#----------------------------------------------------------------------------------------
# description  Factory method for a PDFClient object.
#               Initializes based on the current configuration. Uses
#               a different HTTP timeout, that can be configured in epages.conf
#               [DE_EPAGES::EPagesJ]
#               HTTPTimeOutPdf = 180 ; 180 sec is the default timeout for PDF requests.
#----------------------------------------------------------------------------------------
# input        $Object |  This context object will be used for initializing BaseURL
#               and Resource parameters for the rest client. | object
# input        $hArgs | Additional arguments for constructor | ref.hash
# return       $PdfClient | PdfClient object | DE_EPAGES::EPagesJ::API::PDF::PdfClient
#========================================================================================
sub create {
    my $class = shift;
    my ( $Object, $hArgs ) = @_;
    $hArgs->{HTTPTimeOut}  //= EpagesConfig->get('DE_EPAGES::EPagesJ', 'HTTPTimeOutPdf', $DEFAULT_PDF_TIMEOUT);
    my $hTemplates = CreateUriTemplates($Object, [$TRANSFORM_PATH, $MERGE_PATH], $hArgs->{BaseURL});
    $hArgs->{TransformUrl} = $hTemplates->{$TRANSFORM_PATH}->uri;
    $hArgs->{MergeUrl}     = $hTemplates->{$MERGE_PATH}->uri;
    return __PACKAGE__->SUPER::create($Object, $hArgs);
}

#========================================================================================
# function     transform
# state        public
#----------------------------------------------------------------------------------------
# syntax       $PdfClient->transform( $HtmlContent );
#----------------------------------------------------------------------------------------
# description  Transforms HTML content into a pdf document.
#               It will try its best to create valid XHTML from the user input, as
#               invalid XHTML input gets rejected by the PDF generator.
#----------------------------------------------------------------------------------------
# input        $HtmlContent | HTML | string
# return       $Content | binary content | bytes
#========================================================================================
sub transform {
    my $self = shift;
    my ($HtmlContent) = @_;
    $HtmlContent = BalanceHTMLTags($HtmlContent, { 'getCompleteDocument' => 1 });
    $HtmlContent =~ s,#INPUT.OutputEncoding,utf-8,g;
    my $Response = $self->client->post( $self->transformUrl, {'Content-type' => 'text/html'},
                                        encode('UTF-8',$HtmlContent) );
    # return binary content
    return $Response->content;
}

#========================================================================================
# function     merge
# state        public
#----------------------------------------------------------------------------------------
# syntax       $PdfClient->merge( $hParams );
#----------------------------------------------------------------------------------------
# example      my $Client = DE_EPAGES::EPagesJ::API::PDF::PdfClient->create($Shop);
#               my $Pdf = $Client->merge( { 'files' => [
#                 { 'path' => 'WebRoot/File1.pdf' },
#                 { 'path' => 'WebRoot/File2.pdf' } ] } );
#               WriteFile('merged.pdf', \$Pdf, { binary => 1});
#----------------------------------------------------------------------------------------
# description  Merges multiple PDF files into one.
#----------------------------------------------------------------------------------------
# input        $hParams - | input params. Valid keys are: <ul>
#                 <li>files - files list. Valid keys for elements: <ul>
#                   <li>path - path relative to Shared folder - string</li>
#                 </ul> - ref.array.hash </li>
#               </ul> | ref.hash
# return       $Content | binary content | bytes
#========================================================================================
sub merge {
    my $self = shift;
    my ($hParams) = @_;
    my $Response = $self->client->post( $self->mergeUrl,
        {'Content-type' => 'application/json'},
        $self->encodeRequest($hParams)
    );
    # return binary content
    return $Response->content;
}

#========================================================================================
# function     transformUrl
# state        private
#----------------------------------------------------------------------------------------
# syntax       my $url = $Client->transformUrl();
#----------------------------------------------------------------------------------------
# description  Returns url for HTTP request to transform HTML into PDF.
#----------------------------------------------------------------------------------------
# return       $url | url | URI::URL
#========================================================================================
sub transformUrl {
    return shift->{TransformUrl};
}

#========================================================================================
# function     mergeUrl
# state        private
#----------------------------------------------------------------------------------------
# syntax       my $url = $Client->mergeUrl();
#----------------------------------------------------------------------------------------
# description  Returns url for HTTP request to merge PDF files
#----------------------------------------------------------------------------------------
# return       $url | url | URI::URL
#========================================================================================
sub mergeUrl {
    return shift->{MergeUrl};
}

1;
