#========================================================================================
# package      DE_EPAGES::GoogleSmartShopping::API::GoogleApiService
# state        public
#----------------------------------------------------------------------------------------
# description  Service to (directly) connect to Google API services
#========================================================================================
package DE_EPAGES::GoogleSmartShopping::API::GoogleApiService;

use DE_EPAGES::Object::API::Factory qw ( LoadRootObject );

use Encode qw ( encode );
use HTTP::Request;
use HTTP::Status qw ( is_server_error );
use LWP::UserAgent;
use JSON::MaybeXS qw ( decode_json );

use strict;

use constant GOOGLE_OPENID_USERINFO_URL => 'https://openidconnect.googleapis.com/v1/userinfo';

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       DE_EPAGES::GoogleSmartShopping::API::GoogleApiService->new( %args );
#----------------------------------------------------------------------------------------
# description  Instantiates a new GoogleApiService.
#----------------------------------------------------------------------------------------
# return       GoogleApiService | object | DE_EPAGES::GoogleSmartShopping::API::GoogleApiService
#========================================================================================
sub new {
    my $class = shift;
    my $self  = bless {}, $class;

    return $self;
}


#========================================================================================
# function     getOAuthTokens
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hResponse = $self->getOAuthTokens();
#----------------------------------------------------------------------------------------
# description  Given an OAuth code it gets Google API access tokens.
#----------------------------------------------------------------------------------------
# input        $OAuthCode | authorization code string | string
# return       $hResponse | hash with response | hashref
#========================================================================================
sub getOAuthTokens {
    my $self = shift;
    my ($OAuthCode) = @_;

    my $hSystemSettings = LoadRootObject()->get([qw(
        GoogleSmartShopping_AuthTokenURL
        GoogleSmartShopping_AppClientID
        GoogleSmartShopping_AppClientSecret
        GoogleSmartShopping_AppRedirectURL
    )]);

    my $Url = $hSystemSettings->{'GoogleSmartShopping_AuthTokenURL'};
    my $Header = [ 'Content-Type' => 'application/x-www-form-urlencoded' ];
    my $RequestContent = sprintf(
        'code=%s&client_id=%s&client_secret=%s&grant_type=%s&redirect_uri=%s',
        $OAuthCode,
        $hSystemSettings->{'GoogleSmartShopping_AppClientID'},
        $hSystemSettings->{'GoogleSmartShopping_AppClientSecret'},
        'authorization_code',
        $hSystemSettings->{'GoogleSmartShopping_AppRedirectURL'}
    );

    my $request = HTTP::Request->new('POST', $Url, $Header, encode("UTF-8", $RequestContent));
    my $userAgent = LWP::UserAgent->new();
    my $response = $userAgent->request($request);

    my $hResponse;
    if ( is_server_error($response->code) ) {
        $hResponse = {
            error => 'INTERNAL_SERVER_ERROR',
            errorMessage => 'Internal Server Error',
        }
    } else {
        $hResponse = decode_json($response->decoded_content());
    }

    return $hResponse;
}


#========================================================================================
# function     getUserInfo
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hResponse = $self->getUserInfo($AccessToken);
#----------------------------------------------------------------------------------------
# description  Given an $AccessToken it gets email ofthe connected account
#----------------------------------------------------------------------------------------
# input        $AccessToken | access token string | string
# return       $hResponse | hash with response | hashref
#========================================================================================
sub getUserInfo {
    my $self = shift;
    my ($AccessToken) = @_;

    my $Url = GOOGLE_OPENID_USERINFO_URL;
    my $Header = [ 'Accept'  => 'application/json',
                   'Authorization' => "Bearer $AccessToken"];

    my $request = HTTP::Request->new('GET', $Url, $Header);
    my $userAgent = LWP::UserAgent->new();
    my $response = $userAgent->request($request);

    my $hResponse;
    # connection error should also be handled
    if ( is_server_error($response->code) ) {
        $hResponse = {
            error => 'INTERNAL_SERVER_ERROR',
            errorMessage => 'Internal Server Error',
        };
    } else {
        $hResponse = decode_json($response->decoded_content());
    }

    return $hResponse;
}
1;