#==============================================================================
# package      DE_EPAGES::GoogleSmartShopping::UI::Shop
# state        private
#==============================================================================
package DE_EPAGES::GoogleSmartShopping::UI::Shop;
use base DE_EPAGES::Presentation::UI::Object;

use DE_EPAGES::GoogleSmartShopping::API::GoogleApiService;
use DE_EPAGES::GoogleSmartShopping::API::GoogleSmartShoppingUrlBuilder;
use DE_EPAGES::GoogleSmartShopping::API::Tools qw ( LogGoogleSmartShopping );
use DE_EPAGES::Shop::API::Url qw ( BuildShopUrl );

use strict;

use constant OAUTH_ONBOARDING => 0;
use constant OAUTH_ONBOARDING_CALLBACK_SUCCESS => 1;
use constant OAUTH_ONBOARDING_CALLBACK_ERROR => 2;
use constant ACCOUNT_CONNECTION_GET_EMAIL => 3;

#==============================================================================
# Event handlers
#==============================================================================
sub UnityViewGoogleSmartShopping {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object();
    my $Form = $Servlet->form();
    my $hQueryParameters = $Form->queryValues();
    my $aSetupStages = $self->_reckonSetupStages($Shop, $hQueryParameters);

    if (OAUTH_ONBOARDING ~~ $aSetupStages) {
        $Servlet->vars('GoogleOAuthConnect', $self->_buildConnectOAuthURL($Servlet, $Shop));
    }
    if (OAUTH_ONBOARDING_CALLBACK_SUCCESS ~~ $aSetupStages) {
        LogGoogleSmartShopping('Callback Success: ', $hQueryParameters);
        my $hResponse = $self->_exchangeAuthCodeForAccessTokens($hQueryParameters->{'code'}->[0]);
        LogGoogleSmartShopping('OAuth code exchange response: ', $hResponse);

        if ($hResponse->{'access_token'}) {
            $self->_saveTokensToConfig($Shop, $hResponse->{'access_token'}, $hResponse->{'refresh_token'});
            $Servlet->vars('ConnectedGoogleAccountEmail', $self->_getConnectedAccountEmail($Shop));
        } else {
            $Servlet->vars('OnboardingError', $hResponse->{'error'} // 'UNKNOWN_ERROR');
        }
    }
    if (OAUTH_ONBOARDING_CALLBACK_ERROR ~~ $aSetupStages) {
        LogGoogleSmartShopping('Callback Error: ', $hQueryParameters);
        $Servlet->vars('OnboardingError', $hQueryParameters->{'error'}->[0]);
    }
    if (ACCOUNT_CONNECTION_GET_EMAIL ~~ $aSetupStages) {
        $Servlet->vars('ConnectedGoogleAccountEmail', $self->_getConnectedAccountEmail($Shop));
    }

    return $self->SUPER::View($Servlet);
}

#----------------------------------------------------------------------------------------
sub _reckonSetupStages {
    my $self = shift;
    my ($Shop, $hQueryParameters) = @_;

    return [ OAUTH_ONBOARDING, OAUTH_ONBOARDING_CALLBACK_SUCCESS ] if ($hQueryParameters->{'code'});
    return [ OAUTH_ONBOARDING, OAUTH_ONBOARDING_CALLBACK_ERROR ] if ($hQueryParameters->{'error'});

    return [ OAUTH_ONBOARDING ] unless ($Shop->existsChild('GoogleSmartShoppingConfig'));
    my $GoogleSmartShoppingConfig = $Shop->get('GoogleSmartShoppingConfig');
    return [ OAUTH_ONBOARDING ] unless ($GoogleSmartShoppingConfig->get('AccessToken'));
    return [ OAUTH_ONBOARDING ] unless ($GoogleSmartShoppingConfig->get('RefreshToken'));

    my $aSecondStageMultiStep = [ ACCOUNT_CONNECTION_GET_EMAIL ];

    return $aSecondStageMultiStep;
}

#----------------------------------------------------------------------------------------
sub _buildConnectOAuthURL {
    my $self = shift;
    my ( $Servlet, $Shop ) = @_;

    my $hFormValues = $Servlet->params();
    my $hParams = {
        'ViewAction' => $hFormValues->{'ViewAction'},
        'ObjectID'   => $Shop->id(),
    };
    my $hOptions = {
        'Type' => 'admin',
        'UseSSL' => 1,
    };
    my $CurrentUrl = BuildShopUrl($Shop, $hParams, $hOptions, $Servlet);
    my $GoogleUrlBuilder = DE_EPAGES::GoogleSmartShopping::API::GoogleSmartShoppingUrlBuilder->new();

    return $GoogleUrlBuilder->buildConnectOAuthUrl($CurrentUrl);
}

#----------------------------------------------------------------------------------------
sub _exchangeAuthCodeForAccessTokens {
    my $self = shift;
    my ($OAuthCode) = @_;

    my $GoogleService = DE_EPAGES::GoogleSmartShopping::API::GoogleApiService->new();
    return $GoogleService->getOAuthTokens($OAuthCode);
}

#----------------------------------------------------------------------------------------
sub _saveTokensToConfig {
    my $self = shift;
    my ($Shop, $AccessToken, $RefreshToken) = @_;

    unless($Shop->existsChild('GoogleSmartShoppingConfig')) {
        $Shop->insertChild('GoogleSmartShoppingConfig', { 'Alias' => 'GoogleSmartShoppingConfig' });
    }
    my $GoogleSmartShoppingConfig = $Shop->get('GoogleSmartShoppingConfig');
    $GoogleSmartShoppingConfig->set({ 'AccessToken' => $AccessToken });

    if (defined $RefreshToken) {
        $GoogleSmartShoppingConfig->set({ 'RefreshToken' => $RefreshToken });
    }

}
#----------------------------------------------------------------------------------------
sub _getConnectedAccountEmail {
    my $self = shift;
    my ($Shop) = @_;

    my $GoogleSmartShoppingConfig = $Shop->get('GoogleSmartShoppingConfig');
    return unless defined $GoogleSmartShoppingConfig;

    my $service = DE_EPAGES::GoogleSmartShopping::API::GoogleApiService->new();
    my $getUserInfoResult = $service->getUserInfo($GoogleSmartShoppingConfig->get('AccessToken'));
    if (defined $getUserInfoResult->{'email'}) {
        return $getUserInfoResult->{'email'};
    }

    LogGoogleSmartShopping("Google OpenID userinfo lookup failed: ", $getUserInfoResult);

    return;
}

1;
