#========================================================================================
# package      DE_EPAGES::GoogleSmartShopping::t::unit::GoogleApiService
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for UI::Provider
#========================================================================================
package DE_EPAGES::GoogleSmartShopping::t::unit::GoogleApiService;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use Test::Mockify::Sut;
use Test::Mockify::Matcher qw ( Object );
use Test::Exception;
use Test::MockObject::Extends;
use Test::More;

use JSON::MaybeXS qw ( encode_json );
use strict;

use constant AUTH_CODE => 'SOME_AUTH_CODE';
use constant ACCESS_TOKEN => 'SOME_ACCESS_TOKEN';
use constant REFRESH_TOKEN => 'SOME_REFRESH_TOKEN';
use constant ACCOUNT_EMAIL => 'some_email@example.com';

use constant APP_CLIENT_ID => 'SomeClientId';
use constant APP_CLIENT_SECRET => 'SomeClientSecret';
use constant AUTH_TOKEN_URL => 'SomeAuthTokenUrl';
use constant APP_REDIRECT_URL => 'SomeAppRedirectUrl';
use constant HTTP_OK => '200';
use constant HTTP_BAD_REQUEST => '400';
use constant HTTP_INTERNAL_SERVER_ERROR => '500';
use constant EXPECTED_ACCESS_TOKEN_RESPONSE => { 'access_token' => ACCESS_TOKEN };
use constant EXPECTED_USERINFO_RESPONSE => { 'email' => ACCOUNT_EMAIL };
use constant EXPECTED_ERROR_RESPONSE => { 'error' => 'INTERNAL_SERVER_ERROR', 'errorMessage' => 'Internal Server Error' };
use constant INVALID_GRANT_ERROR_RESPONSE => {'error' => 'invalid_grant', 'error_description' => 'Bad Request'};

sub test {
    my $self = shift;

    $self->test_get_oauth_token_success();
    $self->test_get_oauth_token_failure();
    $self->test_get_oauth_token_bad_request();
    $self->test_get_user_info_success();
    $self->test_get_user_info_failure();

    done_testing();
}

#----------------------------------------------------------------------------------------
# Tests
#----------------------------------------------------------------------------------------

sub test_get_oauth_token_success {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedUserAgent = $self->_createUserAgent(HTTP_OK, EXPECTED_ACCESS_TOKEN_RESPONSE);
    my $SUT;
    my $hTokenResponse;

    lives_ok(sub { $SUT = $self->_createSUT($MockedUserAgent) },
        "$SubTestName - Service initialization works.");

    lives_ok(sub { $hTokenResponse = $SUT->getOAuthTokens();},
        "$SubTestName - test if handleStatusOnPaymentWebhook can be called without errors");

    is($hTokenResponse->{'access_token'}, ACCESS_TOKEN, "$SubTestName - AccessToken correctly returned.");

}

sub test_get_oauth_token_failure {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedUserAgent = $self->_createUserAgent(HTTP_INTERNAL_SERVER_ERROR, EXPECTED_ERROR_RESPONSE);
    my $SUT;
    my $hTokenResponse;

    lives_ok(sub { $SUT = $self->_createSUT($MockedUserAgent) },
        "$SubTestName - Service initialization works.");

    lives_ok(sub { $hTokenResponse = $SUT->getOAuthTokens();},
        "$SubTestName - test if handleStatusOnPaymentWebhook can be called without errors");

    is($hTokenResponse->{'error'}, 'INTERNAL_SERVER_ERROR', "$SubTestName - AccessToken lookup failed.");

}

sub test_get_oauth_token_bad_request {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedUserAgent = $self->_createUserAgent(HTTP_BAD_REQUEST, INVALID_GRANT_ERROR_RESPONSE);
    my $SUT;
    my $hTokenResponse;

    lives_ok(sub { $SUT = $self->_createSUT($MockedUserAgent) },
        "$SubTestName - Service initialization works.");

    lives_ok(sub { $hTokenResponse = $SUT->getOAuthTokens();},
        "$SubTestName - test if handleStatusOnPaymentWebhook can be called without errors");

    is($hTokenResponse->{'error'}, 'invalid_grant', "$SubTestName - AccessToken bad request result (invalid grant).");

}

sub test_get_user_info_success {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedUserAgent = $self->_createUserAgent(HTTP_OK, EXPECTED_USERINFO_RESPONSE);
    my $SUT;
    my $hUserInfo;

    lives_ok(sub { $SUT = $self->_createSUT($MockedUserAgent) },
        "$SubTestName - Service initialization works.");

    lives_ok(sub { $hUserInfo = $SUT->getUserInfo(ACCESS_TOKEN);},
        "$SubTestName - test if handleStatusOnPaymentWebhook can be called without errors");

    is($hUserInfo->{'email'}, ACCOUNT_EMAIL, "$SubTestName - UserInfo email correctly returned.");

}


sub test_get_user_info_failure {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedUserAgent = $self->_createUserAgent(HTTP_INTERNAL_SERVER_ERROR, EXPECTED_ERROR_RESPONSE);
    my $SUT;
    my $hUserInfo;

    lives_ok(sub { $SUT = $self->_createSUT($MockedUserAgent) },
        "$SubTestName - Service initialization works.");

    lives_ok(sub { $hUserInfo = $SUT->getUserInfo(ACCESS_TOKEN);},
        "$SubTestName - test if handleStatusOnPaymentWebhook can be called without errors");

    is($hUserInfo->{'error'}, 'INTERNAL_SERVER_ERROR', "$SubTestName - UserInfo lookup failed.");

}

#----------------------------------------------------------------------------------------
# Helpers
#----------------------------------------------------------------------------------------
sub _createSUT {
    my $self = shift;
    my ($UserAgent) = @_;

    my $hParams = { 'oauth_code' => AUTH_CODE };

    my $Mockify = Test::Mockify::Sut->new('DE_EPAGES::GoogleSmartShopping::API::GoogleApiService', [$hParams]);

    $Mockify->mockImported('DE_EPAGES::Object::API::Factory', 'LoadRootObject')
        ->whenAny()
        ->thenReturn($self->_createSystem());
    $Mockify->mockStatic('LWP::UserAgent::new')->whenAny()->thenReturn($UserAgent);

    return $Mockify->getMockObject();
}

sub _createSystem {
    my $self = shift;

    my $hAttributes = {
        'GoogleSmartShopping_AppClientID' => APP_CLIENT_ID,
        'GoogleSmartShopping_AppClientSecret' => APP_CLIENT_SECRET,
        'GoogleSmartShopping_AuthTokenURL' => AUTH_TOKEN_URL,
        'GoogleSmartShopping_AppRedirectURL' => APP_REDIRECT_URL
    };

    my $MockifySystem = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Object::API::Object::Site', $hAttributes
    );

    return $MockifySystem->getMockObject();
}

sub _createUserAgent {
    my $self = shift;
    my ($Code, $hResponse) = @_;

    my $MockifyUA = Test::Mockify->new('LWP::UserAgent');
    $MockifyUA->mock('request')->when(Object('HTTP::Request'))->thenReturn($self->_createResponse($Code, $hResponse));
    return $MockifyUA->getMockObject();
}

sub _createResponse {
    my $self = shift;
    my ($Code, $hResponse) = @_;

    my $MockifyResponse = Test::Mockify->new('HTTP::Response');
    $MockifyResponse->mock('code')->when()->thenReturn($Code);
    my $UserInfoResponse = encode_json($hResponse);
    $MockifyResponse->mock('decoded_content')->when()->thenReturn($UserInfoResponse);
    return $MockifyResponse->getMockObject();

}

__PACKAGE__->runScript();

1;