#========================================================================================
# package      DE_EPAGES::GoogleSmartShopping::t::unit::UI_Shop
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for the GoogleSmartShopping view setup
#========================================================================================
package DE_EPAGES::GoogleSmartShopping::t::unit::UI_Shop;
use base DE_EPAGES::Core::API::Test;

use strict;

use DE_EPAGES::Test::Mock::Mockify qw ( WasCalled );
use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use DE_EPAGES::Test::Mock::MockifyServlet;

use Test::Mockify;
use Test::Mockify::Matcher qw ( Object String );
use Test::Mockify::Sut;
use Test::More;

use constant CONNECT_OAUTH_URL => 'https://www.example.com';
use constant GOOGLE_OAUTH_ERROR => 'oauth-access-denied';
use constant GOOGLE_OAUTH_TOKENS_EXCHANGE_ERROR => 'some-oauth-tokens-exchange-error';
use constant SUCCESSFUL_GOOGLE_OAUTH_CALLBACK_RESPONSE => {
    code  => [ 'some-google-authorization-code' ],
    scope => [ 'space-separated-list-of-scopes-string']
};
use constant UNSUCCESSFUL_GOOGLE_OAUTH_CALLBACK_RESPONSE => {
    error  => [ GOOGLE_OAUTH_ERROR ]
};
use constant SUCCESSFUL_GOOGLE_OAUTH_TOKENS_EXCHANGE_RESPONSE => {
    access_token  => 'some-google-access-token',
    refresh_token => 'some-google-refresh-token',
    id_token      => 'some-long-encrypted-key',
    expires_in    => 3599,
    token_type    => 'Bearer',
    scope         => 'space-separated-list-of-scopes-string'
};
use constant UNSUCCESSFUL_GOOGLE_OAUTH_TOKENS_EXCHANGE_RESPONSE => {
    error => GOOGLE_OAUTH_TOKENS_EXCHANGE_ERROR
};
use constant SUCCESSFUL_USER_INFO_RESPONSE => {
    email => 'google-account-email'
};

sub test {
    my $self = shift;

    $self->test_ViewGoogleSmartShopping_On_OnboardingStart_NoGoogleSmartShoppingConfig();
    $self->test_ViewGoogleSmartShopping_On_OnboardingStart_NoAccessToken();
    $self->test_ViewGoogleSmartShopping_On_OnboardingStart_NoRefreshToken();
    $self->test_ViewGoogleSmartShopping_On_OnboardingCallbackSuccess_And_SuccessfulTokensExchange();
    $self->test_ViewGoogleSmartShopping_On_OnboardingCallbackError();
    $self->test_ViewGoogleSmartShopping_On_OnboardingCallbackSuccess_And_UnsuccessfulTokensExchange();
    $self->test_ViewGoogleSmartShopping_On_GettingConnectedAccountEmail();

    done_testing();
}

sub test_ViewGoogleSmartShopping_On_OnboardingStart_NoGoogleSmartShoppingConfig {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = undef;
    my $AlreadyExistingConfig = 0;
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);
    my $hFormQueryParameters = {};
    my $ServletForm = $self->_createServletForm($hFormQueryParameters);
    my $Servlet = $self->_createServlet($Shop, $ServletForm);
    my $SUT = $self->_createSUT();

    $SUT->UnityViewGoogleSmartShopping($Servlet);

    is(
        $Servlet->vars('GoogleOAuthConnect'),
        CONNECT_OAUTH_URL,
        "$SubTestName - GoogleSmartShopping setup view has OAuth URL when there is no GoogleSmartShoppingConfig object"
    );
    ok( WasCalled($SUT, '_buildConnectOAuthURL'), $SubTestName . ' - Verify that OAuth connect URL was built' );
}

sub test_ViewGoogleSmartShopping_On_OnboardingStart_NoAccessToken {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = $self->_createGoogleSmartShoppingConfig(undef, undef);
    my $AlreadyExistingConfig = 1;
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);
    my $hFormQueryParameters = {};
    my $ServletForm = $self->_createServletForm($hFormQueryParameters);
    my $Servlet = $self->_createServlet($Shop, $ServletForm);
    my $SUT = $self->_createSUT();

    $SUT->UnityViewGoogleSmartShopping($Servlet);

    is(
        $Servlet->vars('GoogleOAuthConnect'),
        CONNECT_OAUTH_URL,
        "$SubTestName - GoogleSmartShopping setup view has OAuth URL when GoogleSmartShoppingConfig access token is empty"
    );
    ok( WasCalled($SUT, '_buildConnectOAuthURL'), $SubTestName . ' - Verify that OAuth connect URL was built' );
}

sub test_ViewGoogleSmartShopping_On_OnboardingStart_NoRefreshToken {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = $self->_createGoogleSmartShoppingConfig('some-access-token', undef);
    my $AlreadyExistingConfig = 1;
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);
    my $hFormQueryParameters = {};
    my $ServletForm = $self->_createServletForm($hFormQueryParameters);
    my $Servlet = $self->_createServlet($Shop, $ServletForm);
    my $SUT = $self->_createSUT();

    $SUT->UnityViewGoogleSmartShopping($Servlet);

    is(
        $Servlet->vars('GoogleOAuthConnect'),
        CONNECT_OAUTH_URL,
        "$SubTestName - GoogleSmartShopping setup view has OAuth URL when GoogleSmartShoppingConfig refresh token is empty"
    );
    ok( WasCalled($SUT, '_buildConnectOAuthURL'), $SubTestName . ' - Verify that OAuth connect URL was built' );
}

sub test_ViewGoogleSmartShopping_On_OnboardingCallbackSuccess_And_SuccessfulTokensExchange {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = $self->_createGoogleSmartShoppingConfig('some-access-token', 'some-refresh-token');
    my $AlreadyExistingConfig = 0;
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);
    my $hFormQueryParameters = SUCCESSFUL_GOOGLE_OAUTH_CALLBACK_RESPONSE;
    my $ServletForm = $self->_createServletForm($hFormQueryParameters);
    my $Servlet = $self->_createServlet($Shop, $ServletForm);
    my $ApiService = $self->_createGoogleApiService({
        OAuthTokens => SUCCESSFUL_GOOGLE_OAUTH_TOKENS_EXCHANGE_RESPONSE
    });
    my $SUT = $self->_createSUT($ApiService);

    $SUT->UnityViewGoogleSmartShopping($Servlet);

    ok(
        WasCalled($Shop, 'existsChild'),
        $SubTestName . ' - Verify that access token is being saved into GoogleSmartShoppingConfig object '
    );
    ok(
        WasCalled($Shop, 'insertChild'),
        $SubTestName . ' - Verify that GoogleSmartShoppingConfig object gets created because did not exist before'
    );
    ok( WasCalled($SUT, '_buildConnectOAuthURL'), $SubTestName . ' - Verify that OAuth connect URL was built' );
}

sub test_ViewGoogleSmartShopping_On_OnboardingCallbackError {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = undef;
    my $AlreadyExistingConfig = 0;
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);
    my $hFormQueryParameters = UNSUCCESSFUL_GOOGLE_OAUTH_CALLBACK_RESPONSE;
    my $ServletForm = $self->_createServletForm($hFormQueryParameters);
    my $Servlet = $self->_createServlet($Shop, $ServletForm);
    my $SUT = $self->_createSUT();

    $SUT->UnityViewGoogleSmartShopping($Servlet);

    is(
        $Servlet->vars('OnboardingError'),
        GOOGLE_OAUTH_ERROR,
        "$SubTestName - We get OAuth granting call error when it fails"
    );
    is(
        $Servlet->vars('GoogleOAuthConnect'),
        CONNECT_OAUTH_URL,
        "$SubTestName - GoogleSmartShopping setup view has OAuth URL when OAuth granting call fails"
    );
    ok( WasCalled($SUT, '_buildConnectOAuthURL'), $SubTestName . ' - Verify that OAuth connect URL was built' );
}

sub test_ViewGoogleSmartShopping_On_OnboardingCallbackSuccess_And_UnsuccessfulTokensExchange {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = $self->_createGoogleSmartShoppingConfig('some-access-token', 'some-refresh-token');
    my $AlreadyExistingConfig = 0;
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);
    my $hFormQueryParameters = SUCCESSFUL_GOOGLE_OAUTH_CALLBACK_RESPONSE;
    my $ServletForm = $self->_createServletForm($hFormQueryParameters);
    my $Servlet = $self->_createServlet($Shop, $ServletForm);
    my $ApiService = $self->_createGoogleApiService({
        OAuthTokens => UNSUCCESSFUL_GOOGLE_OAUTH_TOKENS_EXCHANGE_RESPONSE
    });
    my $SUT = $self->_createSUT($ApiService);

    $SUT->UnityViewGoogleSmartShopping($Servlet);

    is(
        $Servlet->vars('OnboardingError'),
        GOOGLE_OAUTH_TOKENS_EXCHANGE_ERROR,
        "$SubTestName - We get tokens exchange API call error when it fails");
    is(
        $Servlet->vars('GoogleOAuthConnect'),
        CONNECT_OAUTH_URL,
        "$SubTestName - GoogleSmartShopping setup view has OAuth URL when tokens exchange API call fails"
    );
    ok( WasCalled($SUT, '_buildConnectOAuthURL'), $SubTestName . ' - Verify that OAuth connect URL was built' );
}

sub test_ViewGoogleSmartShopping_On_GettingConnectedAccountEmail {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = $self->_createGoogleSmartShoppingConfig('some-access-token', 'some-refresh-token');
    my $AlreadyExistingConfig = 1;
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);
    my $hFormQueryParameters = {};
    my $ServletForm = $self->_createServletForm($hFormQueryParameters);
    my $Servlet = $self->_createServlet($Shop, $ServletForm);
    my $ApiService = $self->_createGoogleApiService({
        UserInfo => SUCCESSFUL_USER_INFO_RESPONSE
    });
    my $SUT = $self->_createSUT($ApiService);

    $SUT->UnityViewGoogleSmartShopping($Servlet);

    ok(
        WasCalled($ApiService, 'getUserInfo'),
        $SubTestName . ' - Verify that Google account email is being retrieved'
    );
    ok( !WasCalled($SUT, '_buildConnectOAuthURL'), $SubTestName . ' - Verify that OAuth connect URL was NOT built' );
}

#----------------------------------------------------------------------------------------
# Mockery
#----------------------------------------------------------------------------------------
sub _createSUT {
    my $self = shift;
    my ($ApiService) = @_;

    my $MockifySut = Test::Mockify::Sut->new('DE_EPAGES::GoogleSmartShopping::UI::Shop');
    $MockifySut->mock('_buildConnectOAuthURL')
        ->when(Object('DE_EPAGES::Presentation::Servlet::Permission'), Object('DE_EPAGES::Shop::API::Object::Shop'))
        ->thenReturn(CONNECT_OAUTH_URL);
    $MockifySut->mockImported('DE_EPAGES::GoogleSmartShopping::API::Tools', 'LogGoogleSmartShopping')
        ->whenAny()
        ->thenReturnUndef();
    if (defined($ApiService)) {
        $MockifySut->mockStatic('DE_EPAGES::GoogleSmartShopping::API::GoogleApiService::new')
            ->when()
            ->thenReturn($ApiService);
    }

    return $MockifySut->getMockObject();
}

sub _createGoogleSmartShoppingConfig {
    my $self = shift;
    my ($AccessToken, $RefreshToken) = @_;

    my $hAttributes = {
        AccessToken  => $AccessToken,
        RefreshToken => $RefreshToken
    };
    my $MockifyConfig = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::GoogleSmartShopping::API::Object::GoogleSmartShoppingConfig', $hAttributes
    );
    $MockifyConfig->addMockWithReturnValueAndParameterCheck('set', undef, [ 'hashref' ]);

    return $MockifyConfig->getMockObject();
}

sub _createGoogleApiService {
    my $self = shift;
    my ($hServiceResponse) = @_;

    my $Mockify = Test::Mockify->new('DE_EPAGES::GoogleSmartShopping::API::GoogleApiService');
    $Mockify->mock('getOAuthTokens')
        ->when(String())
        ->thenReturn($hServiceResponse->{'OAuthTokens'} // {});
    $Mockify->mock('getUserInfo')
        ->when(String())
        ->thenReturn($hServiceResponse->{'UserInfo'} // {});

    return $Mockify->getMockObject();
}

sub _createServlet {
    my $self = shift;
    my ($Shop, $ServletForm) = @_;

    my $MockifyServlet = DE_EPAGES::Test::Mock::MockifyServlet->new($Shop);
    $MockifyServlet->addMockWithReturnValue('form', $ServletForm);

    return $MockifyServlet->getMockObject();
}

sub _createShop {
    my $self = shift;
    my ($GoogleSmartShoppingConfig, $AlreadyExistingConfig) = @_;

    my $hAttributes = {
        GoogleSmartShoppingConfig => $GoogleSmartShoppingConfig
    };
    my $MockifyShop = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Shop::API::Object::Shop', $hAttributes);
    $MockifyShop->addMockWithReturnValueAndParameterCheck(
        'existsChild', $AlreadyExistingConfig, [ {'string' => 'GoogleSmartShoppingConfig'} ]
    );
    $MockifyShop->addMockWithReturnValueAndParameterCheck(
        'insertChild', $GoogleSmartShoppingConfig, [ {'string' => 'GoogleSmartShoppingConfig'}, 'hashref' ]
    );

    return $MockifyShop->getMockObject();
}

sub _createServletForm {
    my $self = shift;
    my ($hQueryParameters) = @_;

    my $Mockify = Test::Mockify->new( 'DE_EPAGES::Presentation::API::Form');
    $Mockify->mock('queryValues')
        ->when()
        ->thenReturn($hQueryParameters);

    return $Mockify->getMockObject();
}

__PACKAGE__->runScript();

