#========================================================================================
# package      DE_EPAGES::Object::API::Object::Site
# state        public
# base         DE_EPAGES::Object::API::Object::Object
# description  object interface for Site.
#========================================================================================
package DE_EPAGES::Object::API::Object::Site;
use base DE_EPAGES::Object::API::Object::Object;

use strict;
use DE_EPAGES::Core::API::Error qw( Error );
use DE_EPAGES::Core::API::File qw( DeleteDirectory ExistsDirectory RenameDirectory ConvertPathChars);
use DE_EPAGES::Core::API::Object::DateTimeFormatter;
use DE_EPAGES::Core::API::PerlTools qw (AddDuration GetCurrentDateTime GetLocalTimeZone);
use DE_EPAGES::Core::API::TestParameter qw ( TestArray TestInteger TestObject TestString );
use DE_EPAGES::Database::API::Connection qw (GetCurrentDBHandle);
use DE_EPAGES::Object::API::Domain qw (GetUrlInfo PoolDBEntries);
use DE_EPAGES::Object::API::Factory qw ( GetRootObjectID LoadClassByAlias LoadRootObject LoadObjectByGUID LoadObject);
use DE_EPAGES::Object::API::PageCache;
use DE_EPAGES::Object::API::Table::LastUpdate; # OnDeleteSite
use DE_EPAGES::Object::API::Table::Locale qw ( GetInfoLocale GetAllInfoLocales );
use DE_EPAGES::Object::API::Table::ObjectDelete; # OnDeleteSite
use DE_EPAGES::Object::API::Table::SiteLocaleMap qw ( DeleteSiteLocaleMap ExistsSiteLocaleMap InsertSiteLocaleMap GetPKeySiteLocaleMapsBySiteID );
use DE_EPAGES::Object::API::Table::SecondaryDomain qw ( GetPKeySecondaryDomainsBySiteID DeleteSecondaryDomain );
use DE_EPAGES::Object::DAL::LastUpdate qw( DAL_ClearDeletedObjects DAL_GetDeletedObjects DAL_GetCreates );
use DE_EPAGES::Trigger::API::Trigger qw ( TriggerHook );
use DE_EPAGES::WebInterfacePoolDB::API::ASPoolDB qw( RunOnASPoolDB );

#ERRORDEFINITION
sub ErrorDefinition {
    return {
        'LocaleDisabled' => 'Locale #LocaleID is currently disabled',
    };
}
#ERRORDEFINITION END

#========================================================================================
# function     className
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ClassName = $Site->className;
#----------------------------------------------------------------------------------------
# description  Returns the class name, used for DAL access.
#----------------------------------------------------------------------------------------
# return       $ClassName | class name | String
#========================================================================================
sub className  { 'Site' }

#========================================================================================
# function     dbPackage
# state        public
#----------------------------------------------------------------------------------------
# syntax       $DbPackage = $Site->dbPackage;
#----------------------------------------------------------------------------------------
# description  Returns the database package, used for DAL access.
#----------------------------------------------------------------------------------------
# return       $DbPackage | database package name | String
#========================================================================================
sub dbPackage { __PACKAGE__ }

#========================================================================================
# function     getSite
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Site = $Object->getSite;
#----------------------------------------------------------------------------------------
# description  Returns the site of an object. In this case returns the object itself.
#----------------------------------------------------------------------------------------
# return       $Site | site object | DE_EPAGES::Object::API::Object::Site
#========================================================================================
sub getSite {
    return shift;
}

#========================================================================================
# function     insert
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Site = DE_EPAGES::Object::API::Object::Site->insert($hInfo);
# example      my $Class = LoadClassByAlias('Site');
#               my $Site = $Class->insertObject($hInfo);
#----------------------------------------------------------------------------------------
# description  Insert this info as object, afterwards the hook 'OBJ_InsertSite' is triggered.
#----------------------------------------------------------------------------------------
# input        $hInfo | attributes of new object | ref.hash
# return       $Site | new object | object
# hook         OBJ_InsertSite | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>SiteID        - site id              - int
#               </ul>
#========================================================================================
sub insert {
    my $class = shift;
    my ($hInfo) = @_;

    my $self = $class->SUPER::insert($hInfo);
    $self->set({'Site'=>$self});

    TriggerHook('OBJ_InsertSite', {'Object' => $self, 'SiteID' => $self->id });
    delete $hInfo->{$_} foreach qw ( CurrencyID LocaleID IsStandalone DomainName CachedPagesString LoginRequired LoginCookie MailEncoding HasSSLCertificate WebServerScriptNamePart DefaultPageSize IsPageCacheUsed PageCacheLastInvalidation IsClosed MaxAgePagesString UseMobileSF ContractNumber ); ## no critic qw(ProhibitLongLines)

    $self->insertIntoPoolDB();
    return $self;
}

#========================================================================================
# function     insertIntoPoolDB
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Site->insertIntoPoolDB();
#----------------------------------------------------------------------------------------
# description  Insert this object in to the pooldb.
#========================================================================================
sub insertIntoPoolDB {
    my $self = shift;
    # create site in PoolDB

    my $GUID = $self->get('GUID');
    my $ObjectPath = $self->pathString;
    my $DatabaseName = GetCurrentDBHandle()->cacheKey;
    my $aPoolDBEntries = PoolDBEntries(GetUrlInfo($self));
    RunOnASPoolDB( sub {
        my $ASPoolDB = shift;
        $ASPoolDB->insertSite( $DatabaseName, $GUID, $ObjectPath );
        $ASPoolDB->setSiteRequests( $DatabaseName, $ObjectPath, $aPoolDBEntries );
    } );

    return;
}

#========================================================================================
# function     delete
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Site->delete;
#----------------------------------------------------------------------------------------
# description  Deletes this object, therefore the hook 'OBJ_DeleteSite' is triggered.
#----------------------------------------------------------------------------------------
# hook         OBJ_DeleteSite | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>SiteID        - site id              - int
#               </ul>
#========================================================================================
sub delete {
    my $self = shift;

    my $GUID = $self->get('GUID');

    TriggerHook('OBJ_DeleteSite', {'Object' => $self, 'SiteID' => $self->id });
    DeleteSiteLocaleMap($_) foreach @{GetPKeySiteLocaleMapsBySiteID($self->id)};
    DeleteSecondaryDomain($_) foreach @{GetPKeySecondaryDomainsBySiteID($self->id)};
    DE_EPAGES::Object::API::Table::LastUpdate::OnDeleteSite({ SiteID => $self->id });
    DE_EPAGES::Object::API::Table::ObjectDelete::OnDeleteSite({ SiteID => $self->id });

    $self->SUPER::delete;

    # delete site from PoolDB
    RunOnASPoolDB( sub { shift->deleteSite( $GUID ) } );
    return;
}

#========================================================================================
# function     set
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Site->set($hValues, $LanguageID);
#----------------------------------------------------------------------------------------
# description  This function triggers hooks 'OBJ_BeforeUpdateSite' and 'OBJ_AfterUpdateSite'.
#----------------------------------------------------------------------------------------
# input        $hValues | attribute name => value hash | ref.hash
# input        $LanguageID | (optional; required for localized attributes) language id | int
#----------------------------------------------------------------------------------------
# hook         OBJ_BeforeUpdateSite | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>SiteID        - site id              - int
#                 <li>Values        - new values for object - ref.hash
#                 <li>LanguageID    - values of this language will be updated - int
#               </ul>
# hook         OBJ_AfterUpdateSite | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>SiteID        - site id              - int
#                 <li>Values        - updated values       - ref.hash
#                 <li>LanguageID    - values updated of this language - int
#               </ul>
#========================================================================================
sub set {
    my $self = shift;
    my ($hValues, $LanguageID) = @_;

    TriggerHook('OBJ_BeforeUpdateSite', {'Object'=>$self, 'Values'=> $hValues, 'SiteID' => $self->id, 'LanguageID' => $LanguageID});
    # remember if the Alias or Parent has changed
    my $ChangedAlias = 0;
    my $ChangedParent = 0;

    # update children cache if parent or alias was changed
    if (exists $hValues->{'Parent'}) {
        my $Parent = $self->parent or die 'Can\'t change the parent of the root object';
        die 'Can\'t change the parent to me ' .  $Parent->id if $Parent->id == $self->id;
        $ChangedParent = $Parent->id != $hValues->{'Parent'}->id;
        delete $hValues->{'Parent'} unless $ChangedParent;
    } # else no parent change
    if (exists $hValues->{'Alias'}) {
        my $Alias = $self->alias;
        $ChangedAlias = $Alias ne $hValues->{'Alias'};
        delete $hValues->{'Alias'} unless $ChangedAlias;
    } # else no alias change

    # remember the old directory names, because they will be renamed
    my ($OldPublicPath, $OldPrivatePath, $OldStaticPath);
    if( $ChangedAlias || $ChangedParent ) {
        $OldPublicPath = $self->get('PublicPath');
        $OldPrivatePath = $self->get('PrivatePath');
        $OldStaticPath = $self->get('StaticPath');
    }

    $self->SUPER::set($hValues, $LanguageID);

    # update ObjectPath in PoolDB
    if ($ChangedParent || $ChangedAlias) {
        my $GUID = $self->get('GUID');
        my $ObjectPath = $self->pathString;
        RunOnASPoolDB( sub { shift->updateSite( $GUID, { ObjectPath => $ObjectPath }) } );
    }

    # update requests in PoolDB
    if (exists $hValues->{'DomainName'}
        || exists $hValues->{'SecondaryDomains'}
        || exists $hValues->{'HasSSLCertificate'}
        || exists $hValues->{'WebServerScriptNamePart'}
        # the following are System attributes
        || exists $hValues->{'WebServerDirectory'}
        || exists $hValues->{'WebServerPort'}
        || exists $hValues->{'SSLWebServerPort'}
    ) {
        $self->clearPageCache;
        $self->clearSavedPages;

        my $DatabaseName = GetCurrentDBHandle()->cacheKey;
        my @SiteRequests = ( [ $self->pathString, PoolDBEntries(GetUrlInfo($self)) ] );
        if ($self->class->existsAttribute('Shops')) {
            push @SiteRequests, map { [ $_->pathString, PoolDBEntries(GetUrlInfo($_)) ] } @{$self->get('Shops')};
        }
        if ($self->class->existsAttribute('Providers')) {
            push @SiteRequests, map { [ $_->pathString, PoolDBEntries(GetUrlInfo($_)) ] } @{$self->get('Providers')};
        }

        RunOnASPoolDB( sub {
            my $ASPoolDB = shift;
            $ASPoolDB->setSiteRequests( $DatabaseName, @$_ ) foreach @SiteRequests;
        });
    }

    # rename the object directories if the path has changed
    if( $ChangedParent || $ChangedAlias ) {
        RenameDirectory( $OldPublicPath, $self->get('PublicPath'), { create_directory => 1 } ) if ExistsDirectory( $OldPublicPath );
        RenameDirectory( $OldPrivatePath, $self->get('PrivatePath'), { create_directory => 1 } ) if ExistsDirectory( $OldPrivatePath );
        DeleteDirectory( $OldStaticPath , {'recursive' => 1, 'move' => 1} );
    }

    # clear page cache (if pagecache switched off)
    if(exists($hValues->{'IsPageCacheUsed'}) && !$hValues->{'IsPageCacheUsed'}) {
        $self->clearPageCache();
    }

    TriggerHook('OBJ_AfterUpdateSite',  {'Object'=>$self, 'Values'=> $hValues, 'SiteID' => $self->id, 'LanguageID' => $LanguageID});
    return;
}

#========================================================================================
# function     filePath
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Path = $Site->filePath;
#----------------------------------------------------------------------------------------
# description  Call <function DE_EPAGES::Core::API::File::ConvertPathChars> with site 'Alias' and parent of site.
#----------------------------------------------------------------------------------------
# return       $Path | relative file path | string
#========================================================================================
sub filePath {
    my $self = shift;

    return '' if $self->id == GetRootObjectID();
    return join '/', ConvertPathChars($self->parent->alias), ConvertPathChars($self->alias);
}

#========================================================================================
# function     isTemporaryClosed
# state        public
#----------------------------------------------------------------------------------------
# syntax       $IsClosed = $Site->isTemporaryClosed;
#----------------------------------------------------------------------------------------
# description  This method should be overwritten. Otherwise it returns always 0.
#               That means the site is open.
#               Returns true if the site is closed temporary. In this case all pages
#               should be delivered with the HTTP status 503 (Service Temporarily
#               Unavailable) to prevent search engines from caching the "shop closed"
#               message.
#----------------------------------------------------------------------------------------
# return       $IsClosed | true if site is closed | boolean
#========================================================================================
sub isTemporaryClosed {
    my $self = shift;

    return 0;
}

#========================================================================================
# function     existsSiteLocale
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Exists = $Site->existsSiteLocale($LocaleID);
# example      if( $Site->existsSiteLocale('en_GB') ) { ... }
#----------------------------------------------------------------------------------------
# description  Returns true if the site supports the given locale.
#----------------------------------------------------------------------------------------
# input        $LocaleID | locale id, e.g. 'en_GB' | string
# return       $Exists | true if the locale is supported by the site | boolean
#========================================================================================
sub existsSiteLocale {
    my $self = shift;
    my ($LocaleID) = @_;

    my $hLocale = GetInfoLocale($LocaleID);
    return 0 unless defined $hLocale;
    return $hLocale->{'IsUsed'};
}

#========================================================================================
# function     siteLocale
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Locale = $Site->siteLocale($LocaleID);
# example      $Site->siteLocale('en_GB')->roundMoney($amount, $CurrencyID);
#----------------------------------------------------------------------------------------
# description  Returns locale object with given locale identifier.
#----------------------------------------------------------------------------------------
# input        $LocaleID | locale id, e.g. 'en_GB' | string
# return       $Locale | supported locale of site | object
#========================================================================================
sub siteLocale {
    my $self = shift;
    my ($LocaleID) = @_;
    TestString( 'LocaleID', $LocaleID );

    # return an existsing SiteLocale object if possible
    my $Folder = LoadRootObject()->child('Locales');
    return $Folder->child($LocaleID) if $Folder->existsChild($LocaleID);

    # test that the locale is enabled
    my $hLocale = GetInfoLocale( $LocaleID );
    Error( 'LocaleDisabled', { 'LocaleID' => $LocaleID } ) unless $hLocale->{'IsUsed'};

    # create a new SiteLocale object
    return $Folder->insertChild('SiteLocale', {
        Alias => $LocaleID,
        LocaleID => $LocaleID
    });
}

#========================================================================================
# function     backofficeLocales
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aLocales = $Site->backofficeLocales;
#----------------------------------------------------------------------------------------
# description  Returns all locales that can be used in the back-office of the site.
#----------------------------------------------------------------------------------------
# return       $aLocales | list of SiteLocale objects | ref.array.object
#========================================================================================
sub backofficeLocales {
    my $self = shift;

    return [
        grep { $_->get('Language')->{'IsForBackoffice'} }
        map { $self->siteLocale( $_->{'LocaleID'} ) }
        grep { $_->{'IsUsed'} }
        @{GetAllInfoLocales()}
    ];
}

#========================================================================================
# function     addLocale
# state        public
#----------------------------------------------------------------------------------------
# syntax       $SiteLocale = $Site->addLocale($LocaleID);
# example      $SiteLocale = $Site->addLocale('de_DE');
#----------------------------------------------------------------------------------------
# description  Adds a storefront locale to the site.
#               Use $Site->get('SiteLocales') to get a list of all storefront locales.
#----------------------------------------------------------------------------------------
# input        $LocaleID | Locale identifier | string
# return       $SiteLocale | SiteLocale object | object
#========================================================================================
sub addLocale {
    my $self = shift;
    my ($LocaleID) = @_;
    TestString('LocaleID', $LocaleID, 5, 5);

    Error( 'LocaleDisabled', { 'LocaleID' => $LocaleID } ) unless $self->existsSiteLocale($LocaleID);

    my $count = scalar @{$self->get('SiteLocales')};
    InsertSiteLocaleMap({ 'SiteID' => $self->id, 'LocaleID' => $LocaleID, 'Position' => $count*10+10 }); ## no critic qw(ProhibitMagicNumbers)
    return $self->siteLocale( $LocaleID );
}

#========================================================================================
# function     existsLocale
# state        public
#----------------------------------------------------------------------------------------
# syntax       $exists = $Site->existsLocale($LocaleID);
# example      $exists = $Site->existsLocale('de_DE');
#----------------------------------------------------------------------------------------
# description  Returns true if the locale is $LocaleID is a storefront locale of the site.
#----------------------------------------------------------------------------------------
# input        $LocaleID | Locale identifier | string
# return       $exists | true if locale exists in site | boolean
#========================================================================================
sub existsLocale {
    my $self = shift;
    my ($LocaleID) = @_;

    return ExistsSiteLocaleMap( $self->id, $LocaleID );
}

#========================================================================================
# function     removeLocale
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Site->removeLocale($LocaleID);
# example      $Site->removeLocale('de_DE');
#----------------------------------------------------------------------------------------
# description  Removes a storefront locale from the site.
#----------------------------------------------------------------------------------------
# input        $LocaleID | Locale identifier | string
#========================================================================================
sub removeLocale {
    my $self = shift;
    my ($LocaleID) = @_;

    DeleteSiteLocaleMap( $self->id, $LocaleID ) if ExistsSiteLocaleMap( $self->id, $LocaleID );
    return;
}

#========================================================================================
# function     units
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aUnits = $Site->units;
#----------------------------------------------------------------------------------------
# description  Returns a list of all units of measurements that are available in the
#               site.
#----------------------------------------------------------------------------------------
# return       $aUnits | units of measurements (class UnitOfMeasurement) | ref.array.object
#========================================================================================
sub units {
    my $self = shift;

    # use site-specific units if the folder "Units" exists
    my $UnitFolder  = undef;
    $UnitFolder = $self->child('Units') if $self->existsChild('Units');
    $UnitFolder = LoadRootObject()->child('Units') unless defined $UnitFolder;

    my @Units;
    foreach my $Child (@{$UnitFolder->children}) {
        push @Units, $Child;
        push @Units, @{$Child->children};
    }
    return \@Units;
}

#========================================================================================
# function     deleteFiles
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Object->deleteFiles;
#----------------------------------------------------------------------------------------
# description  Deletes the directories PublicPath, PrivatePath.
#========================================================================================
sub deleteFiles {
    my $self = shift;

    DeleteDirectory( $self->get('PublicPath'),  { recursive => 1, move => 1 } );
    DeleteDirectory( $self->get('PrivatePath'), { recursive => 1, move => 1 } );
    $self->SUPER::deleteFiles();
    return;
}

#========================================================================================
# function     clearPageCache
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Shop->clearPageCache();
#----------------------------------------------------------------------------------------
# description  Clears the page cache of the complete site.
#========================================================================================
sub clearPageCache {
    my $self = shift;

    DE_EPAGES::Object::API::PageCache->new( Site => $self )->clearSiteCache;
    $self->set({'PageCacheLastInvalidation' => undef});
    return;
}

#========================================================================================
# function     invalidatePageCache
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Shop->invalidatePageCache();
#----------------------------------------------------------------------------------------
# description  invalidates the page cache, use <function clearPageCache> to clear it.
#========================================================================================
sub invalidatePageCache {
    my $self = shift;
    $self->set({'PageCacheLastInvalidation' => GetCurrentDateTime()}) if $self->get('IsPageCacheUsed');
    return;
}

#========================================================================================
# function     default
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hValues = DE_EPAGES::Object::API::Object::Site->default( $Parent, $hInfo );
# example      $hDefaults = $Class->defaultAttributes($Parent, $hInfo); # for insert a new object
#               $hDefaults = $Object->class->defaultAttributes($Object->parent, $hValues); # for an existing object
#----------------------------------------------------------------------------------------
# description  Returns default attribute values for new objects.
#               Used by <function DE_EPAGES::Object::Object::Class::defaultAttributes>
#----------------------------------------------------------------------------------------
# input        $Parent | parent object of the new object | object
# input        $hInfo | attribute name => value hash (passed to insert()) | ref.hash
# return       $hValues | hash with default attribute values (name => value) | ref.hash
#========================================================================================
sub default {
    my $class = shift;
    my ($Parent, $hInfo) = @_;
    my $LocaleID = $hInfo->{'LocaleID'};
    $LocaleID    = LoadRootObject()->get('LocaleID') unless defined $LocaleID;
    $LocaleID    = 'en_US' unless defined $LocaleID;
    my $hLocale = GetInfoLocale($LocaleID);
    return {
        %{$class->SUPER::default($Parent, $hInfo)},
        'IsStandalone' => 0,
        'LocaleID'     => $LocaleID,
        'CurrencyID'   => $hLocale->{'CurrencyID'},
        'LoginRequired' => 0,
        'LoginCookie' => 0,
        'HasSSLCertificate' => 0,
        'DefaultPageSize' => 10,
        'CachedPagesString' => 'Shop;1430;ContentObject;10080;Product;1430;Category;10080',
        'MaxAgePagesString' => 'Category;60;ContentObject;60;Product;60;Shop;60',
        'IsPageCacheUsed' => 0,
        'IsClosed' => 0,
        'UseMobileSF' => 1,
    };
}

#========================================================================================
# function     currentDate
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Date = DE_EPAGES::Object::API::Object::Site->currentDate();
#----------------------------------------------------------------------------------------
# description  Returns the current date in the time zone of the site with time set to
#               midnight (0:00:00).
#----------------------------------------------------------------------------------------
# return       $Date | DateTime object with the current Site date | object
#========================================================================================
sub currentDate {
    my $self = shift;

    # get the current system time
    my $TimeZone = LoadRootObject()->get('TimeZone') // GetLocalTimeZone();
    my $DateTime = GetCurrentDateTime($TimeZone)->clone;
    $DateTime->set_time_zone( 'floating');
    $DateTime->truncate( to => 'day' );

    return $DateTime;
}

#========================================================================================
# function     useMobileView
# state        public
#----------------------------------------------------------------------------------------
# syntax       $UseMobileView = DE_EPAGES::Object::API::Object::Site->useMobileView();
#----------------------------------------------------------------------------------------
# description  if the site allows mobile view
#----------------------------------------------------------------------------------------
# return       $UseMobileView | true if the mobile view should be used | boolean
#========================================================================================
sub useMobileView {
    my $self = shift;

    if ($self->get('UseMobileSF')) {
        return 1;
    }
    return 0;
}

sub _ValidateObject {
    my ($Object, $Site, $Class, $hFlags) = @_;

    if( $Class->isSubClassOf('Object') && $Site->id != $Object->getSite->id ) {
        unless( $hFlags->{'AllowSystemSite'} && $Object->getSite->id == GetRootObjectID() ) {
            Error('SiteMismatch', {
                'ObjectID' => $Object->id,
                'GUID' => $Object->guid,
                'SiteID' => $Site->id,
                'ObjectSiteID' => $Object->getSite->id,
                'SitePath' => $Site->pathString,
                'ObjectPath' => $Object->pathString,
            });
        }
    }

    unless( $Object->instanceOf($Class) ) {
        Error('ClassMismatch', {
            'ObjectID' => $Object->id,
            'GUID' => $Object->guid,
            'Class' => $Class->alias,
            'ObjectClass' => $Object->class->pathString,
            'ObjectPath' => $Object->pathString,
        })
    }
}

#========================================================================================
# function     loadObject
#----------------------------------------------------------------------------------------
# syntax       $Object = $Site->loadObject( $ObjectID, $Class );
# syntax       $Object = $Site->loadObject( $ObjectID, $ClassName );
# syntax       $Object = $Site->loadObject( $ObjectID, $Class, $hFlags );
# syntax       $Object = $Site->loadObject( $ObjectID, $ClassName, $hFlags );
# example      my $ProductID = $Servlet->form->value('ProductID');
#               my $Object = $Shop->loadObject( $ProductID, 'Product' );
# example      my $Unit = $Shop->loadObject( $UnitID, 'UnitOfMeasurement', { AllowSystemSite => 1 } );
#----------------------------------------------------------------------------------------
# description  Loads an object by object id.
#               Verifies that the object belongs to the given class and site.
#----------------------------------------------------------------------------------------
# input        $ObjectID | object id | int
# input        $Class | expected class of the object | DE_EPAGES::Object::API::Object::Class
# input        $ClassName | name of the expected class of the object | string
# input        $hFlags | (optional), <ul>
#                   <li>AllowSystemSite - the site of $Object may also be the System object - boolean</li>
#                   </ul> | ref. hash;
# return       $Object | object | DE_EPAGES::Object::API::Object::Object
#========================================================================================
sub loadObject {
    my $self = shift;
    my ($ObjectID, $Class, $hFlags) = @_;
    TestInteger('ObjectID', $ObjectID);

    if( ref $Class ) {
        TestObject('Class', $Class, 'DE_EPAGES::Object::API::Object::Class');
    }
    else {
        TestString('ClassName', $Class);
        $Class = LoadClassByAlias($Class);
    }
    $hFlags //= {};

    my $Object = $Class->loadObject($ObjectID);

    _ValidateObject($Object, $self, $Class, $hFlags);

    return $Object;
}

#========================================================================================
# function     loadObjects
#----------------------------------------------------------------------------------------
# syntax       $aObjects = $Site->loadObjects( $aObjectIDs, $Class );
# syntax       $aObjects = $Site->loadObjects( $aObjectIDs, $ClassName );
# syntax       $aObjects = $Site->loadObjects( $aObjectIDs, $Class, $hFlags );
# syntax       $aObjects = $Site->loadObjects( $aObjectIDs, $ClassName, $hFlags );
# example      my @ProductIDs = $Servlet->form->value('ProductID');
#               my $aProducts = $Shop->loadObjects( \@ProductIDs, 'Product' );
#----------------------------------------------------------------------------------------
# description  Loads a list of objects by object ids.
#               Verifies that the objects belongs to the given class and site.
#----------------------------------------------------------------------------------------
# input        $aObjectIDs | list of object ids | ref.array.int
# input        $Class | expected class of the objects | DE_EPAGES::Object::API::Object::Class
# input        $ClassName | name of the expected class of the objects | string
# input        $hFlags | (optional), see <function loadObject> | ref. hash;
# return       $aObjects | list of objects | ref.array.DE_EPAGES::Object::API::Object::Object
#========================================================================================
sub loadObjects {
    my $self = shift;
    my ($aObjectIDs, $Class, $hFlags) = @_;
    TestArray('aObjectIDs', $aObjectIDs);
    $Class = LoadClassByAlias($Class) unless ref $Class;

    return [ map { $self->loadObject($_, $Class, $hFlags) } @$aObjectIDs ];
}

#========================================================================================
# function     loadObjectByGuid
#----------------------------------------------------------------------------------------
# syntax       $Object = $Site->loadObjectByGuid( $GUID, $Class );
# syntax       $Object = $Site->loadObjectByGuid( $GUID, $ClassName );
# syntax       $Object = $Site->loadObjectByGuid( $GUID, $Class, $hFlags );
# syntax       $Object = $Site->loadObjectByGuid( $GUID, $ClassName, $hFlags );
# example      my $ProductGUID = $Servlet->form->value('ProductGUID');
#               my $Object = $Shop->loadObject( $ProductGUID, 'Product' );
# example      my $Unit = $Shop->loadObject( $UnitGUID, 'UnitOfMeasurement', { AllowSystemSite => 1 } );
#----------------------------------------------------------------------------------------
# description  Loads an object by GUID.
#               Verifies that the object belongs to the given class and site.
#----------------------------------------------------------------------------------------
# input        $GUID | object GUID | string
# input        $Class | expected class of the object | DE_EPAGES::Object::API::Object::Class
# input        $ClassName | name of the expected class of the object | string
# input        $hFlags | (optional), <ul>
#                   <li>AllowSystemSite - the site of $Object may also be the System object - boolean</li>
#                   </ul> | ref. hash;
# return       $Object | object | DE_EPAGES::Object::API::Object::Object
#========================================================================================
sub loadObjectByGuid {
    my $self = shift;
    my ($GUID, $Class, $hFlags) = @_;
    TestString('GUID', $GUID);

    if( ref $Class ) {
        TestObject('Class', $Class, 'DE_EPAGES::Object::API::Object::Class');
    }
    else {
        TestString('ClassName', $Class);
        $Class = LoadClassByAlias($Class);
    }
    $hFlags //= {};

    my $Object = LoadObjectByGUID($GUID);

    _ValidateObject($Object, $self, $Class, $hFlags);

    return $Object;
}

#========================================================================================
# function     loadObjectsByGuid
#----------------------------------------------------------------------------------------
# syntax       $aObjects = $Site->loadObjectsByGuid( $aGUIDs, $Class );
# syntax       $aObjects = $Site->loadObjectsByGuid( $aGUIDs, $ClassName );
# syntax       $aObjects = $Site->loadObjectsByGuid( $aGUIDs, $Class, $hFlags );
# syntax       $aObjects = $Site->loadObjectsByGuid( $aGUIDs, $ClassName, $hFlags );
# example      my @ProductGUID = $Servlet->form->value('ProductGUID');
#               my $aProducts = $Shop->loadObjectsByGuid( \@ProductGUIDs, 'Product' );
#----------------------------------------------------------------------------------------
# description  Loads a list of objects by GUID.
#               Verifies that the objects belongs to the given class and site.
#----------------------------------------------------------------------------------------
# input        $aGUIDs | list of GUIDs | ref.array.int
# input        $Class | expected class of the objects | DE_EPAGES::Object::API::Object::Class
# input        $ClassName | name of the expected class of the objects | string
# input        $hFlags | (optional), see <function loadObject> | ref. hash;
# return       $aObjects | list of objects | ref.array.DE_EPAGES::Object::API::Object::Object
#========================================================================================
sub loadObjectsByGuid {
    my $self = shift;
    my ($aObjectIDs, $Class, $hFlags) = @_;
    TestArray('aObjectIDs', $aObjectIDs);
    $Class = LoadClassByAlias($Class) unless ref $Class;

    return [ map { $self->loadObjectByGuid($_, $Class, $hFlags) } @$aObjectIDs ];
}

#========================================================================================
# function     getDeletedObjects
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ahDeletedObjects= $Site->getDeletedObjects( $ClassName, $DeletedAfter );
#----------------------------------------------------------------------------------------
# description  Returns information about objects in the site of a given type that have
#               been deleted after a given date, for example which products have been
#               deleted in a shop since last week.
#----------------------------------------------------------------------------------------
# input        $ClassName | class name, for example 'Product' | string
# input        $DeletedAfter | date and time | DateTime
# return       $Iterator | iterator over hash references with following keys:
#               <ul>
#                   <li>GUID          - GUID of the deleted object (primary key)- varchar(36)
#                   <li>PathFromSite  - Object Path relative to the object site. - varchar(1000)
#                   <li>DeletedOn     - Date and time of the deletion on an object - datetime
#               </ul>| iterator of hashes
#========================================================================================
sub getDeletedObjects {
    my $self = shift;
    my ($ClassName, $DeletedAfter) = @_;
    TestString('ClassName', $ClassName);
    TestObject('DeletedAfter', $DeletedAfter, 'DateTime');

    $DeletedAfter = AddDuration($DeletedAfter, 'seconds', -1); # subtract 1 second
    my $ClassID = LoadClassByAlias($ClassName)->id;
    return DAL_GetDeletedObjects($self->id, $ClassID, $DeletedAfter);
}

#========================================================================================
# function     clearDeletedObjects
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Site->clearDeletedObjects( $ClassName, $DeletedBefore );
#----------------------------------------------------------------------------------------
# description  Deletes information about objects in the site of a given type that have
#               been deleted before a given date, for example which products have been
#               deleted more than 2 weeks ago.
#----------------------------------------------------------------------------------------
# input        $ClassName | class name, for example 'Product' | string
# input        $DeletedBefore | date and time | DateTime
#========================================================================================
sub clearDeletedObjects {
    my $self = shift;
    my ($ClassName, $DeletedBefore) = @_;
    TestString('ClassName', $ClassName);
    TestObject('DeletedBefore', $DeletedBefore, 'DateTime');

    my $ClassID = LoadClassByAlias($ClassName)->id;
    return DAL_ClearDeletedObjects($self->id, $ClassID, $DeletedBefore);
}

#========================================================================================
# function     findUpdatedObjects
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Iterator = $Site->findUpdatedObjects( $Profile, $UpdatedAfter );
#----------------------------------------------------------------------------------------
# description  Returns information about objects in the site that have been updated
#               regarding a LastUpdateProfile after a given date
#----------------------------------------------------------------------------------------
# input        $Profile | profile object | DE_EPAGES::Object::API::Object::LastUpdateProfile
# input        $UpdatedAfter | date and time | DateTime
# return       $Iterator | iterator over hash references with following keys:
#               <ul>
#                   <li>ObjectID      - object id - int
#                   <li>LastUpdatedOn - Date and time of the update on an object - datetime
#               </ul>| iterator of hashes
#========================================================================================
sub findUpdatedObjects {
    my $self = shift;
    my ($Profile, $UpdatedAfter) = @_;
    TestObject('UpdatedAfter', $UpdatedAfter, 'DateTime');

    return $Profile->findLastUpdates( $self, $UpdatedAfter );
}

#========================================================================================
# function     getUpdatedObjects
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Iterator = $Site->getUpdatedObjects( $Profile, $UpdatedAfter );
#----------------------------------------------------------------------------------------
# description  Returns objects in the site that have been updated
#               regarding a LastUpdateProfile after a given date
#----------------------------------------------------------------------------------------
# input        $Profile | profile object | DE_EPAGES::Object::API::Object::LastUpdateProfile
# input        $UpdatedAfter | date and time | DateTime
# return       $Iterator | iterator over hash references with following keys:
#               <ul>
#                   <li>Object       - object - object
#                   <li>LastUpdatedOn - Date and time of the update on an object - datetime
#               </ul>| iterator of hashes
#========================================================================================
sub getUpdatedObjects {
    my $self = shift;
    my ($Profile, $UpdatedAfter) = @_;
    TestObject('UpdatedAfter', $UpdatedAfter, 'DateTime');

    return $Profile->getLastUpdates( $self, $UpdatedAfter );
}

#========================================================================================
# function     clearUpdatedObjects
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Site->clearUpdatedObjects( $Profile, $UpdatedBefore );
#----------------------------------------------------------------------------------------
# description  Deletes information about updated objects in the site of a given profile.
#----------------------------------------------------------------------------------------
# input        $Profile | profile object | DE_EPAGES::Object::API::Object::LastUpdateProfile
# input        $UpdatedBefore | date and time | DateTime
#========================================================================================
sub clearUpdatedObjects {
    my $self = shift;
    my ($Profile, $UpdatedBefore) = @_;
    TestObject('UpdatedBefore', $UpdatedBefore, 'DateTime');

    return $Profile->clearLastUpdates( $self, $UpdatedBefore );
}

#========================================================================================
# function     findCreatedObjects
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Iterator = $Site->findCreatedObjects( $Class, $CreatedAfter );
#----------------------------------------------------------------------------------------
# exmple       $Iterator = $Shop->findCreatedObjects( $Shop->get('ProductClass'), $Created );
#----------------------------------------------------------------------------------------
# description  Returns information about objects of given class/subclass in the site
#               that have been created after a given date
#----------------------------------------------------------------------------------------
# input        $Class | site related class | object
# input        $CreatedAfter | date and time | DateTime
# return       $Iterator | iterator over hash references with following keys:
#               <ul>
#                   <li>ObjectID      - object id - int
#                   <li>CreatedOn - Date and time of creation of an object - datetime
#               </ul>| iterator of hashes
#========================================================================================
sub findCreatedObjects {
    my $self = shift;
    my( $Class, $CreatedAfter ) = @_;
    TestObject('Class', $Class, 'DE_EPAGES::Object::API::Object::Class');
    TestObject('CreatedAfter', $CreatedAfter, 'DateTime');

    $CreatedAfter = AddDuration($CreatedAfter, 'seconds', -1); # subtract 1 second
    my @ClassIDs = (@{$Class->subClasses}, $Class->id);
    return DAL_GetCreates($self->id, \@ClassIDs, $CreatedAfter);
}

#========================================================================================
# function     getCreatedObjects
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Iterator = $Site->getCreatedObjects( $Class, $CreatedAfter );
#----------------------------------------------------------------------------------------
# exmple       $Iterator = $Shop->getCreatedObjects( $Shop->get('ProductClass'), $Created );
#----------------------------------------------------------------------------------------
# description  Returns objects of given class/subclass in the site
#               that have been created after a given date
#----------------------------------------------------------------------------------------
# input        $Class | site related class | object
# input        $CreatedAfter | date and time | DateTime
# return       $Iterator | iterator over hash references with following keys:
#               <ul>
#                   <li>Object    - object - object
#                   <li>CreatedOn - Date and time of creation of an object - datetime
#               </ul>| iterator of hashes
#========================================================================================
sub getCreatedObjects {
    my $self = shift;
    my( $Class, $CreatedAfter ) = @_;

    return DE_EPAGES::Core::API::Iterator->wrap(
        Source => $self->findCreatedObjects( $Class, $CreatedAfter ),
        Handle => sub {
            my $hCreated = $_[1];
            $hCreated->{Object} = LoadObject( delete $hCreated->{ObjectID} );
            return $hCreated;
        }
    );

}

#========================================================================================
# function     getSiteEmailAddress
# state        public
#----------------------------------------------------------------------------------------
# syntax       $EMailAddress = $Site->getSiteEmailAddress();
#----------------------------------------------------------------------------------------
# exmple       $EMailAddress = $Site->getSiteEmailAddress();
#----------------------------------------------------------------------------------------
# description  Placeholder function to be overriden by derived classes (e.g. Shop)
#----------------------------------------------------------------------------------------
# return       $EmailAddress | email | string
#========================================================================================
sub getSiteEmailAddress {
    my $self = shift;

    return undef;
}

1;
