 #========================================================================================
# package      DE_EPAGES::Order::API::Object::LineItem
# state        public
# base         DE_EPAGES::Shop::API::Object::ShopObject
# description  object interface for LineItem.
#========================================================================================
package DE_EPAGES::Order::API::Object::LineItem;
use base DE_EPAGES::Shop::API::Object::ShopObject;

use strict;
use DE_EPAGES::Trigger::API::Trigger qw ( TriggerHook );

#========================================================================================
# function     className
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ClassName = $LineItem->className;
#----------------------------------------------------------------------------------------
# description  Returns the class name, used for DAL access.
#----------------------------------------------------------------------------------------
# return       $ClassName | class name | String
#========================================================================================
sub className  { 'LineItem' }

#========================================================================================
# function     dbPackage
# state        public
#----------------------------------------------------------------------------------------
# syntax       $DbPackage = $LineItem->dbPackage;
#----------------------------------------------------------------------------------------
# description  Returns the database package, used for DAL access.
#----------------------------------------------------------------------------------------
# return       $DbPackage | database package name | String
#========================================================================================
sub dbPackage { __PACKAGE__ }

#========================================================================================
# function     insert
# state        private
#----------------------------------------------------------------------------------------
# syntax       $LineItem = DE_EPAGES::Order::API::Object::LineItem->insert($hInfo);
# example      my $Class = LoadClassByAlias('LineItem');
#               my $LineItem = $Class->insertObject($hInfo);
#----------------------------------------------------------------------------------------
# description  Insert this info as object, afterwards the hook 'OBJ_InsertLineItem' is triggered.
#----------------------------------------------------------------------------------------
# input        $Package | class package | string
# input        $hInfo | attributes of new object | ref.hash
# return       $LineItem | new object | object
# hook         OBJ_InsertLineItem | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>LineItemID    - line item id         - int
#               </ul>
#========================================================================================
sub insert {
    my ($Package, $hInfo) = @_;

    my $self = $Package->SUPER::insert($hInfo);

    TriggerHook('OBJ_InsertLineItem', {'Object' => $self, 'LineItemID' => $self->id });
    delete $hInfo->{$_} foreach qw ( LineItemContainer TaxClass Name LineItemPrice OriginalLineItemPrice TaxRate BasePrice Quantity Discount SKU TaxAmount IsChanged IsShippingRequired TaxClassDigital ); ## no critic qw(ProhibitLongLines)
    return $self;
}

#========================================================================================
# function     delete
# state        private
#----------------------------------------------------------------------------------------
# syntax       $LineItem->delete;
#----------------------------------------------------------------------------------------
# description  Deletes this object, therefore the hook 'OBJ_DeleteLineItem' is triggered.
#----------------------------------------------------------------------------------------
# hook         OBJ_DeleteLineItem | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>LineItemID    - line item id         - int
#               </ul>
#========================================================================================
sub delete {
    my $self = shift;

    TriggerHook('OBJ_DeleteLineItem', {'Object' => $self, 'LineItemID' => $self->id });
    $self->SUPER::delete;
    return;
}

#========================================================================================
# function     set
# state        private
#----------------------------------------------------------------------------------------
# syntax       $LineItem->set($hValues, $LanguageID);
#----------------------------------------------------------------------------------------
# description  This function triggers hooks 'OBJ_BeforeUpdateLineItem' and 'OBJ_AfterUpdateLineItem'.
#----------------------------------------------------------------------------------------
# hook         OBJ_BeforeUpdateLineItem | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>LineItemID    - line item id         - int
#                 <li>Values        - new values for object - ref.hash
#                 <li>LanguageID    - values of this language will be updated - int
#               </ul>
# hook         OBJ_AfterUpdateLineItem | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>LineItemID    - line item id         - int
#                 <li>Values        - updated values       - ref.hash
#                 <li>LanguageID    - values updated of this language - int
#               </ul>
#========================================================================================
sub set {
    my $self = shift;
    my ($hValues, $LanguageID) = @_;

    TriggerHook('OBJ_BeforeUpdateLineItem', {'Object'=>$self, 'Values'=> $hValues, 'LineItemID' => $self->id, 'LanguageID' => $LanguageID});
    $self->SUPER::set($hValues, $LanguageID);
    TriggerHook('OBJ_AfterUpdateLineItem',  {'Object'=>$self, 'Values'=> $hValues, 'LineItemID' => $self->id, 'LanguageID' => $LanguageID});
    return;
}

#========================================================================================
# function     importXML
# state        private
# syntax       $Object->importXML($ObjectPackage, $Handler, $hElement, $Class, $hAttributes);
# description  adds special handling for tax class. The attribute TaxClass must contain
#               the alias of a tax class in the current shop.
#========================================================================================
sub importXML {
    my ($ObjectPackage, $Handler, $hElement, $Class, $hAttributes) = @_;

    if (defined $hAttributes->{'TaxClass'}) {
        my $Parent = $Handler->topID('Object');
        my $Shop = $Parent->getSite();

        # map TaxClass name to TaxClass object
        my $TaxMatrix = $Shop->get('TaxMatrix');
        $hAttributes->{'TaxClass'} = $TaxMatrix->child($hAttributes->{'TaxClass'});

    } elsif(defined $hAttributes->{'TaxClassDigital'}) {
        my $Parent = $Handler->topID('Object');
        my $Shop = $Parent->getSite();

        # map TaxClassDigital name to TaxClassDigital object
        my $TMD = $Shop->get('TaxMatrixDigital');
        $hAttributes->{'TaxClassDigital'} = $TMD->child($hAttributes->{'TaxClassDigital'});
    }

    return $ObjectPackage->SUPER::importXML($Handler, $hElement, $Class, $hAttributes);
}

#========================================================================================
# function     exportXMLAttributes
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hAttributes = $Object->exportXMLAttributes( $Driver, $hAllAttributes );
#----------------------------------------------------------------------------------------
# description  Returns plain attributes of object to XML export driver. This function
#               removes bit attributes with value 0.
#----------------------------------------------------------------------------------------
# input        $Driver | xml export driver with support function addObject | object
# input        $hAllAttributes | all attributes of object,
#                                 see $Object-><function exportableAttributes> | ref.hash
# return       $hAttributes | plain attributes | ref.hash.string
#========================================================================================
sub exportXMLAttributes {
    my $self = shift;
    my ($Driver, $hAttributes) = @_;

    $hAttributes = $self->SUPER::exportXMLAttributes($Driver, $hAttributes);

    # map TaxClass and TaxClassDigital object to their respective alias
    if (exists $hAttributes->{'TaxClass'}) {
        my $TaxClass = $self->get('TaxClass');
        $hAttributes->{'TaxClass'} = defined $TaxClass ? $TaxClass->alias : undef;
    } elsif (exists $hAttributes->{'TaxClassDigital'} and defined $hAttributes->{'TaxClassDigital'}) {
        my $TCD = $self->get('TaxClassDigital');
        $hAttributes->{'TaxClassDigital'} = $TCD->alias;
    }
    return $hAttributes;
}

#========================================================================================
# function     getExtraExportableAttributes
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aAttributes = $Object->getExtraExportableAttributes();
#----------------------------------------------------------------------------------------
# description  Returns names of attributes of object for XML export of orders.
#----------------------------------------------------------------------------------------
# return       $aAttributes | plain attributes | ref.array.string
#========================================================================================
sub getExtraExportableAttributes { return []; }

#========================================================================================
# function     recalculate
# state        public
#----------------------------------------------------------------------------------------
# syntax       $LineItem->recalculate($Container, $hValues);
#----------------------------------------------------------------------------------------
# description  This standard recalculate function save the given values in database. This
#               method should be used for other more specific line items to recalculate
#               their prices and taxes.
#----------------------------------------------------------------------------------------
# input        $Container | line item container | object
# input        $hValues | values which will be changed | ref.hash
#========================================================================================
sub recalculate {
    my $self = shift;
    my ($Container, $hValues) = @_;

    $self->set($hValues);
    return;
}

#========================================================================================
# function     container
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Container = $LineItem->container;
#----------------------------------------------------------------------------------------
# description  Returns the container of this item (most parent).
#----------------------------------------------------------------------------------------
# return       $container | line item container | LineItemContainer
#========================================================================================
sub container {
    my $self = shift;
    return $self->parent->container;
}

#========================================================================================
# function     formatter
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Formatter = $LineItem->formatter;
# example      $self->formatter->round($BasePrice * $Discount / -100),
#----------------------------------------------------------------------------------------
# description  Returns the formatter (and rounder) of this line item.
#----------------------------------------------------------------------------------------
# return       $Formatter | line item container ($self->container->formatter) | MoneyFormatter
#========================================================================================
sub formatter {
    return shift->container->formatter;
}

#========================================================================================
# function     globalKeys
#----------------------------------------------------------------------------------------
# syntax       @Keys = __PACKAGE__->globalKeys;
# syntax       @Keys = $LineItem->globalKeys;
#----------------------------------------------------------------------------------------
# description  Returns list of keys. The lineitemprice depends on this, only keys
#               are returned which are saved at lineitemcontainer and basket or order. If
#               your line item has more external dependencies, overwrite this function.
#----------------------------------------------------------------------------------------
# return       @Keys | list of attributes names (TaxArea User Customer CurrencyID TaxModel) | array.string
#========================================================================================
sub globalKeys {
    return qw ( TaxArea User Customer CurrencyID TaxModel );
}

#========================================================================================
# function     clone
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Clone = $LineItem->clone( $hInfo );
# syntax       $Clone = $LineItem->clone( $hInfo, $Recursive );
#----------------------------------------------------------------------------------------
# description  Inserts a new LineItem into the database as copy from old LineItem.
#----------------------------------------------------------------------------------------
# input        $hInfo | hash with mandatory attribute values.
#                       <ul>
#                         <li>ClassID - class id - int</li>
#                         <li>Alias - alias - varchar(255)</li>
#                         <li>ParentID - parent objectid - int</li>
#                         <li>Inherit - (optional) inherit permissions from parent object - boolean</li>
#                       </ul> | hash ref
# input        $Recursive | clone child object too (optional, default is false) | boolean
# input        $CloneFiles | clone files associated with objects (optional, default is true) | boolean
# return       $Clone | the new object | object
#========================================================================================
sub clone {
    my $self = shift;
    my ($hInfo, $Recursive, $CloneFiles) = @_;
    $Recursive = 0 unless defined $Recursive;

    my $Clone = $self->SUPER::clone($hInfo, $Recursive, $CloneFiles);
    # correct the LineItemContainer
    $Clone->set({'LineItemContainer' => $Clone->get('Parent')}) ;
    return $Clone;
}

#========================================================================================
# function     template
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Name = $Object->template($Name, $ObjectPageType);
#----------------------------------------------------------------------------------------
# description  This function is used to overwrite the given template for the purpose that
#               each lineitem can defined there own template (visualisation).
#----------------------------------------------------------------------------------------
# input        $Name | include name | string
# input        $ObjectPageType | object related pagetype | DE_EPAGES::Presentation::API::Object::ObjectPageType
# return       $Name | include name | string
#========================================================================================
sub template {
    my $self = shift;
    my ($Name, $ObjectPageType) = @_;
    return $self->classTemplate($Name, $ObjectPageType);
}

#========================================================================================
# function     default
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hValues = DE_EPAGES::Order::API::Object::LineItem->default( $Parent, $hInfo );
# example      $hDefaults = $Class->defaultAttributes($Parent, $hInfo); # for insert a new object
#               $hDefaults = $Object->class->defaultAttributes($Object->parent, $hValues); # for an existing object
#----------------------------------------------------------------------------------------
# description  Returns default attribute values for new objects.
#               Used by <function DE_EPAGES::Object::Object::Class::defaultAttributes>
#----------------------------------------------------------------------------------------
# input        $Parent | parent object of the new object | object
# input        $hInfo | attribute name => value hash (passed to insert()) | ref.hash
# return       $hValues | hash with default attribute values (name => value) | ref.hash
#========================================================================================
sub default {
    my $class = shift;
    my ($Parent, $hInfo) = @_;

    return {
        %{$class->SUPER::default($Parent, $hInfo)},
        'LineItemContainer' => $Parent,
        'IsChanged' => 0,
        'IsShippingRequired' => 0,
    };
}

#========================================================================================
# function     grepAvailableShippingMethods
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aShippingMethods = $LineItem->grepAvailableShippingMethods($aShippingMethods);
#----------------------------------------------------------------------------------------
# description  Empty methode to allow products to reduce available shipping methods.
#----------------------------------------------------------------------------------------
# input        $aShippingMethods | shipping methods | ref.array.object
# return       $aShippingMethods | selected shipping methods | ref.array.object
#========================================================================================
sub grepAvailableShippingMethods {
    my $self = shift;
    my ($aShippingMethods) = @_;
    return $aShippingMethods;
}

#========================================================================================
# function     addMailParams
# state        public
#----------------------------------------------------------------------------------------
# syntax       $LineItem->addMailParams($hParams,$hRegionalParams);
# example      my $LineItems = $Order->get('LineItemContainer')->get('Positions');
#               $_->addMailParams(\%MailParams, \%RegionalParams) foreach @$LineItems;
#----------------------------------------------------------------------------------------
# description  allow to change email params for CustomerOrderConfirmation mail
#----------------------------------------------------------------------------------------
# input        $hParams | MailParams | ref.hash:
# input        $hRegionalParams | RegionalParams | ref.hash
#========================================================================================
sub addMailParams { }

#========================================================================================
# function     clearMail
# state        public
#----------------------------------------------------------------------------------------
# syntax       $LineItem->clearMail($hParams,$hRegionalParams);
# example      my $LineItems = $Order->get('LineItemContainer')->get('Positions');
#               $_->clearMail(\%MailParams, \%RegionalParams) foreach @$LineItems;
#----------------------------------------------------------------------------------------
# description  cleanup data from <function addMailParams>
#----------------------------------------------------------------------------------------
# input        $hParams | MailParams | ref.hash:
# input        $hRegionalParams | RegionalParams | ref.hash
#========================================================================================
sub clearMail { }

1;
