#========================================================================================
# package      DE_EPAGES::Order::DAL::SQL::SalesVolume
# state        private
# description  standard SQL database interface for oder sales volume
#========================================================================================
package DE_EPAGES::Order::DAL::SQL::SalesVolume;

use strict;

use DE_EPAGES::Core::API::PerlTools qw (GetLoopFromDoubleArray);
use DE_EPAGES::Shop::API::Constants qw ( TAX_MODEL_GROSS );

#========================================================================================
# function     GetSalesVolume
#----------------------------------------------------------------------------------------
# syntax       ($Sum, $Count) = $Package->GetSalesVolume($dbi,$ShopID,$FromDate,$ToDate,$CurrencID,$TaxModel);
#----------------------------------------------------------------------------------------
# description  return the sales volume for a given date range, currencyid and taxmodel
#----------------------------------------------------------------------------------------
# input        $dbh | database handle | object
# input        $ShopID | Shop ID | integer
# input        $FromDate | start date | datetime
# input        $ToDate | end date | datetime
# input        $CurrencyID | currency id | string
# input        $TaxModel | (optional default:TAX_MODEL_NET)  tax model | int
# input        $CustomerID | (optional, default: undef) customer id | string
# return       ($Sum, $Count) | lineitemcontainer.totalbeforetax or lineitemcontainer.grandtotal
#               of orders, count of orders | string
#========================================================================================
sub GetSalesVolume {
    my $self = shift;
    my ($dbi, $ShopID, $FromDate, $ToDate, $CurrencyID, $TaxModel, $CustomerID) = @_;

    my $Column = (defined $TaxModel and ($TaxModel == TAX_MODEL_GROSS)) ? 'lic.grandtotal' : 'lic.totalbeforetax';
    my $CustomerIDClause = defined $CustomerID ? 'AND o.customerid=' . $CustomerID : '';

    my $aResult;

    $dbi->atIsolationLevel(
        'READ UNCOMMITTED',
        sub {
            $aResult = $dbi->execute(<<END_SQL,[$ShopID, $FromDate, $ToDate, $CurrencyID]);
                SELECT SUM($Column),COUNT($Column) FROM customerorder o
                INNER JOIN object oo ON oo.objectid = o.orderid
                AND o.shopid=?
                $CustomerIDClause
                AND oo.creationdate >= ?
                AND oo.creationdate <= ?
                AND o.cancelledon is NULL
                INNER JOIN object p ON p.objectid = oo.parentid AND p.alias = 'Orders'
                INNER JOIN lineitemcontainer   lic ON lic.lineitemcontainerid = o.lineitemcontainerid
                AND lic.currencyid=?
END_SQL
        }
    );

    my $Sum = $aResult->[0]->[0] //0;
    my $Count = $aResult->[0]->[1] //0;

    return ($Sum, $Count);
}

#========================================================================================
# function     GetSalesVolumeOverAllCurrencies
#----------------------------------------------------------------------------------------
# syntax       ($aResult) = $Package->GetSalesVolumeOverAllCurrencies($dbi, $ShopID, $FromDate, $ToDate;
#----------------------------------------------------------------------------------------
# description  return the sales volume for a given date range over all currencies for a shop
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | object
# input        $ShopID | Shop ID | integer
# input        $FromDate | start date | datetime
# input        $ToDate | end date | datetime
# return       ($aResult) | query result | array
#========================================================================================
sub GetSalesVolumeOverAllCurrencies {
    my $self = shift;
    my ($dbi, $ShopID, $FromDate, $ToDate) = @_;

    my $aResult;

    $dbi->atIsolationLevel(
        'READ UNCOMMITTED',
        sub {
            $aResult = $dbi->execute(<<END_SQL,[$ShopID, $FromDate, $ToDate]);
                SELECT SUM(lic.totalbeforetax), COUNT(lic.totalbeforetax), lic.currencyid,  SUM(lic.grandtotal), COUNT(co.orderid)
                FROM customerorder co
                JOIN object o ON o.objectid = co.orderid
                JOIN object po ON po.objectid = o.parentid AND po.alias = 'Orders'
                JOIN lineitemcontainer lic ON lic.lineitemcontainerid = co.lineitemcontainerid
                WHERE co.shopid= ?
                AND o.creationdate >= ?
                AND o.creationdate <= ?
                AND co.cancelledon is NULL
                GROUP BY lic.currencyid;
END_SQL
        }
    );

    return $aResult;
}

#========================================================================================
# function     GetStoreSalesVolume
#----------------------------------------------------------------------------------------
# syntax       $rhResult = $Package->GetStoreSalesVolume($dbi,$FromDate,$ToDate);
#----------------------------------------------------------------------------------------
# description  return the sales volume for all shops for a given date range and a currencyid
#----------------------------------------------------------------------------------------
# input        $dbh | database handle | object
# input        $FromDate | start date | datetime
# input        $ToDate | end date | datetime
# return       $rhResult =  | with following keys:
#               <ul>
#                   <li>ShopID        - shop id              (part of primary key)- int</li>
#                   <li>CurrencyID    - alpha currency code (ISO 4217) (part of primary key)- char(3)</li>
#                   <li>Amount        - value                - net amount per shop and currency</li>
#                   <li>OrderCount    - value                - number of orders per shop and currency</li>
#               </ul>| reference to array of hashes
#========================================================================================
sub GetStoreSalesVolume {
    my $self = shift;
    my ($dbi, $FromDate, $ToDate) = @_;

    my $dbResult;

    $dbi->atIsolationLevel(
        'READ UNCOMMITTED',
        sub {
            $dbResult = $dbi->execute(<<END_SQL,[$FromDate, $ToDate]);
                SELECT o.shopid, lic.currencyid, SUM(lic.totalbeforetax) AS amount,COUNT(lic.lineitemcontainerid) as ordercount
                FROM customerorder o
                INNER JOIN object oo ON oo.objectid = o.orderid
                AND oo.creationdate >= ?
                AND oo.creationdate <= ?
                AND o.cancelledon is NULL
                INNER JOIN object p ON p.objectid = oo.parentid AND p.alias = 'Orders'
                INNER JOIN lineitemcontainer   lic ON lic.lineitemcontainerid = o.lineitemcontainerid
                GROUP BY o.shopid, lic.currencyid
                ORDER BY amount DESC
END_SQL
        }
    );

    return GetLoopFromDoubleArray($dbResult, [qw (ShopID CurrencyID Amount OrderCount)]);
}

1;
