#========================================================================================
# package      DE_EPAGES::Ebay::DAL::Categories
# state        private
# description  database interface for sales volume:
#========================================================================================
package DE_EPAGES::Order::DAL::SalesVolume;
use base qw (Exporter);

use strict;
use DE_EPAGES::Core::API::Log qw ( GetLog );
use DE_EPAGES::Database::API::Connection qw (  GetCurrentDBHandle );

our @EXPORT_OK = qw (
    DAL_GetSalesVolume
    DAL_GetSalesVolumeByTaxModel
    DAL_GetSalesVolumeOverAllCurrencies
    DAL_GetStoreSalesVolume
);

sub _CallDriver {
    my ($FunctionName, @Parameter) = @_;
    my $dbi = GetCurrentDBHandle();
    my $Package = 'DE_EPAGES::Order::DAL::'.$dbi->driverName.'::SalesVolume';
    DE_EPAGES::Core::API::PerlTools::UsePackage($Package);
    GetLog->debug("call $Package->$FunctionName()");
    return $Package->$FunctionName($dbi, @Parameter);
}

#========================================================================================
# function     DAL_GetSalesVolume
#----------------------------------------------------------------------------------------
# syntax       $SalesVolume = DAL_GetSalesVolume($ShopID,$FromDate,$ToDate,$CurrencyID);
#----------------------------------------------------------------------------------------
# description  Return the sales volume for orders by a time range
#----------------------------------------------------------------------------------------
# input        $ShopID | Shop ID | integer
# input        $FromDate | start date | datetime
# input        $ToDate | end date | datetime
# input        $CurrencyID | currency id | string
# return       $Summ | lineitemcontainer.totalbeforetax of orders | string
#========================================================================================
sub DAL_GetSalesVolume ($$$$) {
    my ($ShopID,$FromDate,$ToDate,$CurrencyID) = @_;

    return _CallDriver('GetSalesVolume',$ShopID,$FromDate,$ToDate,$CurrencyID);
}

#========================================================================================
# function     DAL_GetSalesVolumeByTaxModel
#----------------------------------------------------------------------------------------
# syntax       $SalesVolume = DAL_GetSalesVolumeByTaxModel($ShopID,$FromDate,$ToDate,$CurrencyID,$TaxModel);
#----------------------------------------------------------------------------------------
# description  Return the sales volume (gross or net) for orders by a time range
#----------------------------------------------------------------------------------------
# input        $ShopID | Shop ID | integer
# input        $FromDate | start date | datetime
# input        $ToDate | end date | datetime
# input        $CurrencyID | currency id | string
# input        $TaxModel | tax model | int
# input        $CustomerID | (optional, default: undef) customer id | string
# return       $Summ | lineitemcontainer.grandtotal or lineitemcontainer.totalbeforetax of orders | string
#========================================================================================
sub DAL_GetSalesVolumeByTaxModel ($$$$$;$) {
    my ($ShopID,$FromDate,$ToDate,$CurrencyID, $TaxModel, $CustomerID) = @_;

    return _CallDriver('GetSalesVolume',$ShopID,$FromDate,$ToDate,$CurrencyID,$TaxModel,$CustomerID);
}

#========================================================================================
# function     DAL_GetSalesVolumeOverAllCurrencies
#----------------------------------------------------------------------------------------
# syntax       $SalesVolume = DAL_GetSalesVolumeOverAllCurrencies($ShopID,$FromDate,$ToDate);
#----------------------------------------------------------------------------------------
# description  Return the sales volume for orders by a time range
#----------------------------------------------------------------------------------------
# input        $ShopID | Shop ID | integer
# input        $FromDate | start date | datetime
# input        $ToDate | end date | datetime
# return       $Sum | query result | array
#========================================================================================
sub DAL_GetSalesVolumeOverAllCurrencies ($$$) {
    my ($ShopID,$FromDate,$ToDate) = @_;

    return _CallDriver('GetSalesVolumeOverAllCurrencies',$ShopID,$FromDate,$ToDate);
}

#========================================================================================
# function     DAL_GetStoreSalesVolume
#----------------------------------------------------------------------------------------
# syntax       $rhResult = DAL_GetStoreSalesVolume($FromDate,$ToDate);
#----------------------------------------------------------------------------------------
# description  Return the sales volume for all shops for orders by a time range
#----------------------------------------------------------------------------------------
# input        $FromDate | start date | datetime
# input        $ToDate | end date | datetime
# return       $rhResult | with following keys:
#               <ul>
#                   <li>ShopID        - shop id              (part of primary key)- int</li>
#                   <li>CurrencyID    - alpha currency code (ISO 4217) (part of primary key)- char(3)</li>
#                   <li>Amount        - value                - net amount per shop and currency</li>
#                   <li>OrderCount    - value                - number of orders per shop and currency</li>
#               </ul>| reference to hash
#========================================================================================
sub DAL_GetStoreSalesVolume ($$) {
    my ($FromDate,$ToDate) = @_;

    return _CallDriver('GetStoreSalesVolume',$FromDate,$ToDate);
}

1;
