#========================================================================================
# package      DE_EPAGES::Order::Hooks::User
# state        public
# description  actions on changing the currency of a user
#========================================================================================
package DE_EPAGES::Order::Hooks::User;

use strict;
use DE_EPAGES::Core::API::Error qw (Error);
use DE_EPAGES::Core::API::Warning qw (BlockWarnings);
use DE_EPAGES::Object::API::Factory qw ( Transaction );
use DE_EPAGES::Order::API::LineItemContainer qw ( SavePayment SaveShipping );

#========================================================================================
# function     OnAfterChangeCurrency
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnAfterChangeCurrency( $hParams );
#----------------------------------------------------------------------------------------
# description  AfterChangeCurrency triggered to change the currency of the current
#               basket for a user
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>Servlet
#                   <li>User
#                   <li>NewCurrencyID
#                   <li>OldCurrencyID
#               </ul> | ref.hash.*
#========================================================================================
sub OnAfterChangeCurrency {
    my ($hParams) = @_;

    # update the basket currency when the user changed the currency directly or
    # inderectly by changing the locale
    my $Servlet = $hParams->{'Servlet'};
    my $User = $hParams->{'User'};
    my $CurrencyID = $hParams->{'NewCurrencyID'};

    # get the basket for the user
    foreach my $Basket (@{$User->get('Baskets')}) {
        my $LineItemContainer = $Basket->get('LineItemContainer');
        # do nothing without changed currency
        next if $CurrencyID eq $LineItemContainer->get('CurrencyID');

        BlockWarnings(sub {
            Transaction( sub {
                $LineItemContainer->set({ 'CurrencyID'=> $CurrencyID });
                $Basket->recalculate;
            });
        }, ['ProductNoPrice', 'ProductCanNotChangedAtBasket', 'CouponCurrencyMismatch',
            'ShippingMethodNoPrice', 'PaymentMethodNoPrice', 'ShippingOptionNoPrice']);

        # show the basket only if there are lineitems in the basket
        next unless scalar(@{$LineItemContainer->get('LineItems')});

        # show the Basket
        $Servlet->vars('ViewObjectID',$Basket->id );
        $Servlet->vars('ViewAction',undef );
        $Servlet->vars('ItemsChanged',1);
    }
    return;
}

#========================================================================================
# function     OnAfterLoginUser
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnAfterLoginUser( $hParams );
#----------------------------------------------------------------------------------------
# description  AfterLoginUser triggered to change the shippingmethod and/or paymentmethod
#               of the current basket after login user
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>Servlet
#               </ul> | ref.hash.*
#========================================================================================
sub OnAfterLoginUser {
    my ($hParams) = @_;
    my $Servlet = $hParams->{'Servlet'};

    my $CurrencyID = $Servlet->currencyID;
    my $User = $Servlet->user;
    foreach my $Basket (@{$User->get('Baskets')}) {
        my $LineItemContainer = $Basket->get('LineItemContainer');
        next unless defined $LineItemContainer;

        # EPG-11243: update the basket currency after logon
        my $CurrencyChanged = ($CurrencyID eq $LineItemContainer->get('CurrencyID')) ? 0 : 1;
        my $aWarnings = BlockWarnings( sub {
            Transaction( sub {
                $LineItemContainer->set({ 'CurrencyID'=> $CurrencyID }) if $CurrencyChanged;
                # check if shipping and payment method are valid for the region of logged-in user
                SaveShipping( $LineItemContainer );
                SavePayment( $LineItemContainer );
                $Basket->recalculate;
            });
        }, [qw(
            ShippingMethodChanged PaymentMethodChanged
            ProductNoPrice ProductCanNotChangedAtBasket CouponCurrencyMismatch
            ShippingMethodNoPrice PaymentMethodNoPrice ShippingOptionNoPrice NoShippingMethod NoPaymentMethod
        )]);
        if( scalar @$aWarnings ) {
            # UNITY-7062: prevent specific warnings from triggering errors for Unity checkout
            my $IsUnity = $Basket->getSite()->get('IsUnityRegistered');

            foreach my $warning ( @$aWarnings ) {
                my $Name = $warning->{'Name'};
                $Name = $warning->code unless defined $Name;

                # remap because we have a address change here
                $Name .= 'ByAddress' if $Name eq 'ShippingMethodChanged';
                $Name .= 'ByAddress' if $Name eq 'PaymentMethodChanged';

                unless ($IsUnity and
                        ($Name eq 'ShippingMethodChangedByAddress' or
                         $Name eq 'PaymentMethodChangedByAddress')) {
                    $Servlet->form->mapFormError( {'Form' => 'Save', 'Name' => $Name}, $warning );
                }
            }

            Error('INVALID_FORM', {
                'ViewVars'=> {
                    'FormError' => 0,
                    'ViewObjectID' => $Basket->id,
                    'ViewAction' => 'View',
                },
            }) unless $IsUnity;
        }
        # show the Basket
        if( $CurrencyChanged ) {
            $Servlet->vars('ViewObjectID',$Basket->id );
            $Servlet->vars('ViewAction',undef );
            $Servlet->vars('ItemsChanged',1);
        }
    }
    return;
}

#========================================================================================
# function     OnAfterChangeLocale
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnAfterChangeLocale( $hParams );
#----------------------------------------------------------------------------------------
# description  AfterChangeLocale triggered to change the locale of the current
#               basket for a user
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>Servlet
#                   <li>User
#                   <li>NewLocaleID
#                   <li>OldLocaleID
#               </ul> | ref.hash.*
#========================================================================================
sub OnAfterChangeLocale {
    my ($hParams) = @_;

    # update the basket locale when the user changed the locale
    my $User = $hParams->{'User'};
    my $LocaleID = $hParams->{'NewLocaleID'};

    # get the basket for the user
    foreach my $Basket (@{$User->get('Baskets')}) {
        my $LineItemContainer = $Basket->get('LineItemContainer');
        # do nothing without changed locale
        next if $LocaleID eq $LineItemContainer->get('LocaleID');
        $LineItemContainer->set({ 'LocaleID'=> $LocaleID });
    }
    return;
}

1;
