#========================================================================================
# package      DE_EPAGES::Order::t::unit::BasketPatch
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for DE_EPAGES::Order::API::BasketPatch
#========================================================================================
package DE_EPAGES::Order::t::unit::BasketPatch;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::Order::API::BasketPatch;
use DE_EPAGES::Test::Mock::Mockify qw (GetParametersFromMockifyCall WasCalled);
use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use DE_EPAGES::Test::Mock::EpagesObject;
use Test::Mockify::Matcher qw ( Any HashRef Number Object String );
use Test::Mockify::Sut;
use Test::More;

use strict;



sub test {
    my $self = shift;

    $self->test_addProductLineItem();
    $self->test_modifyProductLineItem();
    $self->test_modifyProductLineItemPrice();
    $self->test_removeProductLineItem();

    $self->test_addBillingAddress();
    $self->test_addShippingAddress();

    $self->test_removeBillingAddress();
    $self->test_removeShippingAddress();

    $self->test_patchLocale();

    $self->test_addShippingMethod();
    $self->test_addPaymentMethod();

    done_testing();
}



sub test_addProductLineItem {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Basket = $self->_createBasket($self->_createLineItemContainer());
    my $BasketPatch = $self->_createBasketPatch_addProductLineItem($Basket);

    my $hPatchOperation = {
        'op' => 'add',
        'path' => '/lineItemContainer/productLineItems',
        'value' => {
            'productId' => 'myProduct123',
            'quantity' => '1',
        },
    };

    $BasketPatch->patch($hPatchOperation);

    my $aProdudctAddToContainerCallParams = GetParametersFromMockifyCall($self->{'__objectsToCheck'}->{'Product'}, 'addToContainer');
    my $hExpected2ndAddToContainerParam = {
        'Quantity' => 1
    };
    is($aProdudctAddToContainerCallParams->[0]->guid(), 'line-item-container-guid', "$SubTestName - test if 1st addToContainer is correct line item container");
    is_deeply($aProdudctAddToContainerCallParams->[1], $hExpected2ndAddToContainerParam, "$SubTestName - test if 2nd addToContainer param is as expected.");

    my $LineItemContainer = $self->{'__objectsToCheck'}->{'LineItemContainer'};
    ok(WasCalled($LineItemContainer, 'recalculate'), "$SubTestName - test if recalculate on LineItemContainer was called.");
}



sub test_modifyProductLineItem {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Basket = $self->_createBasket($self->_createLineItemContainer());
    my $BasketPatch = $self->_createBasketPatch_modifyProductLineItem($Basket);

    my $hPatchOperation = {
        'op' => 'add',
        'path' => '/lineItemContainer/productLineItems/Line-Item-Product-GUID',
        'value' => {
            'quantity' => '11',
        }
    };

    $BasketPatch->patch($hPatchOperation);

    my $LineItemProduct = $self->{'__objectsToCheck'}->{'LineItemProduct_modifyProductLineItem'};
    is($LineItemProduct->get('Quantity'), 11, "$SubTestName - test if quantity was updated."); ## no critic (ProhibitMagicNumbers)

    my $LineItemContainer = $self->{'__objectsToCheck'}->{'LineItemContainer'};
    ok(WasCalled($LineItemContainer, 'recalculate'), "$SubTestName - test if recalculate on LineItemContainer was called.");
}


sub test_modifyProductLineItemPrice {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Basket = $self->_createBasket($self->_createLineItemContainer());
    my $BasketPatch = $self->_createBasketPatch_modifyProductLineItem($Basket);

    my $hPatchOperation = {
        'op' => 'add',
        'path' => '/lineItemContainer/productLineItems/Line-Item-Product-GUID/product-price',
        'value' => {
            'singleItemPrice' => '29.90',
        }
    };

    $BasketPatch->patch($hPatchOperation);

    my $LineItemProduct = $self->{'__objectsToCheck'}->{'LineItemProduct_modifyProductLineItem'};
    is($LineItemProduct->get('BasePrice'), 29.90, "$SubTestName - test if product price was updated."); ## no critic (ProhibitMagicNumbers)

    my $LineItemContainer = $self->{'__objectsToCheck'}->{'LineItemContainer'};
    ok(WasCalled($LineItemContainer, 'recalculate'), "$SubTestName - test if recalculate on LineItemContainer was called.");
}



sub test_removeProductLineItem {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Basket = $self->_createBasket($self->_createLineItemContainer());
    my $BasketPatch = $self->_createBasketPatch_removeProductLineItem($Basket);

    my $hPatchOperation = {
        'op' => 'remove',
        'path' => '/lineItemContainer/productLineItems/Line-Item-Product-GUID',
    };

    $BasketPatch->patch($hPatchOperation);

    my $LineItemProduct = $self->{'__objectsToCheck'}->{'LineItemProduct_removeProductLineItem'};
    ok(WasCalled($LineItemProduct, 'delete'), "$SubTestName - test if delete on LineItemProduct was called.");
    my $LineItemContainer = $self->{'__objectsToCheck'}->{'LineItemContainer'};
    ok(WasCalled($LineItemContainer, 'recalculate'), "$SubTestName - test if recalculate on LineItemContainer was called.");
}



sub test_addBillingAddress {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Basket = $self->_createBasket();
    my $BasketPatch = $self->_createBasketPatch_AddressPatch($Basket);

    my $hPatchOperation = {
        'op' => 'add',
        'path' => '/billingAddress',
        'value' => {
            'firstName' => 'Joe',
            'lastName' => 'Doe',
        },
    };

    $BasketPatch->patch($hPatchOperation);

    my $BasketAddress = $self->{'__objectsToCheck'}->{'BasketAddress'};
    my $setAddressCallParams = GetParametersFromMockifyCall($BasketAddress, 'setAddress');
    my $hExpectedAddressData = {
        'firstName' => 'Joe',
        'lastName' => 'Doe',
    };
    is_deeply($setAddressCallParams->[0], $hExpectedAddressData, "$SubTestName - test if correct address data was stored.");
    is($setAddressCallParams->[1], 'Billing', "$SubTestName - test address type was billing");
    ok(WasCalled($Basket, 'updateTaxArea'), "$SubTestName - test if updateTaxArea on Basket was called.");
    ok(WasCalled($Basket, 'recalculate'), "$SubTestName - test if recalculate on Basket was called.");
}



sub test_removeBillingAddress {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Basket = $self->_createBasket();
    my $BasketPatch = $self->_createBasketPatch_AddressPatch($Basket);

    my $hPatchOperation = {
        'op' => 'remove',
        'path' => '/billingAddress',
    };

    $BasketPatch->patch($hPatchOperation);

    my $BasketAddress = $self->{'__objectsToCheck'}->{'BasketAddress'};
    my $setAddressCallParams = GetParametersFromMockifyCall($BasketAddress, 'removeAddress');
    is($setAddressCallParams->[0], 'Billing', "$SubTestName - test address type was billing");
    ok(WasCalled($Basket, 'updateTaxArea'), "$SubTestName - test if updateTaxArea on Basket was called.");
    ok(WasCalled($Basket, 'recalculate'), "$SubTestName - test if recalculate on Basket was called.");
}



sub test_addShippingAddress {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Basket = $self->_createBasket();
    my $BasketPatch = $self->_createBasketPatch_AddressPatch($Basket);

    my $hPatchOperation = {
        'op' => 'add',
        'path' => '/shippingAddress',
        'value' => {
            'firstName' => 'Joe',
            'lastName' => 'Doe',
        },
    };

    $BasketPatch->patch($hPatchOperation);

    my $BasketAddress = $self->{'__objectsToCheck'}->{'BasketAddress'};
    my $setAddressCallParams = GetParametersFromMockifyCall($BasketAddress, 'setAddress');
    my $hExpectedAddressData = {
        'firstName' => 'Joe',
        'lastName' => 'Doe',
    };
    is_deeply($setAddressCallParams->[0], $hExpectedAddressData, "$SubTestName - test if correct address data was stored.");
    is($setAddressCallParams->[1], 'Shipping', "$SubTestName - test address type was shipping");
    ok(WasCalled($Basket, 'updateTaxArea'), "$SubTestName - test if updateTaxArea on Basket was called.");
    ok(WasCalled($Basket, 'recalculate'), "$SubTestName - test if recalculate on Basket was called.");
}



sub test_removeShippingAddress {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Basket = $self->_createBasket();
    my $BasketPatch = $self->_createBasketPatch_AddressPatch($Basket);

    my $hPatchOperation = {
        'op' => 'remove',
        'path' => '/shippingAddress',
    };

    $BasketPatch->patch($hPatchOperation);

    my $BasketAddress = $self->{'__objectsToCheck'}->{'BasketAddress'};
    my $setAddressCallParams = GetParametersFromMockifyCall($BasketAddress, 'removeAddress');
    is($setAddressCallParams->[0], 'Shipping', "$SubTestName - test address type was shipping");
    ok(WasCalled($Basket, 'updateTaxArea'), "$SubTestName - test if updateTaxArea on Basket was called.");
    ok(WasCalled($Basket, 'recalculate'), "$SubTestName - test if recalculate on Basket was called.");
}



sub test_patchLocale {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Basket = $self->_createBasket($self->_createLineItemContainer());
    my $User = $self->_createUser();
    $Basket->set({'User' => $User});

    my $BasketPatch = DE_EPAGES::Order::API::BasketPatch->new($Basket);

    my $hPatchOperation = {
        'op' => 'replace',
        'path' => '/locale',
        'value' => 'en_US',
    };

    $BasketPatch->patch($hPatchOperation);
    is($Basket->container()->get('LocaleID'), 'en_US', "$SubTestName - test if locale was set for basket");
    is($Basket->get('User')->get('LocaleID'), 'en_US', "$SubTestName - test if locale was set for user");
}


sub test_addShippingMethod {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Basket = $self->_createBasket($self->_createLineItemContainer());
    my $ExpectedShippingMethodAlias = 'NewShippingMethodAlias';
    my $BasketPatch = $self->_createBasketPatch_addShippingMethod($Basket, $ExpectedShippingMethodAlias);

    my $hPatchOperation = {
        'op' => 'add',
        'path' => '/lineItemContainer/lineItemShipping',
        'value' => {
            'id' => {
                'value' => 'myShipping123'
            },
            'priceAmount' => {
                'value' => 7.50
            }
        }
    };

    $BasketPatch->patch($hPatchOperation);

    my $LineItemContainer = $self->{'__objectsToCheck'}->{'LineItemContainer'};

    ok(WasCalled($BasketPatch, '_getValidatedShippingMethod'), "$SubTestName - test if _getValidatedShippingMethod on SUT was called.");
    ok(WasCalled($LineItemContainer, 'recalculate'), "$SubTestName - test if recalculate on LineItemContainer was called.");
    is($LineItemContainer->get('Shipping')->get('ShippingMethod')->alias(), $ExpectedShippingMethodAlias, "$SubTestName - test if ShippingMethod was updated correctly");
}

sub test_addPaymentMethod {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Basket = $self->_createBasket($self->_createLineItemContainer());
    my $BasketPatch = $self->_createBasketPatch_addPaymentMethod($Basket);

    my $hPatchOperation = {
        'op' => 'add',
        'path' => '/lineItemContainer/lineItemPayment',
        'value' => {
            'name' => 'Paid via Marketplace',
            'priceAmount' => 2.35
        }
    };
    $BasketPatch->patch($hPatchOperation);

    ok(WasCalled($BasketPatch, '_setPaymentMethod'), "$SubTestName - test that _setPaymentMethod patch method was called.");
    ok(WasCalled($BasketPatch, '_setPaymentMethodDiscount'), "$SubTestName - test that _setPaymentMethodDiscount method was called.");
}

### helper methods ###

sub _createBasketPatch_AddressPatch {
    my $self = shift;
    my ($Basket) = @_;
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
        'DE_EPAGES::Order::API::BasketPatch',
        [$Basket]
    );
    $Mockify->addMockWithReturnValueAndParameterCheck(
        '_createBasketAddress',
        $self->_createBasketAddress(),
        [{'object' => 'DE_EPAGES::Order::API::Object::Basket'}]
    );
    return $Mockify->getMockObject();
}


sub _createBasketPatch_addProductLineItem {
    my $self = shift;
    my ($Basket) = @_;

    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new( 'DE_EPAGES::Order::API::BasketPatch', [$Basket] );
    $Mockify->addMockWithReturnValueAndParameterCheck( '_loadShopObjectByGUID', $self->_createProduct(), ['object', {'string'=>'myProduct123'}, 'string'] );
    $Mockify->addMockWithReturnValueAndParameterCheck( '_ValidateStockLevelSettings', undef, [{'object' => 'DE_EPAGES::Product::API::Object::Product'}, {'string' => '1'}] );
    $Mockify->addMockWithReturnValueAndParameterCheck( '_ValidateAvailibilityVisibilitySettings', undef, [{'object' => 'DE_EPAGES::Product::API::Object::Product'}] );
    return $Mockify->getMockObject();
}


sub _createBasketPatch_addShippingMethod {
    my $self = shift;
    my ($Basket, $NewShippingMethodAlias) = @_;

    my $MockifySut = Test::Mockify::Sut->new( 'DE_EPAGES::Order::API::BasketPatch', [$Basket]);

    $MockifySut->mock( '_getValidatedShippingMethod')->when(String(), Object())->thenCall(sub{ $self->_createShippingMethod($NewShippingMethodAlias); });

    return $MockifySut->getMockObject();
}

sub _createBasketPatch_addPaymentMethod {
    my $self = shift;
    my ($Basket) = @_;

    my $MockifySut = Test::Mockify::Sut->new( 'DE_EPAGES::Order::API::BasketPatch', [$Basket]);

    $MockifySut->spy('_setPaymentMethod')
        ->when(HashRef());

    $MockifySut->mockImported('DE_EPAGES::Object::API::Factory', 'Transaction')
        ->when(Any())
        ->thenCall(sub{ return $_[0]();});

    $MockifySut->mockImported('DE_EPAGES::Core::API::Error', 'ExistsError')
        ->when()->thenReturn(0);

    $MockifySut->mock('_setPaymentMethodDiscount')
        ->when(Object('DE_EPAGES::Order::API::Object::LineItemContainer'), Number())
        ->thenReturnUndef();

    return $MockifySut->getMockObject();
}

sub _createBasketPatch_modifyProductLineItem {
    my $self = shift;
    my ($Basket) = @_;
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new( 'DE_EPAGES::Order::API::BasketPatch', [$Basket] );
    $Mockify->addMockWithReturnValueAndParameterCheck( '_loadShopObjectByGUID', $self->_createLineItemProduct_modifyProductLineItem(), ['object', {'string'=>'Line-Item-Product-GUID'}, 'string'] );
    $Mockify->addMockWithReturnValueAndParameterCheck( '_ValidateStockLevelSettings', undef, [{'object' => 'DE_EPAGES::Product::API::Object::Product'}, {'string' => '11'}] );
    $Mockify->addMockWithReturnValueAndParameterCheck( '_ValidateAvailibilityVisibilitySettings', undef, [{'object' => 'DE_EPAGES::Product::API::Object::Product'}] );
    return $Mockify->getMockObject();
}


sub _createBasketPatch_removeProductLineItem {
    my $self = shift;
    my ($Basket) = @_;
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
        'DE_EPAGES::Order::API::BasketPatch',
        [$Basket]
    );
    $Mockify->addMockWithReturnValueAndParameterCheck(
        '_loadShopObjectByGUID',
        $self->_createLineItemProduct_removeProductLineItem(),
        ['object', {'string'=>'Line-Item-Product-GUID'}, 'string']
    );
    return $Mockify->getMockObject();
}


sub _createBasket {
    my $self = shift;
    my ($LineItemContainer) = @_;

    my $hAtributes = {
        'GUID' => 'basket-guid-123',
        'Site' => $self->_createShop(),
        'LineItemContainer' => $LineItemContainer,
    };
    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::Order::API::Object::Basket',
            $hAtributes
    );
    $Mockify->addMockWithReturnValue('updateTaxArea', undef);
    $Mockify->addMockWithReturnValue('recalculate', undef);
    return $Mockify->getMockObject();
}


sub _createUser {
    my $self = shift;

    my $hAtributes = {
        'LocaleID' => '',
    };
    return DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::Permission::API::Object::User',
            $hAtributes
    )->getMockObject();
}


sub _createLineItemContainer {
    my $self = shift;

    my $hAttributes =  {
        'CurrencyID'      => 'EUR',
        'GUID'            => 'line-item-container-guid',
        'Parent'          => $self->_createBasket(),
        'Site'            => $self->_createShop(),
        'LineItems'       => undef,
        'Shipping'        => $self->_createLineItemShipping("OldShippingMethodAlias"),
        'Payment'         => $self->_createLineItemPayment(),
        'TaxMatrix'       => $self->_createTaxMatrix(),
        'PaymentDiscount' => undef
    };
    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Order::API::Object::LineItemContainer',
        $hAttributes
    );

    $Mockify->addMock('recalculate', sub {});
    $Mockify->addMock('insertLineItem', sub {});
    $Mockify->addMock('taxRate', sub {});
    $Mockify->addMock('nextAlias', sub {return 'NewLineItemShipping'});

    my $LineItemContainer = $Mockify->getMockObject();
    $self->{'__objectsToCheck'}->{'LineItemContainer'} = $LineItemContainer;

    return $LineItemContainer;
}


sub _createShop {
    my $self = shift;
    my $hAtributes = {
    };
    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Shop::API::Object::Shop',
        $hAtributes
    );
    my $Formatter = DE_EPAGES::Core::API::Object::NumberFormat->new(
            int_currencyid => 'EUR',
            decimal_digits => 2
    );
    $Mockify->addMockWithReturnValueAndParameterCheck('formatter', $Formatter, [ { 'string'=>'EUR' } ]);

    return $Mockify->getMockObject();
}


sub _createProduct {
    my $self = shift;
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new( 'DE_EPAGES::Product::API::Object::Product', [] );
    $Mockify->addMockWithReturnValueAndParameterCheck('addToContainer', undef, ['object', 'hashref']);
    my $Product = $Mockify->getMockObject();
    $self->{'__objectsToCheck'}->{'Product'} = $Product;
    return $Product;
}


sub _createLineItemProduct_modifyProductLineItem {
    my $self = shift;
    my $hAtributes = {
        'GUID' => 'line-item-guid',
        'Parent' => $self->{'__objectsToCheck'}->{'LineItemContainer'},
        'Product' => $self->_createProduct(),
    };
    my $MockedLineItemProduct = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::Product::API::Object::LineItemProduct',
            $hAtributes
    )->getMockObject();
    $self->{'__objectsToCheck'}->{'LineItemProduct_modifyProductLineItem'} = $MockedLineItemProduct;
    return $MockedLineItemProduct;
}


sub _createLineItemProduct_removeProductLineItem {
    my $self = shift;
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
        'DE_EPAGES::Product::API::Object::LineItemProduct',
        []
    );
    $Mockify->addMockWithReturnValue('delete', undef);
    $Mockify->addMockWithReturnValue('guid', 'line-item-guid');
    $Mockify->addMockWithReturnValue('parent', $self->{'__objectsToCheck'}->{'LineItemContainer'});
    my $LineItemProduct = $Mockify->getMockObject();
    $self->{'__objectsToCheck'}->{'LineItemProduct_removeProductLineItem'} = $LineItemProduct;
    return $LineItemProduct;
}


sub _createBasketAddress {
    my $self = shift;
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
        'DE_EPAGES::Order::API::BasketAddress',
        []
    );
    $Mockify->addMockWithReturnValueAndParameterCheck(
        'setAddress',
        undef,
        ['hashref', 'string']
    );
    $Mockify->addMockWithReturnValueAndParameterCheck(
        'removeAddress',
        undef,
        ['string']
    );
    my $BasketAddress = $Mockify->getMockObject();
    $self->{'__objectsToCheck'}->{'BasketAddress'} = $BasketAddress;
    return $BasketAddress;
}

sub _createLineItemShipping {
    my $self = shift;
    my ($ShippingMethodAlias) = @_;

    my $hAttributes = {
        'Alias' => 'LineItemShipping',
        'Quantity' => 1,
        'NameOrAlias' => 'LineItemShipping',
        'GUID' => 'Test GUID',
        'Name' => 'LineItemShipping',
        'ShippingMethod' =>  $self->_createShippingMethod($ShippingMethodAlias),
    };
    my $MockedLineItemShipping = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Order::API::Object::LineItemShipping',$hAttributes);
    return $MockedLineItemShipping;
}

sub _createLineItemPayment {
    my $self = shift;

    my $hAttributes = {};
    my $MockedLineItemPayment = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::Order::API::Object::LineItemPayment', $hAttributes
    );
    $MockedLineItemPayment->addMockWithReturnValue('delete', undef);

    return $MockedLineItemPayment->getMockObject();
}

sub _createShippingMethod {
    my $self = shift;
    my ($Alias) = @_;

    my $MockedObject =
      DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Payment::API::Attributes::ShippingMethod');

    $MockedObject->mock(
        'alias',
        sub {
            shift;
            return $Alias;
        }
    );

    return $MockedObject;
}

sub _createTaxMatrix {
    my $self = shift;

    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::Tax::API::Object::TaxMatrix', []);
    $Mockify->addMockWithReturnValue('defaultTaxClass', undef);

    return $Mockify->getMockObject();
}

__PACKAGE__->runScript();
