#========================================================================================
# package      DE_EPAGES::Saferpay2::API::InitializePaymentPageMapper
# state        public
#----------------------------------------------------------------------------------------
# description  Mapper for InitializePaymentPage request
#========================================================================================
package DE_EPAGES::Saferpay2::API::InitializePaymentPageMapper;

use DE_EPAGES::Saferpay2::API::Tools qw ( FormatFloatForSaferpay );

use strict;

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       new( );
#----------------------------------------------------------------------------------------
# description  constructor
#----------------------------------------------------------------------------------------
# return       $self | self | DE_EPAGES::Saferpay2::API::InitializePaymentPageMapper
#========================================================================================
sub new {
    my $class = shift;
    my ( $hData, $LineItemContainer) = @_;

    my $hContainerInfo = $LineItemContainer->get([ qw( CurrencyID LanguageID BasketOrOrder )]);
    my $Order = $hContainerInfo->{'BasketOrOrder'};
    my $self  = bless {
        hData             => $hData,
        LineItemContainer => $LineItemContainer,
        CurrencyID        => $hContainerInfo->{'CurrencyID'},
        LanguageID        => $hContainerInfo->{'LanguageID'},
        BillingAddress    => $Order->get('BillingAddress')
    }, $class;

    return $self;
}

#========================================================================================
# function     MapPayer
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->MapPayer();
#----------------------------------------------------------------------------------------
# description  Add fields required for klarna
#----------------------------------------------------------------------------------------
# return       $hData | hash with new fields| hash
#========================================================================================
sub MapAdditionalKlarnaFields {
    my $self = shift;

    $self->MapPayer();
    $self->MapOrderItems();
    $self->MapRiskFactors();

    return $self->{'hData'};
}

#========================================================================================
# function     MapPayer
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->MapPayer();
#----------------------------------------------------------------------------------------
# description  Add 'Order' with 'Items' into $self->{'hData'} hash
#----------------------------------------------------------------------------------------
# return       $hData | hash with new key 'Order' with 'Items'| hash
#========================================================================================
sub MapPayer {
    my $self = shift;

    $self->{'hData'}->{'Payer'} = {
        'BillingAddress' => $self->_MapAddress($self->{'BillingAddress'})
    };

    return $self->{'hData'};
}

#========================================================================================
# function     MapOrderItems
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hAddress = $self->MapOrderItems( $hData, $LineItemContainer );
#----------------------------------------------------------------------------------------
# description  Add 'Order' with 'Items' into hData hash
#----------------------------------------------------------------------------------------
# return       $hData | hash with new key 'Order' with 'Items'| hash
#========================================================================================
sub MapOrderItems {
    my $self = shift;

    $self->{'hData'}->{'Order'} = {
        'Items' => $self->_GetOrderItems($self->{'LineItemContainer'}),
    };

    return $self->{'hData'};
}

#========================================================================================
# function     MapRiskFactors
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hAddress = $self->MapRiskFactors( $hData, $LineItemContainer );
#----------------------------------------------------------------------------------------
# description  Add 'RiskFactors' with 'DeliveryType' into hData hash
#----------------------------------------------------------------------------------------
# return       $hData | hash with new key 'RiskFactors' | hash
#========================================================================================
sub MapRiskFactors {
    my $self = shift;

    $self->{'hData'}->{'RiskFactors'} = {
        'DeliveryType' => $self->_GetDeliveryType($self->{'LineItemContainer'})
    };
    return $self->{'hData'};
}

sub GetData {
    my $self = shift;
    return $self->{'hData'};
}

#========================================================================================
# function     _MapAddress
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hAddress = $self->_MapAddress( $BillingAddress );
#----------------------------------------------------------------------------------------
# description  Return a has with mapped address fields from an Address object
#----------------------------------------------------------------------------------------
# input        $Address | Billing address | DE_EPAGES::Address::API::Object::Address
# return       $hItem | formatted Address for use in a JSON structure, hash keys are:
#                   <ul><li>FirstName - First name - string</li>
#                       <li>LastName - Last name - string</li>
#                       <li>Email - Email address, e.g. 'buyer@epages.com'</li>
#                       <li>CountryCode - country code, e.g. 'de' - string</li>
#                   </ul> | hash
#========================================================================================
sub _MapAddress {
    my $self = shift;
    my ( $Address ) = @_;

    my $hInfo = $Address->get([qw( FirstName LastName EMail Country )]);
    return {
        'FirstName' => $hInfo->{'FirstName'},
        'LastName'=> $hInfo->{'LastName'},
        'Email' => $hInfo->{'EMail'},
        'CountryCode' => lc($hInfo->{'Country'}->{'Code2'}),
    };

}

#========================================================================================
# function     _GetOrderItems
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Items = $self->_GetOrderItems( $LineItemContainer )
#----------------------------------------------------------------------------------------
# description  Return a list of mapped order items from LineItemContainer.
#               The payment line item is skipped.
#----------------------------------------------------------------------------------------
# input        $LineItemContainer | LineItemContainer object  | Object
# return       $OrderItems | List of order items | ref.array
#========================================================================================
sub _GetOrderItems {
    my $self = shift;
    my ( $LineItemContainer ) = @_;

    my @OrderItems = ();
    foreach my $LineItem (@{$LineItemContainer->children()}) {
        next if $LineItem->instanceOf('LineItemPaymentSaferpay2');
        my $hItem = $self->_MapOrderItem($LineItem);
        push(@OrderItems, $hItem);
    }
    return \@OrderItems;
}

#========================================================================================
# function     _MapOrderItem
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hItem = $self->_MapOrderItem( $LineItem );
#----------------------------------------------------------------------------------------
# description  Map one Item of the order.
#               Use LineItemPrice and Quantity=1 when product type SHIPPINGFEE
#               Use TaxAmount and Quantity=1 when product type SALESTAX
#               Use BasePrice and Quantity for all other item types
#----------------------------------------------------------------------------------------
# input        $LineItem | LineItemProduct object  | object
# return       $hItem | formatted order item for use in a JSON structure, hash keys are:
#                   <ul><li>Type - item type e.g. 'PHYSICAL'- string</li>
#                       <li>Quantity - number products ordered, e.g. 1 - integer</li>
#                       <li>Name - product name, e.g. 'T-Shirt'</li>
#                       <li>UnitPrice - formatted base price of product - string</li>
#                   </ul> | hash
#========================================================================================
sub _MapOrderItem() {
    my $self = shift;
    my ( $LineItem ) = @_;

    my $hLineItemInfo = $LineItem->get([ qw( Quantity NameOrAlias BasePrice LineItemPrice TaxRate Discount )]);
    my $ItemType = $self->_GetOrderItemType( $LineItem );

    my $UnitPrice = $self->_calculateUnitPriceWithDiscount($hLineItemInfo);
    my $Quantity = $hLineItemInfo->{'Quantity'};

    # SALESTAX - the price is stored in TaxAmount
    if($ItemType eq 'SALESTAX' ) {
        $UnitPrice = $LineItem->get('TaxAmount');
        $Quantity = 1;
    }

    # SHIPPINGFEE - use LineItemPrice
    if($ItemType eq 'SHIPPINGFEE' ) {
        $UnitPrice = $hLineItemInfo->{'LineItemPrice'};
        $Quantity = 1;
    }

    # GIFTCARD, DISCOUNT - use LineItemPrice, must be positive
    if($ItemType ~~ ['GIFTCARD', 'DISCOUNT']) {
        $UnitPrice = abs($hLineItemInfo->{'LineItemPrice'});
        $Quantity = 1;
    }

    my $ItemName = $LineItem->get('NameOrAlias', $self->{'LanguageID'});
    my $TaxRate = $hLineItemInfo->{'TaxRate'} // 0;
    my $TaxAmount = $hLineItemInfo->{'TaxAmount'} // 0;
    my $DiscountAmount = $hLineItemInfo->{'Discount'} // 0;

    $DiscountAmount = FormatFloatForSaferpay($DiscountAmount, $self->{'CurrencyID'});

    my $hItem = {
        'Type'      => $ItemType,
        'Quantity'  => $Quantity,
        'Name'      => $ItemName,
        'UnitPrice' => FormatFloatForSaferpay($UnitPrice, $self->{'CurrencyID'}),
        'TaxRate'   => sprintf('%.0f', $TaxRate * 100 * 100),
        'TaxAmount' => FormatFloatForSaferpay($TaxAmount, $self->{'CurrencyID'}),
        'DiscountAmount' => $DiscountAmount,
    };
    return $hItem;
}

sub _calculateUnitPriceWithDiscount() {
    my $self = shift;
    my ($hLineItemInfo) = @_;
    my $Quantity = $hLineItemInfo->{'Quantity'} // 1;
    return ($hLineItemInfo->{'LineItemPrice'} + $hLineItemInfo->{'Discount'}) / $Quantity;
}

#========================================================================================
# function     _GetOrderItemType
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->_GetOrderItemType( $LineItem );
#----------------------------------------------------------------------------------------
# description  Check line item type return one of possible values:
#               DIGITAL, PHYSICAL, SERVICE, GIFTCARD, DISCOUNT,
#               SHIPPINGFEE, SALESTAX, SURCHARGE.
#----------------------------------------------------------------------------------------
# input        $LineItem | LineItemProduct object  | object
# return       Order item type | string
#========================================================================================
sub _GetOrderItemType {
    my $self = shift;
    my ( $LineItem ) = @_;


    if ($LineItem->instanceOf('LineItemShipping')) {
        return 'SHIPPINGFEE';
    }

    if($LineItem->instanceOf('LineItemProduct') && $LineItem->get('IsLineItemDownloadProduct')) {
        return 'DIGITAL';
    }

    if ($LineItem->instanceOf('LineItemCoupon')) {
        return 'GIFTCARD';
    }

    if ($LineItem->instanceOf('LineItemDiscount')) {
        return 'DISCOUNT';
    }

    if ($LineItem->instanceOf('LineItemTax')) {
        return 'SALESTAX';
    }

    return 'PHYSICAL';
}

#========================================================================================
# function     _GetDeliveryType
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->_GetDeliveryType( $LineItemContainer );
#----------------------------------------------------------------------------------------
# description  Get delivery type from LineItemContainer attributes.
#               When HasDownloadProducts is true return EMAIL otherwise HOMEDELIVERY.
#               Delivery type possible values: EMAIL, SHOP, HOMEDELIVERY, PICKUP, HQ.
#----------------------------------------------------------------------------------------
# input        $LineItem | LineItemProduct object  | object
# return       $DeliveryType | Saferpay delivery type | string
#========================================================================================
sub _GetDeliveryType {
    my $self = shift;
    my ( $LineItemContainer ) = @_;

    my $DeliveryType = $LineItemContainer->get('HasDownloadProducts') ? 'EMAIL' : 'HOMEDELIVERY';

    return $DeliveryType;
}

1;
