#========================================================================================
# package      DE_EPAGES::ShopMigration::API::CategoriesCommon
# state        public
#----------------------------------------------------------------------------------------
# description common functions for exporting/importing categories
#========================================================================================
package DE_EPAGES::ShopMigration::API::CategoriesCommon;
use base qw(Exporter);

use strict;
use feature 'switch';
use DE_EPAGES::Content::API::ShortUrl qw(GetShortUrlByClass); ##no critic (RequireCorrectImports)
use DE_EPAGES::Core::API::Error qw(Error ExistsError GetError);
use DE_EPAGES::Core::API::Log qw(LogDebug);
use DE_EPAGES::Core::API::PerlTools qw(ElementOf);
use DE_EPAGES::Core::API::Warning qw(Warning);
use DE_EPAGES::Database::API::Connection qw(RunOnStore GetCurrentDBHandle);
use DE_EPAGES::Database::API::Config qw(DatabaseConnectionData);
use DE_EPAGES::Object::API::Domain qw(GetUnitySubDomain);
use DE_EPAGES::Object::API::Factory qw(ExistsObjectByPath LoadObjectByPath LoadRootObject);
use DE_EPAGES::Object::API::Language qw(GetPKeyLanguageByCode GetCodeByLanguageID);
use DE_EPAGES::ShopMigration::API::Constants qw(
    CATEGORIES_ATTRIBUTES_FOR_EXPORT
    CATEGORIES_LOCALIZEDATTRIBUTES_FOR_EXPORT
    CATEGORIES_ATTRIBUTES_FOR_RESET
    CATEGORIES_LOCALIZEDATTRIBUTES_FOR_RESET
    CATEGORIES_ATTRIBUTES_FOR_IMPORT
    CATEGORIES_LOCALIZEDATTRIBUTES_FOR_IMPORT
);
use DE_EPAGES::ShopMigration::API::ShopCommonData qw(GetActiveShopLanguages);
use DE_EPAGES::ShopMigration::API::ShortURL qw(InsertRedirect GetAllUILocalesForShop);
use DE_EPAGES::Object::API::Table::Locale qw(GetLanguageByLocaleID);

our @EXPORT_OK = qw(
    GetCategories
    SetCategories
    GetCategoryData
    SetCategoryData
);
our $SKIP_UI = 0;

#ERRORDEFINITION
sub ErrorDefinition {
    return {
        'PARENT_CATEGORY_NOTFOUND' => 'Cannot create Category #Alias as parent #ParentPath was not found.',
        'UI_STORE_NOT_DEFINED'     => 'InstanceUnityUI is not defined in Database.d/#Store.conf.',
        'NAME_NOT_VALID'           => 'Category data for category "#Category" field ' .
            'LocalizedAttributes->#LanguageCode2->#Attribute contains invalid value: ' .
            '\'#Value\' must not include special characters only.',
    }
}
#ERRORDEFINITION END

#========================================================================================
# function     GetCategories
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $ahCategories = GetCategories($Shop);
#----------------------------------------------------------------------------------------
# description  gets all relevant data of category pages of a shop
#----------------------------------------------------------------------------------------
# input        $Shop | shop | object
#----------------------------------------------------------------------------------------
# return       $ahCategories | see GetCategoriesData() | reference to array of hashes
#========================================================================================
sub GetCategories {
    my ($Shop) = @_;

    my @Items;
    my $SubCategories = $Shop->get('Categories')->get('RecursiveSubCategories');
    foreach my $Category (@$SubCategories) {
        my $hInfo = GetCategoryData($Shop, $Category);
        push @Items, $hInfo if (keys %$hInfo);
    }
    return \@Items;
}

#========================================================================================
# function     GetCategoryData
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $hCategory = GetCategoryData($Shop, $Category);
#----------------------------------------------------------------------------------------
# description  Constructs a hash with the exportable fields for a single Category
#----------------------------------------------------------------------------------------
# input        $Shop | BaseShop (epages6) | object
# input        $Category | category | object
#----------------------------------------------------------------------------------------
# return       $Category | hash with following fields
#               <ul>
#                   <li>Attributes
#                       <ul>
#                           <li>Alias                        - string</li>
#                           <li>Position                     - float</li>
#                           <li>Parent                       - string</li>
#                           <li>IsVisible                    - boolean</li>
#                           <li>VisibleInNavigation          - 0 (reset only)</li>
#                           <li>GUID                         - string</li>
#                       </ul>
#                   </li>
#                   <li>LocalizedAttributes
#                       <ul>
#                           <li>LanguageCode2                    - en|de|..</li>
#                               <ul>
#                                   <li>Name                     - string</li>
#                                   <li>Title                    - undef (reset only)</li>
#                                   <li>Description              - undef (reset only)</li>
#                                   <li>Text                     - undef (reset only)</li>
#                                   <li>NavigationCaption        - undef (reset only)</li>
#                                   <li>Keywords                 - undef (reset only)</li>
#                                   <li>SEOQuality               - undef (reset only)</li>
#                                   <li>Text1                    - undef (reset only)</li>
#                                   <li>Text2                    - undef (reset only)</li>
#                                   <li>Text3                    - undef (reset only)</li>
#                               </ul>
#                           </li>
#                       </ul>
#                   </li>
#               </ul>| reference to hash
#========================================================================================
sub GetCategoryData {
    my ($Shop, $Category) = @_;

    my $IsPrepShop = $Shop->get('IsMigrationPrepShop');
    my $Attributes = [ @{CATEGORIES_ATTRIBUTES_FOR_EXPORT()} ];
    my $LocalizedAttributes = [ @{CATEGORIES_LOCALIZEDATTRIBUTES_FOR_EXPORT()} ];
    my $hValues = { Attributes => $Category->get($Attributes), LocalizedAttributes => {}, };
    $hValues->{Attributes}{Parent} = $Shop->relativePath($hValues->{Attributes}{Parent});

    if (!$IsPrepShop) {
        $hValues->{Attributes}{IsVisible} =
            $hValues->{Attributes}{IsVisible} && $hValues->{Attributes}{VisibleInNavigation};
    }
    $hValues->{Attributes}{VisibleInNavigation} = 0;
    foreach my $FieldName (@{CATEGORIES_ATTRIBUTES_FOR_RESET()}) {
        $hValues->{'Attributes'}{$FieldName} = undef;
    }
    my $ActiveLanguages = GetActiveShopLanguages($Shop);
    foreach my $hLanguage (@$ActiveLanguages) {
        my $AttrInfo = $Category->get($LocalizedAttributes, $hLanguage->{'LanguageID'});
        $hValues->{LocalizedAttributes}{$hLanguage->{'Code2'}} = $AttrInfo;
    }
    foreach my $Code2 (map {$_->{'Code2'}} @{LoadRootObject()->get('Languages')}) {
        foreach my $FieldName (@{CATEGORIES_LOCALIZEDATTRIBUTES_FOR_RESET()}) {
            $hValues->{'LocalizedAttributes'}{$Code2}{$FieldName} = undef;
        }
    }
    return $hValues;
}

#========================================================================================
# function     SetCategories
# state        public
#----------------------------------------------------------------------------------------
# syntax       SetCategories($Shop, $ahCategories);
#----------------------------------------------------------------------------------------
# description  sets attributes on Categories as specified in the $ahCategories
#----------------------------------------------------------------------------------------
# input        $Shop         | baseshop or prepshop | object
# input        $hCategory    | see GetCategories() | reference to array of hashes
#========================================================================================
sub SetCategories {
    my ($Shop, $ahCategories) = @_;

    foreach my $hCategory (@$ahCategories) {
        SetCategoryData($Shop, $hCategory);
    }
}

#========================================================================================
# function     SetCategoryData
# state        public
#----------------------------------------------------------------------------------------
# syntax       SetCategoryData($Shop, $hCategory);
#----------------------------------------------------------------------------------------
# description  sets attributes on a Category as specified in the hash $hCategory
#               if the shop is a base shop also updates the GUIDs in the ui.pages table
#----------------------------------------------------------------------------------------
# input        $Shop         | baseshop or prepshop | object
# input        $hCategory    | see GetCategories() | reference to array of hashes
# return       $Category     | updated/inserted category | object
#========================================================================================
sub SetCategoryData {
    my ($Shop, $hCategory) = @_;

    my $ParentPath = $hCategory->{Attributes}{Parent};
    my $Alias = $hCategory->{Attributes}{Alias};
    unless (ExistsObjectByPath($ParentPath, $Shop->id)) {
        Error('PARENT_CATEGORY_NOTFOUND', { Alias => $Alias, ParentPath => $ParentPath });
    }
    my $Parent = LoadObjectByPath($ParentPath, $Shop->id);
    delete $hCategory->{Attributes}{Parent};
    my $Category = $Parent->existsChild($Alias) ?
        $Parent->child($Alias) :
        $Parent->insertChild('Category', { Alias => $Alias });
    my $IsBaseShop = $Shop->get('IsMigrationPrepShop') ? 0 : 1;
    if ($IsBaseShop) {
        my $OldGUID = $hCategory->{Attributes}{GUID};
        my $NewGUID = $Category->get('GUID');
        eval {
            _UpdateGUIDinUI($Shop, $OldGUID, $NewGUID) unless $SKIP_UI;
        };
        if (ExistsError()) {
            my $Error = GetError();
            if ($Error->code() eq 'UI_STORE_NOT_DEFINED') {
                Warning($Error)
            } else {
                Error($Error);
            }
        }
    }

    delete $hCategory->{Attributes}{GUID};
    my $Attributes = [ @{CATEGORIES_ATTRIBUTES_FOR_IMPORT()} ];
    my $LocalizedAttributes = [ @{CATEGORIES_LOCALIZEDATTRIBUTES_FOR_IMPORT()} ];
    foreach my $Type (keys %$hCategory) {
        given ($Type) {
            when ('Attributes') {
                foreach my $Key (keys %{$hCategory->{Attributes}}) {
                    die "Invalid Element for Attributes: $Key" unless ElementOf($Key, @$Attributes);
                }
                $Category->set($hCategory->{Attributes});
            }
            when ('LocalizedAttributes') {
                foreach my $LanguageCode2 (keys %{$hCategory->{LocalizedAttributes}}) {
                    foreach my $Key (keys %{$hCategory->{LocalizedAttributes}{$LanguageCode2}}) {
                        die "Invalid Element for Attributes: $Key" unless ElementOf($Key, @$LocalizedAttributes);
                        next unless $Key eq 'Name';
                        my $Value = $hCategory->{LocalizedAttributes}{$LanguageCode2}{$Key};
                        if (defined $Value && $Value !~ /\w/) {
                            Error('NAME_NOT_VALID', {
                                'Category'      => $Alias,
                                'LanguageCode2' => $LanguageCode2,
                                'Attribute'     => 'Name',
                                'Value'         => $Value,
                            });
                        }
                    }
                    my $LanguageID = GetPKeyLanguageByCode($LanguageCode2);
                    $Category->set($hCategory->{LocalizedAttributes}{$LanguageCode2}, $LanguageID);
                }
            } default {
                die "Invalid Type: $Type";
            }
        }
    }

    # add redirect entries for each URI to epages-ui on final migration
    if ($IsBaseShop) {
        my $aLocales = GetAllUILocalesForShop($Shop);
        foreach my $LocaleID (@$aLocales) {
            last if $SKIP_UI;
            my $LanguageID = GetLanguageByLocaleID($LocaleID);
            my $FromSlug = $Category->get('URI', $LanguageID);
            if (defined $FromSlug) {
                my $ToSlug = '/' . GetCodeByLanguageID($LanguageID) . '/c/' . $Category->guid;
                InsertRedirect($Shop, $FromSlug, $ToSlug);
            }
        }
    }

    return $Category;
}



#========================================================================================
# function     _UpdateGUIDinUI
# state        private
#----------------------------------------------------------------------------------------
# syntax       _UpdateGUIDinUI($Shop, $OldGUID, $NewGUID)
#----------------------------------------------------------------------------------------
# description  updates the guid column in the pages table so in matches
#               the guid in the epages6 products table after transfer
#----------------------------------------------------------------------------------------
# input        $Shop         | baseshop or prepshop | object
# input        $OldGUID      | (old) guid in the prepshop | string
# input        $NewGUID      | (new) guid in the base shop | string
#========================================================================================
sub _UpdateGUIDinUI {
    my ($Shop, $OldGUID, $NewGUID) = @_;
    return unless defined $OldGUID;
    my $Store = LoadRootObject()->get('StoreName');
    my $Slug = GetUnitySubDomain($Shop->alias);
    my $UIStore = DatabaseConnectionData($Store)->{InstanceUnityUI};
    if (!defined $UIStore || !$UIStore) {
        Error('UI_STORE_NOT_DEFINED', { Store => $Store })
    } else {
        RunOnStore(
            Store => $UIStore,
            Sub   => sub {
                my $dbi = GetCurrentDBHandle();
                my $sth = $dbi->executeStatement(<<SQL,
UPDATE pages SET guid=?
WHERE guid=? AND
shopid IN (
    SELECT id FROM shops WHERE shopGateID = (
      SELECT id FROM shopGates WHERE slug=?
    )
  )
SQL
                    [ $NewGUID, $OldGUID, $Slug ]);
                LogDebug(sprintf("Updating in %s.pages [%s]: GUID %s -> GUID %s: affected %s rows\n",
                    $UIStore, $Slug, $OldGUID, $NewGUID, $sth->rows), 1);
            },
        );
    }
}

1;
