#========================================================================================
# package      DE_EPAGES::ShopMigration::API::MigrationReason
# state        public
#----------------------------------------------------------------------------------------
# description  This package provides the functions to check reasons which avoid a migration.
#========================================================================================
package DE_EPAGES::ShopMigration::API::MigrationReason;
use base Exporter;
use strict;
use 5.010; # Needed because we use state vars

our @EXPORT_OK = qw(
    GetMaximumFeatureValuesForShopMigration
    HasReasonDomainWithoutSSL
    HasReasonMaxCategoryDepth
    HasReasonCurrencies
    HasReasonCurrencyMatchesLocale
    HasReasonVariations
);

use DE_EPAGES::Database::API::Connection qw( GetCurrentDBHandle);
use DE_EPAGES::Object::API::Factory qw(LoadClassByAlias LoadObject);
use DE_EPAGES::Object::API::Table::Locale qw(GetInfoLocale);
use DE_EPAGES::ShopMigration::API::ShopMigrationConfig qw(ShopMigrationConfig);

#========================================================================================
# function     GetMaximumFeatureValuesForShopMigration
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hMaximumFeatureValues = GetMaximumFeatureValuesForShopMigration();
#----------------------------------------------------------------------------------------
# description  returns a hash with maximum feature values for shopmigration
#               read from shopmigration.conf
#               sections ShopMigrationCheck.Soft, ShopMigrationCheck.Hard
#----------------------------------------------------------------------------------------
# return       $hMaximumFeatureValues | hash with following fields
#               <ul>
#                   <li>FeatureAlias - string
#                       <ul>
#                           <li>Name                - string</li>
#                           <li>Type (Soft or Hard) - string</li>
#                           <li>Allowed             - integer</li>
#                       </ul>
#                   </li>
#               </ul>| reference to hash
#========================================================================================
sub GetMaximumFeatureValuesForShopMigration {
    my $MaximumFeatureValues = {};
    my $MigConfig = ShopMigrationConfig();
    @$MaximumFeatureValues{qw(Soft Hard)} = (
        $MigConfig->section('ShopMigrationCheck.Soft'),
        $MigConfig->section('ShopMigrationCheck.Hard')
    );
    foreach my $Type (qw(Soft Hard)) {
        foreach my $Name (keys %{$MaximumFeatureValues->{$Type}}) {
            @{$MaximumFeatureValues->{$Name}}{ qw(Name Type Allowed) } = (
                $Name, $Type, $MaximumFeatureValues->{$Type}{$Name},
            );
        }
    }
    delete $MaximumFeatureValues->{Soft};
    delete $MaximumFeatureValues->{Hard};
    return $MaximumFeatureValues;
}

#========================================================================================
# function     HasReasonDomainWithoutSSL
# state        public
#----------------------------------------------------------------------------------------
# syntax       $HasReasonDomainWithoutSSL = HasReasonDomainWithoutSSL($Shop, $hMaxFeatureValues);
#----------------------------------------------------------------------------------------
# description  returns true if shop has this reason so that migration is not possible
#----------------------------------------------------------------------------------------
# input        $Shop | BaseShop (epages6) | object
# input        $hMaxFeatureValues | return of sub GetMaximumFeatureValuesForShopMigration | hash ref
#----------------------------------------------------------------------------------------
# return       $HasReasonDomainWithoutSSL | result | boolean
#========================================================================================
sub HasReasonDomainWithoutSSL {
    my ($Shop, $hMaxFeatureValues) = @_;

    return unless exists $hMaxFeatureValues->{'DomainWithoutSSL'};
    my $AllowedDomainWithoutSSL = $hMaxFeatureValues->{'DomainWithoutSSL'}->{'Allowed'};

    my $sql = <<SQL;
SELECT count(1)
FROM site i
WHERE i.siteid = ? AND i.domainname is not null AND i.hassslcertificate = 0;
SQL
    my @Parameter = ($Shop->id);
    my $dbi = GetCurrentDBHandle();
    my $aaRows = $dbi->execute($sql, \@Parameter, [ 'Integer' ]);
    my $Count = $aaRows->[0][0];
    if ($Count > $AllowedDomainWithoutSSL) {
        return 1;
    }
    return;
}

#========================================================================================
# function     HasReasonMaxCategoryDepth
# state        public
#----------------------------------------------------------------------------------------
# syntax       $HasReasonMaxCategoryDepth = HasReasonMaxCategoryDepth($Shop, $hMaxFeatureValues);
#----------------------------------------------------------------------------------------
# description  returns true if shop has this reason so that migration is not possible
#----------------------------------------------------------------------------------------
# input        $Shop | BaseShop (epages6) | object
# input        $hMaxFeatureValues | return of sub GetMaximumFeatureValuesForShopMigration | hash ref
#----------------------------------------------------------------------------------------
# return       $HasReasonMaxCategoryDepth | result | boolean
#========================================================================================
sub HasReasonMaxCategoryDepth {
    my ($Shop, $hMaxFeatureValues) = @_;

    return unless exists $hMaxFeatureValues->{'MaxCategoryDepth'};
    my $AllowedCategoryDepth = $hMaxFeatureValues->{'MaxCategoryDepth'}->{'Allowed'};

    my $MaxDepthAttributeID = LoadClassByAlias('Category')->attribute('CategoryDepth')->id;
    my $sql = <<SQL;
select max(value)
from integerattribute ia
         inner join object o on o.objectid=ia.objectid and o.siteid=?
where attributeid=?;
SQL
    my @Parameter = ($Shop->id, $MaxDepthAttributeID);
    my $dbi = GetCurrentDBHandle();
    my $aaRows = $dbi->execute($sql, \@Parameter, [ 'Integer' ]);
    my $MaxDepth = $aaRows->[0][0];
    if ($MaxDepth > $AllowedCategoryDepth) {
        return 1;
    }
    return;
}

#========================================================================================
# function     HasReasonCurrencies
# state        public
#----------------------------------------------------------------------------------------
# syntax       $HasReasonCurrencies = HasReasonCurrencies($Shop, $hMaxFeatureValues);
#----------------------------------------------------------------------------------------
# description  returns true if shop has this reason so that migration is not possible
#----------------------------------------------------------------------------------------
# input        $Shop | BaseShop (epages6) | object
# input        $hMaxFeatureValues | return of sub GetMaximumFeatureValuesForShopMigration | hash ref
#----------------------------------------------------------------------------------------
# return       $HasReasonCurrencies | result | boolean
#========================================================================================
sub HasReasonCurrencies {
    my ($Shop, $hMaxFeatureValues) = @_;

    return unless exists $hMaxFeatureValues->{'Currencies'};
    my $AllowedVariations = $hMaxFeatureValues->{'Currencies'}->{'Allowed'};

    my $FeatureCurrentValue = $Shop->featureCurrentValue('Currencies');
    if ($FeatureCurrentValue > $AllowedVariations) {
        return 1;
    }
    return;
}

#========================================================================================
# function     HasReasonCurrencyMatchesLocale
# state        public
#----------------------------------------------------------------------------------------
# syntax       $HasReasonCurrencyMatchesLocale = HasReasonCurrencyMatchesLocale($Shop, $hMaxFeatureValues);
#----------------------------------------------------------------------------------------
# description  returns true if shop has this reason so that migration is not possible
#----------------------------------------------------------------------------------------
# input        $Shop | BaseShop (epages6) | object
# input        $hMaxFeatureValues | return of sub GetMaximumFeatureValuesForShopMigration | hash ref
#----------------------------------------------------------------------------------------
# return       $HasReasonCurrencyMatchesLocale | result | boolean
#========================================================================================
sub HasReasonCurrencyMatchesLocale {
    my ($Shop, $hMaxFeatureValues) = @_;

    return unless exists $hMaxFeatureValues->{'CurrencyMatchesLocale'};

    my $CurrencyID = $Shop->get('CurrencyID');
    my $LocaleID = $Shop->get('LocaleID');
    my $hLocale = GetInfoLocale($LocaleID);
    if ($hLocale->{CurrencyID} ne $CurrencyID) {
        return 1;
    }
    return;
}

#========================================================================================
# function     HasReasonVariations
# state        public
#----------------------------------------------------------------------------------------
# syntax       $HasReasonVariations = HasReasonVariations($Shop, $hMaxFeatureValues);
#----------------------------------------------------------------------------------------
# description  returns true if shop has this reason so that migration is not possible
#----------------------------------------------------------------------------------------
# input        $Shop | BaseShop (epages6) | object
# input        $hMaxFeatureValues | return of sub GetMaximumFeatureValuesForShopMigration | hash ref
#----------------------------------------------------------------------------------------
# return       $HasReasonVariations | result | boolean
#========================================================================================
sub HasReasonVariations {
    my ($Shop, $hMaxFeatureValues) = @_;

    return unless exists $hMaxFeatureValues->{'Variations'};
    my $AllowedVariations = $hMaxFeatureValues->{'Variations'}->{'Allowed'};

    my $FeatureCurrentValue = $Shop->featureCurrentValue('Variations');
    # only run the detail query if we have a shop with more then allowed Variations
    # but that does not mean it has them all selected/used for variation products in which case is *should* be fine
    my $FoundProductWithTooManyVariations = 0;
    if ($FeatureCurrentValue > $AllowedVariations) {
        my @Parameter = ($Shop->id);
        my $sql_masterproducts = <<SQL;
SELECT p.productid FROM product p
WHERE p.shopid=?
  AND p.superproductid IS NULL
  AND EXISTS ( SELECT sp.productid FROM product sp WHERE sp.superproductid=p.productid )
SQL
        my $dbi = GetCurrentDBHandle();
        my $aaMasterProducts = $dbi->execute($sql_masterproducts, \@Parameter, [ 'Integer' ]);
        foreach my $aRow (@$aaMasterProducts) {
            my $Product = LoadObject($aRow->[0]);
            my $Count = scalar @{$Product->get('VariationTypeValues')};
            if ($Count > $AllowedVariations) {
                $FoundProductWithTooManyVariations = 1;
                last;
            }
        }
    }
    return $FoundProductWithTooManyVariations ? 1 : 0;
}

1;
