#========================================================================================
# package      DE_EPAGES::ShopMigration::t::CategoryProductMapsExportImport
# base         DE_EPAGES::Test::API::UnitTest::Test
#----------------------------------------------------------------------------------------
# description  unit test for DE_EPAGES::ShopMigration::API::JSONLines::CategoryProductMapsExportImport
#========================================================================================
package DE_EPAGES::ShopMigration::t::CategoryProductMapsExportImport;
use base DE_EPAGES::Test::API::StoreUnitTest;

use strict;
use utf8;
use warnings;
use File::Spec::Functions;
use DE_EPAGES::Core::API::File qw(GetFileContent GetFileContentLines ExistsFile DeleteDirectory DeleteFile Text2Unix GetFileNames );
use DE_EPAGES::Object::API::Factory qw(LoadRootObject LoadClassByAlias  LoadObjectByPath);
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw(ImportObjects);
use DE_EPAGES::ShopMigration::API::JSONLines::CategoryProductMapsExportImport qw(ExportCategoryProductMapsJSON ImportCategoryProductMapsJSON);
use DE_EPAGES::ShopMigration::API::CategoryProductMapsCommon qw(GetCategoryProductMapIDs);
use DE_EPAGES::ShopMigration::API::MigrationShop qw(CreatePrepShop ExistsPrepShop SetShopFeatureValues );
use File::Temp qw(tempdir);
use Test::More;
use Test::Differences qw(context_diff eq_or_diff);

my $TmpDir = tempdir();

if ($ENV{SKIP_IN_PR_TESTS}) {
    plan skip_all => 'Skipping it because SKIP_IN_PR_TESTS == 1';
}

sub test {
    context_diff();
    testExportWithCategoryProductMapsExportImportBase2Prep();
    testImportWithCategoryProductMapsExportImportBase2Prep();
    done_testing();
}

my %TestData = (
    TestShop      => undef,
    TestShopAlias => 'TestShop',
    ImportPath    => '/Test/Shops/TestShop',
);
use constant FILENAME => 'categoryproductmaps.json';
use constant FILE_DIR => [ qw(ShopData.d export TestShop) ];
use constant FILE_LINES => 9;
use constant IMPORT_CATEGORYPRODUCTMAPS_PREP => 9;

sub testExportWithCategoryProductMapsExportImportBase2Prep {
    my $Shop = $TestData{TestShop};

    note('test CategoryProductMapsExportDriver (Base2Prep)');
    isnt($Shop, undef, 'Testshop exists');
    is(scalar @{$Shop->get('Languages')}, 1, 'Testshop has now one Language');
    is(scalar @{$Shop->get('SiteLocales')}, 1, 'Testshop has one SiteLocale');
    ok(!$Shop->get('IsMigrationPrepShop'), 'Testshop has IsMigrationPrepShop=0');

    my $Product = $Shop->folder('Products')->child('BundleProd01');
    my $Category = $Shop->get('Categories');
    my $CategoryProductMapClass = LoadClassByAlias('CategoryProductMap');
    my $aMaps = $CategoryProductMapClass->getObjects('ByCategoryIDProductID', [ $Category->id, $Product->id ]);
    is(scalar @$aMaps, 1, 'CategoryProductMap for BundleProduct exists');

    my $Exporter = ExportCategoryProductMapsJSON($Shop, $TmpDir);
    ok(ExistsFile($Exporter->fileName), 'export file exists: ' . $Exporter->fileName);

    is(scalar @{GetFileContentLines($Exporter->fileName, { encoding => 'utf8' })}, FILE_LINES, 'file lines count');
    ok($Exporter->fileName =~ /categoryproductmaps.json/, 'export file name matches categoryproductmaps.json');

    my $sFileContent = GetFileContent($Exporter->fileName, { encoding => 'utf8' });
    my $sExpected = GetFileContent(catfile(@{FILE_DIR()}, 'base2prep_' . FILENAME), { encoding => 'utf8' });
    eq_or_diff($sFileContent, $sExpected, 'file content matches');

    DeleteFile($Exporter->fileName());
    ok(!ExistsFile($Exporter->fileName()), 'export file cleaned up');
}

sub testImportWithCategoryProductMapsExportImportBase2Prep {
    my $aExpectedValues = [
        {
            'Category'         => 'Categories',
            'Product'          => 'Products/ho_40407',
            'Position'         => '10',
            'PositionCategory' => '0',
        },
        {
            'Category'         => 'Categories',
            'Product'          => 'Products/lt_0401107001',
            'Position'         => '20',
            'PositionCategory' => '0',
        },
        {
            'Category'         => 'Categories',
            'Product'          => 'Products/eg_1000111010',
            'Position'         => '30',
            'PositionCategory' => '0',
        },
        {
            'Category'         => 'Categories/Shoes',
            'Product'          => 'Products/md_49412090',
            'Position'         => '10',
            'PositionCategory' => '0',
        },
        {
            'Category'         => 'Categories/Tents/IndividualTents',
            'Product'          => 'Products/md_49412090/SubProducts/md_49412090105',
            'Position'         => '10',
            'PositionCategory' => '0',
        },
        {
            'Category'         => 'Categories/Tents/IndividualTents',
            'Product'          => 'Products/md_49412090/SubProducts/md_49412090100',
            'Position'         => '10',
            'PositionCategory' => '0',
        },
        {
            'Category'         => 'Categories/Tents/FamilyTents',
            'Product'          => 'Products/hm_0100401001',
            'Position'         => '10',
            'PositionCategory' => '0',
        },
        {
            'Category'         => 'Categories/Tents/FamilyTents',
            'Product'          => 'Products/lt_0401107001',
            'Position'         => '20',
            'PositionCategory' => '0',
        },
        {
            'Category'         => 'Categories/Tents/FamilyTents',
            'Product'          => 'Products/cg_0100504001',
            'Position'         => '30',
            'PositionCategory' => '0',
        },
    ];

    my $PrepShop = $TestData{PrepShop};
    my $DirName = catdir('ShopData.d', 'import');
    note('test CategoryProductMapsImportHandler (Base2Prep)');
    isnt($PrepShop, undef, 'Prepshop exists');

    my $CurrentCategoryProductMapIDs = GetCategoryProductMapIDs($PrepShop);
    is(scalar @$CurrentCategoryProductMapIDs, 0, 'no category product maps on PrepShop');

    my $Importer = ImportCategoryProductMapsJSON($PrepShop, $DirName);
    $CurrentCategoryProductMapIDs = GetCategoryProductMapIDs($PrepShop);
    is(scalar @$CurrentCategoryProductMapIDs, IMPORT_CATEGORYPRODUCTMAPS_PREP, 'imported category product maps count');
    ok($Importer->fileName() =~ /base2prep_${\FILENAME}/, 'ImportFileName was constructed properly');

    my $CategoryProductMapClass = LoadClassByAlias('CategoryProductMap');
    foreach my $hEntry (@$aExpectedValues) {
        my $Category = LoadObjectByPath($hEntry->{'Category'}, $PrepShop->id);
        my $Product = LoadObjectByPath($hEntry->{'Product'}, $PrepShop->id);
        # exists category product map?
        my $aMaps = $CategoryProductMapClass->getObjects('ByCategoryIDProductID', [ $Category->id, $Product->id ]);
        is(scalar @$aMaps, 1, sprintf('CategoryProductMap %s - %s exists', $hEntry->{'Category'}, $hEntry->{'Product'}));
        my $CategoryProductMap = scalar @$aMaps ? $aMaps->[0] : undef;
        my $hInfoCategoryProductMap = $CategoryProductMap->get([ qw(Position PositionCategory) ]);
        foreach my $key (keys %$hInfoCategoryProductMap) {
            is($hInfoCategoryProductMap->{$key}, $hEntry->{$key}, "attribute $key matches.");
        }
    }
}

# We need to create a fake ShopTypeRef UnityDE + ECom5000 in case they don't exists
# otherwise the Mapper won't work
sub init {
    my $self = shift;

    $self->SUPER::init;
    my $ShopAlias = $TestData{TestShopAlias};
    my $System = LoadRootObject();
    my $Shop;
    my $TestFolder = $System->folder('Test')->folder('Shops');
    my $ShopTypeRefFolder = $System->folder('ShopTypeRefs');
    my $Class = LoadClassByAlias('ShopTypeRef');
    $Class->insertObject({ Alias => 'UnityDE', 'Parent' => $ShopTypeRefFolder })
        unless ($ShopTypeRefFolder->existsChild('UnityDE'));
    $Class->insertObject({ Alias => 'ECom5000', 'Parent' => $ShopTypeRefFolder })
        unless ($ShopTypeRefFolder->existsChild('ECom5000'));
    if ($TestFolder->existsChild($ShopAlias)) {
        $Shop = $TestFolder->child($ShopAlias);
    } else {
        note(sprintf('Creating %s', $ShopAlias));
        $Shop = $TestFolder->insertChild('Shop', {
            Alias                   => $ShopAlias,
            ShopTypeRef             => $ShopTypeRefFolder->child('ECom5000'),
            WebServerScriptNamePart => $ShopAlias,
        });
        SetShopFeatureValues($Shop, [
            { Alias => 'Regions', MaxValue => '2' },
            { Alias => 'Languages', MaxValue => '15' },
            { Alias => 'Pages', MaxValue => '100' },
            { Alias => 'Subcategories', MaxValue => '100' },
            { Alias => 'Products', MaxValue => '100' },
            { Alias => 'ProductMaxSubProducts', MaxValue => '1000' },
            { Alias => 'ProductTypes', MaxValue => '100' },
            { Alias => 'ProductTypeAttributes', MaxValue => '100' },
            { Alias => 'Variations', MaxValue => '100' },
            { Alias => 'VariationAttributeValues', MaxValue => '100' },
        ]);
    }
    note('Creating PrepShop') if (!ExistsPrepShop($Shop));
    $TestData{TestShop} = $Shop;
    $TestData{PrepShop} = ExistsPrepShop($Shop) ? $Shop->get('MigrationPrepShop') : CreatePrepShop($Shop);
    foreach my $JSONFile (@{GetFileNames('ShopData.d', { 'recursive' => 1 })}) {
        Text2Unix($JSONFile) if ($^O eq 'linux');
    }
    ImportObjects('ImportFiles/CategoryProductMapsBase.xml', $TestData{TestShop}->pathString());
    ImportObjects('ImportFiles/CategoryProductMapsPrep.xml', $TestData{PrepShop}->pathString());
    return;
}

sub cleanup {
    my $self = shift;

    my $ShopAlias = $TestData{TestShopAlias};
    my $TestFolder = LoadRootObject()->folder('Test')->folder('Shops');
    if ($TestFolder->existsChild($ShopAlias)) {
        my $Shop = $TestFolder->child($ShopAlias);
        my $PrepShop = $Shop->get('MigrationPrepShop');
        if (defined $PrepShop) {
            note('Cleaning up PrepShop');
            $PrepShop->delete()
        }
        note(sprintf('Cleaning up %s', $ShopAlias));
        $Shop->delete();
        LoadRootObject()->folder('Test')->delete();
    }
    DeleteDirectory($TmpDir, { 'recursive' => 1 });
    return;
}

__PACKAGE__->runScript();

1;
